@@ -28,6 +28,7 @@
 
 #include <KDE/KApplication>
 
+#include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusContext>
 
 class Akonadi__ControlAdaptor;
@@ -53,6 +54,26 @@ class Session;
  * - configuration interface
  * - problem reporting
  *
+ * Akonadi Server supports several ways to launch agents and resources:
+ * - As a separate application (@see AKONADI_AGENT_MAIN)
+ * - As a thread in the AgentServer
+ * - As a separate process, using the akonadi_agent_launcher
+ *
+ * The idea is this, the agent or resource is written as a plugin instead of an
+ * executable (@see AgentFactory). In the AgentServer case, the AgentServer
+ * looks up the plugin and launches the agent in a separate thread. In the
+ * launcher case, a new akonadi_agent_launcher process is started for each
+ * agent or resource instance.
+ *
+ * When making an Agent or Resource suitable for running in the AgentServer some
+ * extra caution is needed. Because multiple instances of several kinds of agents
+ * run in the same process, one cannot blindly use global objects like KGlobal.
+ * For this reasons several methods where added to avoid problems in this context,
+ * most notably AgentBase::config() and AgentBase::componentData(). Additionally,
+ * one cannot use QDBusConnection::sessionBus() with dbus < 1.4, because of a
+ * multithreading bug in libdbus. Instead one should use
+ * DBusConnectionPool::threadConnection() which works around this problem.
+ *
  * @author Till Adam <adam@kde.org>, Volker Krause <vkrause@kde.org>
  */
 class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
@@ -80,6 +101,10 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
      * @note In the multiple inheritance approach the init() method automatically
      *       registers itself as the observer.
      *
+     * @note Do not call the base implementation of reimplemented virtual methods!
+     *       The default implementation disconnected themselves from the Akonadi::ChangeRecorder
+     *       to enable internal optimizations for unused notifications.
+     *
      * Example for stand-alone observer:
      * @code
      * class ExampleAgent : public AgentBase
@@ -343,6 +368,7 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
      */
     virtual QString progressMessage() const;
 
+  public Q_SLOTS:
     /**
      * This method is called whenever the agent shall show its configuration dialog
      * to the user. It will be automatically called when the agent is started for
@@ -355,6 +381,7 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
      */
     virtual void configure( WId windowId );
 
+  public:
     /**
      * This method returns the windows id, which should be used for dialogs.
      */
@@ -406,6 +433,16 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
      */
     QString agentName() const;
 
+    /**
+     * Returns the component data object for this agent instance.
+     * In case of stand-alone agents this is identical to KGlobal::mainComponent().
+     * In case of in-process agents there is one component data object
+     * per agent instance thread.
+     * This method provides valid results even when called in the agent ctor.
+     * @since 4.6
+     */
+    static KComponentData componentData();
+
   Q_SIGNALS:
     /**
      * This signal is emitted whenever the name of the agent has changed.
@@ -445,6 +482,14 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
      */
     void error( const QString& message );
 
+    /**
+     * This signal should be emitted whenever the status of the agent has been changed.
+     * @param status The object that describes the status change.
+     *
+     * @since 4.6
+     */
+    void advancedStatus( const QVariantMap &status );
+
     /**
      * Emitted when another application has remotely asked the agent to abort
      * its current operation.
@@ -518,6 +563,11 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
      */
     ChangeRecorder* changeRecorder() const;
 
+    /**
+     * Returns the config object for this Agent.
+     */
+    KSharedConfigPtr config();
+
     /**
      * Marks the current change as processes and replays the next change if change
      * recording is enabled (noop otherwise). This method is called
@@ -550,6 +600,7 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
     //@cond PRIVATE
     AgentBasePrivate *d_ptr;
     explicit AgentBase( AgentBasePrivate* d, const QString &id );
+    friend class ObserverV2;
     //@endcond
 
     /**
@@ -579,6 +630,7 @@ class AKONADI_EXPORT AgentBase : public QObject, protected QDBusContext
     Q_PRIVATE_SLOT( d_func(), void slotWarning( const QString& ) )
     Q_PRIVATE_SLOT( d_func(), void slotError( const QString& ) )
     Q_PRIVATE_SLOT( d_func(), void slotNetworkStatusChange( Solid::Networking::Status ) )
+    Q_PRIVATE_SLOT( d_func(), void slotResumedFromSuspend() )
 
     //@endcond
 };
