@@ -23,6 +23,9 @@
 #include "collectionrightsattribute_p.h"
 #include "entitydisplayattribute.h"
 #include "entityhiddenattribute.h"
+#include "indexpolicyattribute.h"
+#include "persistentsearchattribute.h"
+#include "entitydeletedattribute.h"
 
 #include <KGlobal>
 
@@ -30,6 +33,9 @@
 
 using namespace Akonadi;
 
+namespace Akonadi {
+namespace Internal {
+
 /**
  * @internal
  */
@@ -54,15 +60,6 @@ class DefaultAttribute : public Attribute
     QByteArray mType, mValue;
 };
 
-/**
- * @internal
- */
-class AttributeFactory::Private
-{
-  public:
-    QHash<QByteArray, Attribute*> attributes;
-};
-
 /**
  * @internal
  */
@@ -80,12 +77,28 @@ class StaticAttributeFactory : public AttributeFactory
       AttributeFactory::registerAttribute<CollectionRightsAttribute>();
       AttributeFactory::registerAttribute<EntityDisplayAttribute>();
       AttributeFactory::registerAttribute<EntityHiddenAttribute>();
+      AttributeFactory::registerAttribute<IndexPolicyAttribute>();
+      AttributeFactory::registerAttribute<PersistentSearchAttribute>();
+      AttributeFactory::registerAttribute<EntityDeletedAttribute>();
     }
     bool initialized;
 };
 
 K_GLOBAL_STATIC( StaticAttributeFactory, s_attributeInstance )
 
+}
+
+using Akonadi::Internal::s_attributeInstance;
+
+/**
+ * @internal
+ */
+class AttributeFactory::Private
+{
+  public:
+    QHash<QByteArray, Attribute*> attributes;
+};
+
 
 AttributeFactory* AttributeFactory::self()
 {
@@ -107,6 +120,7 @@ AttributeFactory::~ AttributeFactory()
 void AttributeFactory::registerAttribute(Attribute *attr)
 {
   Q_ASSERT( attr );
+  Q_ASSERT( !attr->type().contains(' ') && !attr->type().contains('\'') && !attr->type().contains('"') );
   QHash<QByteArray, Attribute*>::Iterator it = d->attributes.find( attr->type() );
   if ( it != d->attributes.end() ) {
     delete *it;
@@ -120,5 +134,8 @@ Attribute* AttributeFactory::createAttribute(const QByteArray &type)
   Attribute* attr = self()->d->attributes.value( type );
   if ( attr )
     return attr->clone();
-  return new DefaultAttribute( type );
+  return new Internal::DefaultAttribute( type );
 }
+
+}
+
