@@ -114,7 +114,7 @@ QModelIndex CollectionModel::index( int row, int column, const QModelIndex & par
   Q_D( const CollectionModel );
   if (column >= columnCount() || column < 0) return QModelIndex();
 
-  QList<Collection::Id> list;
+  QVector<Collection::Id> list;
   if ( !parent.isValid() )
     list = d->childCollections.value( Collection::root().id() );
   else
@@ -146,8 +146,8 @@ QModelIndex CollectionModel::parent( const QModelIndex & index ) const
   if ( !parentCol.isValid() )
   {
     return QModelIndex();
-}
-  QList<Collection::Id> list;
+  }
+  QVector<Collection::Id> list;
   list = d->childCollections.value( parentCol.parentCollection().id() );
 
   int parentRow = list.indexOf( parentCol.id() );
@@ -160,7 +160,7 @@ QModelIndex CollectionModel::parent( const QModelIndex & index ) const
 int CollectionModel::rowCount( const QModelIndex & parent ) const
 {
   const  Q_D( CollectionModel );
-  QList<Collection::Id> list;
+  QVector<Collection::Id> list;
   if ( parent.isValid() )
     list = d->childCollections.value( parent.internalId() );
   else
@@ -200,7 +200,7 @@ bool CollectionModel::setData( const QModelIndex & index, const QVariant & value
       return false;
     col.setName( value.toString() );
     CollectionModifyJob *job = new CollectionModifyJob( col, d->session );
-    connect( job, SIGNAL( result( KJob* ) ), SLOT( editDone( KJob* ) ) );
+    connect( job, SIGNAL(result(KJob*)), SLOT(editDone(KJob*)) );
     return true;
   }
   return QAbstractItemModel::setData( index, value, role );
@@ -288,7 +288,7 @@ bool CollectionModel::dropMimeData(const QMimeData * data, Qt::DropAction action
     return false;
 
   KJob *job = PasteHelper::paste( data, parentCol, action != Qt::MoveAction );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( dropResult( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(dropResult(KJob*)) );
   return true;
 }
 
