@@ -1,5 +1,6 @@
 /*
     Copyright (c) 2008 Thomas McGuire <thomas.mcguire@gmx.net>
+    Copyright (c) 2012 Laurent Montel <montel@kde.org>
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -22,11 +23,13 @@
 
 #include <kcolorscheme.h>
 #include <kdebug.h>
+#include <kio/global.h>
 
 #include <QtGui/QPainter>
 #include <QtGui/QStyle>
 #include <QtGui/QStyleOption>
 #include <QtGui/QStyleOptionViewItemV4>
+#include <QtGui/QAbstractItemView>
 #include <QtGui/QTreeView>
 
 #include "entitytreemodel.h"
@@ -47,13 +50,13 @@ enum CountType
 class CollectionStatisticsDelegatePrivate
 {
   public:
-    QTreeView *parent;
+    QAbstractItemView *parent;
     bool drawUnreadAfterFolder;
     DelegateAnimator *animator;
     QColor mSelectedUnreadColor;
     QColor mDeselectedUnreadColor;
 
-    CollectionStatisticsDelegatePrivate( QTreeView *treeView )
+    CollectionStatisticsDelegatePrivate( QAbstractItemView *treeView )
         : parent( treeView ),
           drawUnreadAfterFolder( false ),
           animator( 0 )
@@ -64,29 +67,36 @@ class CollectionStatisticsDelegatePrivate
                                            .foreground( KColorScheme::LinkText ).color();
     }
 
-    template<CountType countType>
-    qint64 getCountRecursive( const QModelIndex &index ) const
+    void getCountRecursive( const QModelIndex &index, qint64 &totalCount, qint64 &unreadCount, qint64 &totalSize ) const
     {
       Collection collection = qvariant_cast<Collection>( index.data( EntityTreeModel::CollectionRole ) );
-      Q_ASSERT( collection.isValid() );
-      CollectionStatistics statistics = collection.statistics();
-      qint64 count = qMax( 0LL, countType == UnreadCount ? statistics.unreadCount() : statistics.count() );
-
-      if ( index.model()->hasChildren( index ) )
-      {
-        const int rowCount = index.model()->rowCount( index );
-        for ( int row = 0; row < rowCount; row++ )
-        {
-          static const int column = 0;
-          count += getCountRecursive<countType>( index.model()->index( row, column, index ) );
+      // Do not assert on invalid collections, since a collection may be deleted
+      // in the meantime and deleted collections are invalid.
+      if ( collection.isValid() ) {
+        CollectionStatistics statistics = collection.statistics();
+        totalCount += qMax( 0LL, statistics.count() );
+        unreadCount += qMax( 0LL, statistics.unreadCount() );
+        totalSize += qMax( 0LL, statistics.size() );
+        if ( index.model()->hasChildren( index ) ) {
+          const int rowCount = index.model()->rowCount( index );
+          for ( int row = 0; row < rowCount; row++ ) {
+            static const int column = 0;
+            getCountRecursive( index.model()->index( row, column, index ), totalCount, unreadCount, totalSize );
+          }
         }
       }
-      return count;
     }
 };
 
 }
 
+CollectionStatisticsDelegate::CollectionStatisticsDelegate( QAbstractItemView *parent )
+  : QStyledItemDelegate( parent ),
+    d_ptr( new CollectionStatisticsDelegatePrivate( parent ) )
+{
+
+}
+
 CollectionStatisticsDelegate::CollectionStatisticsDelegate( QTreeView *parent )
   : QStyledItemDelegate( parent ),
     d_ptr( new CollectionStatisticsDelegatePrivate( parent ) )
@@ -140,7 +150,9 @@ void CollectionStatisticsDelegate::initStyleOption( QStyleOptionViewItem *option
   QStyleOptionViewItemV4 *noTextOption =
       qstyleoption_cast<QStyleOptionViewItemV4 *>( option );
   QStyledItemDelegate::initStyleOption( noTextOption, index );
-  noTextOption->text.clear();
+  if ( option->decorationPosition != QStyleOptionViewItem::Top ) {
+    noTextOption->text.clear();
+  }
 
   if ( d->animator ) {
 
@@ -191,6 +203,7 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
   Q_D( const CollectionStatisticsDelegate );
   PainterStateSaver stateSaver( painter );
 
+  const QColor textColor = index.data( Qt::ForegroundRole ).value<QColor>();
   // First, paint the basic, but without the text. We remove the text
   // in initStyleOption(), which gets called by QStyledItemDelegate::paint().
   QStyledItemDelegate::paint( painter, option, index );
@@ -204,15 +217,17 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
   // Now calculate the rectangle for the text
   QStyle *s = d->parent->style();
   const QWidget *widget = option4.widget;
-  QRect textRect = s->subElementRect( QStyle::SE_ItemViewItemText, &option4, widget );
+  const QRect textRect = s->subElementRect( QStyle::SE_ItemViewItemText, &option4, widget );
+  const QRect iconRect = s->subElementRect( QStyle::SE_ItemViewItemDecoration, &option4, widget );
 
    // When checking if the item is expanded, we need to check that for the first
   // column, as Qt only recogises the index as expanded for the first column
   QModelIndex firstColumn = index.model()->index( index.row(), 0, index.parent() );
-  bool expanded = d->parent->isExpanded( firstColumn );
+  QTreeView* treeView = qobject_cast<QTreeView*>( d->parent );
+  bool expanded = treeView && treeView->isExpanded( firstColumn );
 
   if ( option.state & QStyle::State_Selected ) {
-    painter->setPen( option.palette.highlightedText().color() );
+    painter->setPen( textColor.isValid() ? textColor : option.palette.highlightedText().color() );
   }
 
   Collection collection = index.sibling( index.row(), 0 ).data( EntityTreeModel::CollectionRole ).value<Collection>();
@@ -222,7 +237,10 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
   CollectionStatistics statistics = collection.statistics();
 
   qint64 unreadCount = qMax( 0LL, statistics.unreadCount() );
-  qint64 unreadRecursiveCount = d->getCountRecursive<UnreadCount>( index.sibling( index.row(), 0 ) );
+  qint64 totalRecursiveCount = 0;
+  qint64 unreadRecursiveCount = 0;
+  qint64 totalSize = 0;
+  d->getCountRecursive( index.sibling( index.row(), 0 ), totalRecursiveCount, unreadRecursiveCount, totalSize );
 
   // Draw the unread count after the folder name (in parenthesis)
   if ( d->drawUnreadAfterFolder && index.column() == 0 ) {
@@ -231,12 +249,12 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
     QString unread;
 //     qDebug() << expanded << unreadCount << unreadRecursiveCount;
     if ( expanded && unreadCount > 0 )
-      unread = QString( QLatin1String( " (%1)" ) ).arg( unreadCount );
+      unread = QString::fromLatin1( " (%1)" ).arg( unreadCount );
     else if ( !expanded ) {
       if ( unreadCount != unreadRecursiveCount )
-        unread = QString( QLatin1String( " (%1 + %2)" ) ).arg( unreadCount ).arg( unreadRecursiveCount - unreadCount );
+        unread = QString::fromLatin1( " (%1 + %2)" ).arg( unreadCount ).arg( unreadRecursiveCount - unreadCount );
       else if ( unreadCount > 0 )
-        unread = QString( QLatin1String( " (%1)" ) ).arg( unreadCount );
+        unread = QString::fromLatin1( " (%1)" ).arg( unreadCount );
     }
 
     PainterStateSaver stateSaver( painter );
@@ -247,28 +265,42 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
       painter->setFont( font );
     }
 
-    // Squeeze the folder text if it is to big and calculate the rectangles
-    // where the folder text and the unread count will be drawn to
-    QString folderName = text;
-    QFontMetrics fm( painter->fontMetrics() );
-    int unreadWidth = fm.width( unread );
-    if ( fm.width( folderName ) + unreadWidth > textRect.width() ) {
-      folderName = fm.elidedText( folderName, Qt::ElideRight,
-                                  textRect.width() - unreadWidth );
-    }
-    int folderWidth = fm.width( folderName );
-    QRect folderRect = textRect;
-    QRect unreadRect = textRect;
-    folderRect.setRight( textRect.left() + folderWidth );
-    unreadRect.setLeft( folderRect.right() );
-
-    // Draw folder name and unread count
-    painter->drawText( folderRect, Qt::AlignLeft, folderName );
-
     const QColor unreadColor = (option.state & QStyle::State_Selected) ? d->mSelectedUnreadColor : d->mDeselectedUnreadColor;
-    painter->setPen( unreadColor );
-    painter->drawText( unreadRect, Qt::AlignLeft, unread );
+    const QRect iconRect = s->subElementRect( QStyle::SE_ItemViewItemDecoration, &option4, widget );
 
+    if ( option.decorationPosition == QStyleOptionViewItem::Left
+         || option.decorationPosition == QStyleOptionViewItem::Right ) {
+      // Squeeze the folder text if it is to big and calculate the rectangles
+      // where the folder text and the unread count will be drawn to
+      QString folderName = text;
+      QFontMetrics fm( painter->fontMetrics() );
+      const int unreadWidth = fm.width( unread );
+      int folderWidth( fm.width( folderName ) );
+      const bool enoughPlaceForText = ( option.rect.width() > ( folderWidth + unreadWidth + iconRect.width() ) );
+
+       if ( !enoughPlaceForText && ( folderWidth + unreadWidth > textRect.width() )) {
+        folderName = fm.elidedText( folderName, Qt::ElideRight,
+                                    option.rect.width() - unreadWidth - iconRect.width() );
+        folderWidth = fm.width( folderName );
+      }
+      QRect folderRect = textRect;
+      QRect unreadRect = textRect;
+      folderRect.setRight( textRect.left() + folderWidth );
+      unreadRect = QRect( folderRect.right(), folderRect.top(), unreadRect.width(), unreadRect.height() );
+      if ( textColor.isValid() )
+        painter->setPen( textColor );
+
+      // Draw folder name and unread count
+      painter->drawText( folderRect, Qt::AlignLeft | Qt::AlignVCenter, folderName );
+      painter->setPen( unreadColor );
+      painter->drawText( unreadRect, Qt::AlignLeft | Qt::AlignVCenter, unread );
+    } else if ( option.decorationPosition == QStyleOptionViewItem::Top ) {
+      if ( unreadCount > 0 ) {
+        // draw over the icon
+        painter->setPen( unreadColor );
+        painter->drawText( iconRect, Qt::AlignCenter, QString::number( unreadCount ) );
+      }
+    }
     return;
   }
 
@@ -276,10 +308,7 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
   // is collapsed
   if ( ( index.column() == 1 || index.column() == 2 ) ) {
 
-    PainterStateSaver stateSaver( painter );
-
-    QStyleOptionViewItem opt = option;
-
+    QFont savedFont = painter->font();
     QString sumText;
     if ( index.column() == 1 && ( ( !expanded && unreadRecursiveCount > 0 ) || ( expanded && unreadCount > 0 ) ) ) {
       QFont font = painter->font();
@@ -289,18 +318,27 @@ void CollectionStatisticsDelegate::paint( QPainter *painter,
     } else {
 
       qint64 totalCount = statistics.count();
-      qint64 totalRecursiveCount = d->getCountRecursive<TotalCount>( index.sibling( index.row(), 0 ) );
       if (index.column() == 2 && ( ( !expanded && totalRecursiveCount > 0 ) || ( expanded && totalCount > 0 ) ) ) {
         sumText = QString::number( expanded ? totalCount : totalRecursiveCount );
       }
     }
 
-    painter->drawText( textRect, Qt::AlignRight, sumText );
-
+    painter->drawText( textRect, Qt::AlignRight | Qt::AlignVCenter, sumText );
+    painter->setFont( savedFont );
     return;
   }
 
-  painter->drawText( textRect, option4.displayAlignment, text );
+  //total size
+  if ( index.column() == 3 && !expanded ) {
+    if ( textColor.isValid() )
+      painter->setPen( textColor );
+    painter->drawText( textRect, option4.displayAlignment | Qt::AlignVCenter, KIO::convertSize( (KIO::filesize_t)totalSize ) );
+    return;
+  }
+
+  if ( textColor.isValid() )
+    painter->setPen( textColor );
+  painter->drawText( textRect, option4.displayAlignment | Qt::AlignVCenter, text );
 }
 
 #include "collectionstatisticsdelegate.moc"
