@@ -46,7 +46,7 @@ class EntityTreeModelPrivate;
  * Akonadi models and views provide a high level way to interact with the akonadi server.
  * Most applications will use these classes.
  *
- * Models provide an interface for viewing, deleting and moving Items and Collections.
+ * Models provide an interface for viewing, updating, deleting and moving Items and Collections.
  * Additionally, the models are updated automatically if another application changes the
  * data or inserts of deletes items etc.
  *
@@ -168,6 +168,15 @@ class EntityTreeModelPrivate;
  *
  * This feature is used in KAddressBook.
  *
+ * If items are to be fetched by the model, it is necessary to specify which parts of the items
+ * are to be fetched, using the ItemFetchScope class. By default, only the basic metadata is
+ * fetched. To fetch all item data, including all attributes:
+ *
+ * @code
+ * changeRecorder->itemFetchScope().fetchFullPayload();
+ * changeRecorder->itemFetchScope().fetchAllAttributes();
+ * @endcode
+ *
  * <h2>Using EntityTreeModel with Proxy models</h2>
  *
  * An Akonadi::SelectionProxyModel can be used to simplify managing selection in one view through multiple proxy models to a representation in another view.
@@ -339,18 +348,29 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
       EntityUrlRole,                          ///< The akonadi:/ Url of the entity as a string. Item urls will contain the mimetype.
       UnreadCountRole,                        ///< Returns the number of unread items in a collection. @since 4.5
       FetchStateRole,                         ///< Returns the FetchState of a particular item. @since 4.5
+      CollectionSyncProgressRole,             ///< Returns the progress of synchronization in percent for a particular collection. @since 4.6
       UserRole = Qt::UserRole + 500,          ///< First role for user extensions.
       TerminalUserRole = 2000,                ///< Last role for user extensions. Don't use a role beyond this or headerData will break.
       EndRole = 65535
     };
 
     /**
-     * Describes the state of fetch jobs related to particular entities.
+     * Describes the state of fetch jobs related to particular collections.
+     *
+     * @code
+     *   QModelIndex collectionIndex = getIndex();
+     *   if (collectionIndex.data(EntityTreeModel::FetchStateRole).toLongLong() == FetchingState) {
+     *     // There is a fetch underway
+     *   } else {
+     *     // There is no fetch underway.
+     *   }
+     * @endcode
+     *
      * @since 4.5
      */
     enum FetchState {
-      IdleState,                              ///< There is no fetch in progress.
-      FetchingState                           ///< There is a fetch in progress.
+      IdleState,                              ///< There is no fetch of items in this collection in progress.
+      FetchingState                           ///< There is a fetch of items in this collection in progress.
       // TODO: Change states for reporting of fetching payload parts of items.
     };
 
@@ -415,6 +435,10 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
      * Sets whether unsubscribed entities will be included in the listing.
      * By default it's true
      *
+     * Note that it is possible to change the monitor's fetchscope directly,
+     *  bypassing this method, which will lead to inconsistencies. Use
+     *  this method for turning on/off listing of subscribed folders.
+     *
      * @since 4.5
      */
     void setIncludeUnsubscribed( bool show );
@@ -529,7 +553,7 @@ class AKONADI_EXPORT EntityTreeModel : public QAbstractItemModel
      *
      * This can be useful for example if an id is stored in a config file and needs to be used in the application.
      *
-     * Note however, that to restore view state such as scrolling, selection and expansion of items in trees, the EntityTreeViewStateSaver can be used for convenience.
+     * Note however, that to restore view state such as scrolling, selection and expansion of items in trees, the ETMViewStateSaver can be used for convenience.
      *
      * @see modelIndexesForItem
      * @since 4.5
@@ -598,29 +622,35 @@ private:
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionStatisticsChanged( Akonadi::Collection::Id,
                                                                          const Akonadi::CollectionStatistics& ) )
 
-    Q_PRIVATE_SLOT( d_func(), void rootCollectionFetched(Akonadi::Collection::List) )
     Q_PRIVATE_SLOT( d_func(), void startFirstListJob() )
     Q_PRIVATE_SLOT( d_func(), void serverStarted() )
 
-    // TODO: Can I merge these into one jobResult slot?
     Q_PRIVATE_SLOT( d_func(), void fetchJobDone( KJob *job ) )
+    Q_PRIVATE_SLOT( d_func(), void rootFetchJobDone( KJob *job ) )
     Q_PRIVATE_SLOT( d_func(), void pasteJobDone( KJob *job ) )
     Q_PRIVATE_SLOT( d_func(), void updateJobDone( KJob *job ) )
+    Q_PRIVATE_SLOT( d_func(), void firstFetchJobDone( KJob *job ) )
 
     Q_PRIVATE_SLOT( d_func(), void itemsFetched( Akonadi::Item::List ) )
+    Q_PRIVATE_SLOT( d_func(), void itemsFetched( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void collectionsFetched( Akonadi::Collection::List ) )
+    Q_PRIVATE_SLOT( d_func(), void firstCollectionsFetched( Akonadi::Collection::List ) )
     Q_PRIVATE_SLOT( d_func(), void collectionListFetched( Akonadi::Collection::List ) )
     Q_PRIVATE_SLOT( d_func(), void topLevelCollectionsFetched( Akonadi::Collection::List ) )
     Q_PRIVATE_SLOT( d_func(), void ancestorsFetched( Akonadi::Collection::List ) )
 
     Q_PRIVATE_SLOT( d_func(), void monitoredMimeTypeChanged( const QString&, bool ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionsChanged( const Akonadi::Collection&, bool ) )
+    Q_PRIVATE_SLOT( d_func(), void monitoredItemsChanged( const Akonadi::Item&, bool ) )
+    Q_PRIVATE_SLOT( d_func(), void monitoredResourcesChanged( const QByteArray&, bool ) )
 
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionAdded( const Akonadi::Collection&, const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionRemoved( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionChanged( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredCollectionMoved( const Akonadi::Collection&, const Akonadi::Collection&,
                                                              const Akonadi::Collection&) )
+    Q_PRIVATE_SLOT( d_func(), void collectionSubscribed( const Akonadi::Collection&, const Akonadi::Collection& ) )
+    Q_PRIVATE_SLOT( d_func(), void monitoredCollectionUnsubscribed( const Akonadi::Collection& ) )
 
     Q_PRIVATE_SLOT( d_func(), void monitoredItemAdded( const Akonadi::Item&, const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredItemRemoved( const Akonadi::Item& ) )
@@ -630,6 +660,11 @@ private:
 
     Q_PRIVATE_SLOT( d_func(), void monitoredItemLinked( const Akonadi::Item&, const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void monitoredItemUnlinked( const Akonadi::Item&, const Akonadi::Collection& ) )
+    Q_PRIVATE_SLOT( d_func(), void changeFetchState( const Akonadi::Collection & ) )
+
+    Q_PRIVATE_SLOT( d_func(), void agentInstanceAdvancedStatusChanged( const QString&, const QVariantMap& ) )
+    Q_PRIVATE_SLOT( d_func(), void agentInstanceRemoved( Akonadi::AgentInstance ) )
+    Q_PRIVATE_SLOT( d_func(), void monitoredItemsRetrieved(KJob* job) )
     //@endcond
 };
 
