@@ -22,6 +22,7 @@
 
 #include "collection.h"
 #include "item.h"
+#include "item_p.h"
 #include "itemcreatejob.h"
 #include "itemdeletejob.h"
 #include "itemfetchjob.h"
@@ -43,7 +44,7 @@ class ItemSync::Private
   public:
     Private( ItemSync *parent ) :
       q( parent ),
-      mTransactionMode( Single ),
+      mTransactionMode( SingleTransaction ),
       mCurrentTransaction( 0 ),
       mTransactionJobs( 0 ),
       mPendingJobs( 0 ),
@@ -53,7 +54,8 @@ class ItemSync::Private
       mStreaming( false ),
       mIncremental( false ),
       mLocalListDone( false ),
-      mDeliveryDone( false )
+      mDeliveryDone( false ),
+      mFinished( false )
     {
       // we want to fetch all data by default
       mFetchScope.fetchFullPayload();
@@ -63,6 +65,7 @@ class ItemSync::Private
     void createLocalItem( const Item &item );
     void checkDone();
     void slotLocalListDone( KJob* );
+    void slotLocalDeleteDone( KJob* );
     void slotLocalChangeDone( KJob* );
     void execute();
     void processItems();
@@ -76,13 +79,7 @@ class ItemSync::Private
     QHash<QString, Akonadi::Item> mLocalItemsByRemoteId;
     QSet<Akonadi::Item> mUnprocessedLocalItems;
 
-    // transaction mode, TODO: make this public API?
-    enum TransactionMode {
-      Single,
-      Chunkwise,
-      None
-    };
-    TransactionMode mTransactionMode;
+    ItemSync::TransactionMode mTransactionMode;
     TransactionSequence *mCurrentTransaction;
     int mTransactionJobs;
 
@@ -105,13 +102,17 @@ class ItemSync::Private
     bool mIncremental;
     bool mLocalListDone;
     bool mDeliveryDone;
+    bool mFinished;
 };
 
 void ItemSync::Private::createLocalItem( const Item & item )
 {
+  // don't try to do anything in error state
+  if ( q->error() )
+    return;
   mPendingJobs++;
   ItemCreateJob *create = new ItemCreateJob( item, mSyncCollection, subjobParent() );
-  q->connect( create, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalChangeDone( KJob* ) ) );
+  q->connect( create, SIGNAL(result(KJob*)), q, SLOT(slotLocalChangeDone(KJob*)) );
 }
 
 void ItemSync::Private::checkDone()
@@ -120,7 +121,10 @@ void ItemSync::Private::checkDone()
   if ( mPendingJobs > 0 || !mDeliveryDone || mTransactionJobs > 0 )
     return;
 
-  q->emitResult();
+  if ( !mFinished ) { // prevent double result emission, can happen since checkDone() is called from all over the place
+    mFinished = true;
+    q->emitResult();
+  }
 }
 
 ItemSync::ItemSync( const Collection &collection, QObject *parent ) :
@@ -195,11 +199,15 @@ void ItemSync::doStart()
   // we only can fetch parts already in the cache, otherwise this will deadlock
   job->fetchScope().setCacheOnly( true );
 
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( slotLocalListDone( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(slotLocalListDone(KJob*)) );
 }
 
 bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
 {
+  // we are in error state, better not change anything at all anymore
+  if ( error() )
+    return false;
+
   /*
    * We know that this item has changed (as it is part of the
    * incremental changed list), so we just put it into the
@@ -208,6 +216,13 @@ bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
   if ( d->mIncremental )
     return true;
 
+  if ( newItem.d_func()->mClearPayload )
+    return true;
+
+  // Check whether the remote revisions differ
+  if ( storedItem.remoteRevision() != newItem.remoteRevision() )
+    return true;
+
   // Check whether the flags differ
   if ( storedItem.flags() != newItem.flags() ) {
     kDebug() << "Stored flags "  << storedItem.flags()
@@ -216,15 +231,16 @@ bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
   }
 
   // Check whether the new item contains unknown parts
-  QSet<QByteArray> missingParts = storedItem.loadedPayloadParts();
-  missingParts.subtract( newItem.loadedPayloadParts() );
+  QSet<QByteArray> missingParts = newItem.loadedPayloadParts();
+  missingParts.subtract( storedItem.loadedPayloadParts() );
   if ( !missingParts.isEmpty() )
     return true;
 
   // ### FIXME SLOW!!!
   // If the available part identifiers don't differ, check
   // whether the content of the payload differs
-  if ( storedItem.payloadData() != newItem.payloadData() )
+  if ( newItem.hasPayload()
+    && storedItem.payloadData() != newItem.payloadData() )
     return true;
 
   // check if remote attributes have been changed
@@ -240,16 +256,15 @@ bool ItemSync::updateItem( const Item &storedItem, Item &newItem )
 
 void ItemSync::Private::slotLocalListDone( KJob * job )
 {
-  if ( job->error() )
-    return;
-
-  const Item::List list = static_cast<ItemFetchJob*>( job )->items();
-  foreach ( const Item &item, list ) {
-    if ( item.remoteId().isEmpty() )
-      continue;
-    mLocalItemsById.insert( item.id(), item );
-    mLocalItemsByRemoteId.insert( item.remoteId(), item );
-    mUnprocessedLocalItems.insert( item );
+  if ( !job->error() ) {
+    const Item::List list = static_cast<ItemFetchJob*>( job )->items();
+    foreach ( const Item &item, list ) {
+      if ( item.remoteId().isEmpty() )
+        continue;
+      mLocalItemsById.insert( item.id(), item );
+      mLocalItemsByRemoteId.insert( item.remoteId(), item );
+      mUnprocessedLocalItems.insert( item );
+    }
   }
 
   mLocalListDone = true;
@@ -261,15 +276,21 @@ void ItemSync::Private::execute()
   if ( !mLocalListDone )
     return;
 
-  if ( (mTransactionMode == Single && !mCurrentTransaction) || mTransactionMode == Chunkwise ) {
+  // early exit to avoid unnecessary TransactionSequence creation in MultipleTransactions mode
+  // TODO: do the transaction handling in a nicer way instead, only creating TransactionSequences when really needed
+  if ( !mDeliveryDone && mRemoteItems.isEmpty() )
+    return;
+
+  if ( (mTransactionMode == SingleTransaction && !mCurrentTransaction) || mTransactionMode == MultipleTransactions) {
     ++mTransactionJobs;
     mCurrentTransaction = new TransactionSequence( q );
+    mCurrentTransaction->setAutomaticCommittingEnabled( false );
     connect( mCurrentTransaction, SIGNAL(result(KJob*)), q, SLOT(slotTransactionResult(KJob*)) );
   }
 
   processItems();
   if ( !mDeliveryDone ) {
-    if ( mTransactionMode == Chunkwise && mCurrentTransaction ) {
+    if ( mTransactionMode == MultipleTransactions && mCurrentTransaction ) {
       mCurrentTransaction->commit();
       mCurrentTransaction = 0;
     }
@@ -323,7 +344,8 @@ void ItemSync::Private::processItems()
       remoteItem.setSize( localItem.size() );
       remoteItem.setRemoteId( localItem.remoteId() );  // in case someone clears remoteId by accident
       ItemModifyJob *mod = new ItemModifyJob( remoteItem, subjobParent() );
-      q->connect( mod, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalChangeDone( KJob* ) ) );
+      mod->disableRevisionCheck();
+      q->connect( mod, SIGNAL(result(KJob*)), q, SLOT(slotLocalChangeDone(KJob*)) );
     } else {
       mProgress++;
     }
@@ -333,6 +355,11 @@ void ItemSync::Private::processItems()
 
 void ItemSync::Private::deleteItems( const Item::List &items )
 {
+  // if in error state, better not change anything anymore
+  if ( q->error() )
+    return;
+
+  Item::List itemsToDelete;
   foreach ( const Item &item, items ) {
     Item delItem( item );
     if ( !item.isValid() ) {
@@ -341,24 +368,47 @@ void ItemSync::Private::deleteItems( const Item::List &items )
 
     if ( !delItem.isValid() ) {
 #ifndef NDEBUG
-      kWarning() << "Delete item (remoteeId=" << delItem.remoteId()
-                 << "mimeType=" << delItem.mimeType()
+      kWarning() << "Delete item (remoteeId=" << item.remoteId()
+                 << "mimeType=" << item.mimeType()
                  << ") does not have a valid UID and no item with that remote ID exists either";
 #endif
       continue;
     }
 
-    mPendingJobs++;
-    ItemDeleteJob *job = new ItemDeleteJob( delItem, subjobParent() );
-    q->connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalChangeDone( KJob* ) ) );
+    if ( delItem.remoteId().isEmpty() ) {
+      // don't attempt to remove items that never were written to the backend
+      continue;
+    }
+
+    itemsToDelete.append ( delItem );
   }
+
+  if ( !itemsToDelete.isEmpty() ) {
+    mPendingJobs++;
+    ItemDeleteJob *job = new ItemDeleteJob( itemsToDelete, subjobParent() );
+    q->connect( job, SIGNAL(result(KJob*)), q, SLOT(slotLocalDeleteDone(KJob*)) );
+
+    // It can happen that the groupware servers report us deleted items
+    // twice, in this case this item delete job will fail on the second try.
+    // To avoid a rollback of the complete transaction we gracefully allow the job
+    // to fail :)
+    TransactionSequence *transaction = qobject_cast<TransactionSequence*>( subjobParent() );
+    if ( transaction )
+      transaction->setIgnoreJobFailure( job );
+  }
+}
+
+void ItemSync::Private::slotLocalDeleteDone( KJob* )
+{
+  mPendingJobs--;
+  mProgress++;
+
+  checkDone();
 }
 
 void ItemSync::Private::slotLocalChangeDone( KJob * job )
 {
-  if ( job->error() )
-    return;
-
+  Q_UNUSED( job );
   mPendingJobs--;
   mProgress++;
 
@@ -367,9 +417,6 @@ void ItemSync::Private::slotLocalChangeDone( KJob * job )
 
 void ItemSync::Private::slotTransactionResult( KJob *job )
 {
-  if ( job->error() )
-    return;
-
   --mTransactionJobs;
   if ( mCurrentTransaction == job )
     mCurrentTransaction = 0;
@@ -379,7 +426,7 @@ void ItemSync::Private::slotTransactionResult( KJob *job )
 
 Job * ItemSync::Private::subjobParent() const
 {
-  if ( mCurrentTransaction && mTransactionMode != None )
+  if ( mCurrentTransaction && mTransactionMode != NoTransaction )
     return mCurrentTransaction;
   return q;
 }
@@ -396,4 +443,34 @@ void ItemSync::deliveryDone()
   d->execute();
 }
 
+void ItemSync::slotResult(KJob* job)
+{
+  if ( job->error() ) {
+    // pretent there were no errors
+    Akonadi::Job::removeSubjob( job );
+    // propagate the first error we got but continue, we might still be fed with stuff from a resource
+    if ( !error() ) {
+      setError( job->error() );
+      setErrorText( job->errorText() );
+    }
+  } else {
+    Akonadi::Job::slotResult( job );
+  }
+}
+
+void ItemSync::rollback()
+{
+  setError( UserCanceled );
+  if ( d->mCurrentTransaction )
+    d->mCurrentTransaction->rollback();
+  d->mDeliveryDone = true; // user wont deliver more data
+  d->execute(); // end this in an ordered way, since we have an error set no real change will be done
+}
+
+void ItemSync::setTransactionMode(ItemSync::TransactionMode mode)
+{
+  d->mTransactionMode = mode;
+}
+
+
 #include "itemsync.moc"
