@@ -29,8 +29,10 @@
 #include "itemfetchscope.h"
 #include "job.h"
 #include <akonadi/private/notificationmessage_p.h>
-#include "notificationmanagerinterface.h"
+#include "notificationsourceinterface.h"
 #include "entitycache_p.h"
+#include "servermanager.h"
+#include "changenotificationdependenciesfactory_p.h"
 
 #include <kmimetype.h>
 
@@ -47,13 +49,18 @@ class Monitor;
 class AKONADI_TESTS_EXPORT MonitorPrivate
 {
   public:
-    MonitorPrivate( Monitor *parent );
-    virtual ~MonitorPrivate() {}
+    MonitorPrivate( ChangeNotificationDependenciesFactory *dependenciesFactory_, Monitor *parent );
+    virtual ~MonitorPrivate() {
+      delete dependenciesFactory;
+      delete collectionCache;
+      delete itemCache;
+    }
     void init();
 
     Monitor *q_ptr;
     Q_DECLARE_PUBLIC( Monitor )
-    org::freedesktop::Akonadi::NotificationManager *nm;
+    ChangeNotificationDependenciesFactory *dependenciesFactory;
+    QObject* notificationSource;
     Collection::List collections;
     QSet<QByteArray> resources;
     QSet<Item::Id> items;
@@ -62,9 +69,10 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
     QList<QByteArray> sessions;
     ItemFetchScope mItemFetchScope;
     CollectionFetchScope mCollectionFetchScope;
+    bool mFetchChangedOnly;
     Session *session;
-    CollectionCache collectionCache;
-    ItemCache itemCache;
+    CollectionCache *collectionCache;
+    ItemCache *itemCache;
     QQueue<NotificationMessage> pendingNotifications;
     QQueue<NotificationMessage> pipeline;
     bool fetchCollection;
@@ -74,13 +82,18 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
     virtual bool connectToNotificationManager();
     bool acceptNotification( const NotificationMessage &msg );
     void dispatchNotifications();
+    void flushPipeline();
 
     // Called when the monitored item/collection changes, checks if the queued messages
     // are still accepted, if not they are removed
     void cleanOldNotifications();
 
     bool ensureDataAvailable( const NotificationMessage &msg );
-    void emitNotification( const NotificationMessage &msg );
+    /**
+     * Sends out the change notification @p msg.
+     * @return @c true if the notification was actually send to someone, @c false if no one was listening.
+     */
+    virtual bool emitNotification( const NotificationMessage &msg );
     void updatePendingStatistics( const NotificationMessage &msg );
     void invalidateCaches( const NotificationMessage &msg );
 
@@ -97,15 +110,37 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
     void slotStatisticsChangedFinished( KJob* );
     void slotFlushRecentlyChangedCollections();
 
-    void appendAndCompress( const NotificationMessage &msg  );
+    /**
+      Returns whether a message was appended to @p notificationQueue
+    */
+    bool translateAndCompress( QQueue<NotificationMessage> &notificationQueue, const NotificationMessage &msg  );
 
     virtual void slotNotify( const NotificationMessage::List &msgs );
 
-    void emitItemNotification( const NotificationMessage &msg, const Item &item = Item(),
+    /**
+     * Sends out a change notification for an item.
+     * @return @c true if the notification was actually send to someone, @c false if no one was listening.
+     */
+    bool emitItemNotification( const NotificationMessage &msg, const Item &item = Item(),
                                const Collection &collection = Collection(), const Collection &collectionDest = Collection() );
-    void emitCollectionNotification( const NotificationMessage &msg, const Collection &col = Collection(),
+    /**
+     * Sends out a change notification for a collection.
+     * @return @c true if the notification was actually send to someone, @c false if no one was listening.
+     */
+    bool emitCollectionNotification( const NotificationMessage &msg, const Collection &col = Collection(),
                                      const Collection &par = Collection(), const Collection &dest = Collection() );
 
+    void serverStateChanged( Akonadi::ServerManager::State state );
+
+    /**
+     * This method is called by the ChangeMediator to enforce an invalidation of the passed collection.
+     */
+    void invalidateCollectionCache( qint64 collectionId );
+
+    /**
+     * This method is called by the ChangeMediator to enforce an invalidation of the passed item.
+     */
+    void invalidateItemCache( qint64 itemId );
 
     /**
       @brief Class used to determine when to purge items in a Collection
@@ -148,7 +183,7 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
       }
 
     private:
-      QList<Collection::Id> m_buffer;
+      QVector<Collection::Id> m_buffer;
       int m_index;
       int m_bufferSize;
     } m_buffer;
@@ -162,6 +197,7 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
   private:
     // collections that need a statistics update
     QSet<Collection::Id> recentlyChangedCollections;
+    QTimer statisticsCompressionTimer;
 
     /**
       @returns True if @p msg should be ignored. Otherwise appropriate signals are emitted for it.
@@ -170,6 +206,8 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
 
     bool isCollectionMonitored( Collection::Id collection ) const
     {
+      if (collection < 0)
+        return false;
       if ( collections.contains( Collection( collection ) ) )
         return true;
       if ( collections.contains( Collection::root() ) )
@@ -196,10 +234,9 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
 
     bool isMoveDestinationResourceMonitored( const NotificationMessage &msg )
     {
-      if ( msg.operation() != NotificationMessage::Move || msg.itemParts().isEmpty() )
+      if ( msg.operation() != NotificationMessage::Move )
         return false;
-      const QByteArray res = *(msg.itemParts().begin());
-      return resources.contains( res );
+      return resources.contains( msg.destinationResource() );
     }
 
     void fetchStatistics( Collection::Id colId )
@@ -208,14 +245,7 @@ class AKONADI_TESTS_EXPORT MonitorPrivate
       QObject::connect( job, SIGNAL( result( KJob* ) ), q_ptr, SLOT( slotStatisticsChangedFinished( KJob* ) ) );
     }
 
-    void notifyCollectionStatisticsWatchers( Collection::Id collection, const QByteArray &resource )
-    {
-      if ( collection > 0 && (monitorAll || isCollectionMonitored( collection ) || resources.contains( resource ) ) ) {
-        if (recentlyChangedCollections.empty() )
-          QTimer::singleShot( 500, q_ptr, SLOT( slotFlushRecentlyChangedCollections() ) );
-        recentlyChangedCollections.insert( collection );
-      }
-    }
+    void notifyCollectionStatisticsWatchers( Collection::Id collection, const QByteArray &resource );
 };
 
 }
