@@ -24,7 +24,9 @@
 #include "resourceadaptor.h"
 #include "collectiondeletejob.h"
 #include "collectionsync_p.h"
+#include "dbusconnectionpool.h"
 #include "itemsync.h"
+#include "kdepimlibs-version.h"
 #include "resourcescheduler_p.h"
 #include "tracerinterface.h"
 #include "xdgbasedirs_p.h"
@@ -33,6 +35,7 @@
 #include "collectionfetchjob.h"
 #include "collectionfetchscope.h"
 #include "collectionmodifyjob.h"
+#include "invalidatecachejob_p.h"
 #include "itemfetchjob.h"
 #include "itemfetchscope.h"
 #include "itemmodifyjob.h"
@@ -40,6 +43,7 @@
 #include "session.h"
 #include "resourceselectjob_p.h"
 #include "monitor_p.h"
+#include "servermanager_p.h"
 
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
@@ -58,59 +62,235 @@ using namespace Akonadi;
 
 class Akonadi::ResourceBasePrivate : public AgentBasePrivate
 {
+  Q_OBJECT
+  Q_CLASSINFO( "D-Bus Interface", "org.kde.dfaure" )
+
   public:
     ResourceBasePrivate( ResourceBase *parent )
       : AgentBasePrivate( parent ),
         scheduler( 0 ),
         mItemSyncer( 0 ),
+        mItemSyncFetchScope( 0 ),
+        mItemTransactionMode( ItemSync::SingleTransaction ),
         mCollectionSyncer( 0 ),
-        mHierarchicalRid( false )
+        mHierarchicalRid( false ),
+        mUnemittedProgress( 0 ),
+        mAutomaticProgressReporting( true )
     {
+      Internal::setClientType( Internal::Resource );
       mStatusMessage = defaultReadyMessage();
+      mProgressEmissionCompressor.setInterval( 1000 );
+      mProgressEmissionCompressor.setSingleShot( true );
+    }
+
+    ~ResourceBasePrivate()
+    {
+      delete mItemSyncFetchScope;
     }
 
     Q_DECLARE_PUBLIC( ResourceBase )
 
     void delayedInit()
     {
-      if ( !QDBusConnection::sessionBus().registerService( QLatin1String( "org.freedesktop.Akonadi.Resource." ) + mId ) )
-        kFatal() << "Unable to register service at D-Bus: " << QDBusConnection::sessionBus().lastError().message();
-      AgentBasePrivate::delayedInit();
+      if ( !DBusConnectionPool::threadConnection().registerService( QLatin1String( "org.freedesktop.Akonadi.Resource." ) + mId ) ) {
+        QString reason = DBusConnectionPool::threadConnection().lastError().message();
+        if ( reason.isEmpty() ) {
+          reason = QString::fromLatin1( "this service is probably running already." );
+        }
+        kError() << "Unable to register service at D-Bus: " << reason;
+
+        if ( QThread::currentThread() == QCoreApplication::instance()->thread() )
+          QCoreApplication::instance()->exit(1);
+
+      } else {
+        AgentBasePrivate::delayedInit();
+      }
     }
 
     virtual void changeProcessed()
     {
-      mMonitor->changeProcessed();
-      if ( !mMonitor->isEmpty() )
+      mChangeRecorder->changeProcessed();
+      if ( !mChangeRecorder->isEmpty() )
         scheduler->scheduleChangeReplay();
       scheduler->taskDone();
     }
 
+    void slotAbortRequested();
+
     void slotDeliveryDone( KJob* job );
     void slotCollectionSyncDone( KJob *job );
     void slotLocalListDone( KJob *job );
     void slotSynchronizeCollection( const Collection &col );
     void slotCollectionListDone( KJob *job );
+    void slotSynchronizeCollectionAttributes( const Collection &col );
+    void slotCollectionListForAttributesDone( KJob *job );
+    void slotCollectionAttributesSyncDone( KJob *job );
 
     void slotItemSyncDone( KJob *job );
 
     void slotPercent( KJob* job, unsigned long percent );
+    void slotDelayedEmitProgress();
     void slotDeleteResourceCollection();
     void slotDeleteResourceCollectionDone( KJob *job );
     void slotCollectionDeletionDone( KJob *job );
 
+    void slotInvalidateCache( const Akonadi::Collection &collection );
+
     void slotPrepareItemRetrieval( const Akonadi::Item &item );
     void slotPrepareItemRetrievalResult( KJob* job );
 
     void changeCommittedResult( KJob* job );
 
+    void slotSessionReconnected()
+    {
+      Q_Q( ResourceBase );
+
+      new ResourceSelectJob( q->identifier() );
+    }
+
+    void createItemSyncInstanceIfMissing()
+    {
+      Q_Q( ResourceBase );
+      Q_ASSERT_X( scheduler->currentTask().type == ResourceScheduler::SyncCollection,
+                  "createItemSyncInstance", "Calling items retrieval methods although no item retrieval is in progress" );
+      if ( !mItemSyncer ) {
+        mItemSyncer = new ItemSync( q->currentCollection() );
+        mItemSyncer->setTransactionMode( mItemTransactionMode );
+        if ( mItemSyncFetchScope )
+          mItemSyncer->setFetchScope( *mItemSyncFetchScope );
+        mItemSyncer->setProperty( "collection", QVariant::fromValue( q->currentCollection() ) );
+        connect( mItemSyncer, SIGNAL(percent(KJob*,ulong)), q, SLOT(slotPercent(KJob*,ulong)) );
+        connect( mItemSyncer, SIGNAL(result(KJob*)), q, SLOT(slotItemSyncDone(KJob*)) );
+      }
+      Q_ASSERT( mItemSyncer );
+    }
+
+  public Q_SLOTS:
+    // Dump the contents of the current ChangeReplay
+    Q_SCRIPTABLE QString dumpNotificationListToString() const
+    {
+      return mChangeRecorder->dumpNotificationListToString();
+    }
+
+    // Dump the state of the scheduler
+    Q_SCRIPTABLE QString dumpToString() const
+    {
+      return scheduler->dumpToString();
+    }
+
+    Q_SCRIPTABLE void dump()
+    {
+      scheduler->dump();
+    }
+
+    Q_SCRIPTABLE void clear()
+    {
+      scheduler->clear();
+    }
+
+  protected Q_SLOTS:
+    // reimplementations from AgentbBasePrivate, containing sanity checks that only apply to resources
+    // such as making sure that RIDs are present as well as translations of cross-resource moves
+    // TODO: we could possibly add recovery code for no-RID notifications by re-enquing those to the change recorder
+    // as the corresponding Add notifications, although that contains a risk of endless fail/retry loops
+
+    void itemAdded(const Akonadi::Item& item, const Akonadi::Collection& collection)
+    {
+      if ( collection.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::itemAdded( item, collection );
+    }
+
+    void itemChanged(const Akonadi::Item& item, const QSet< QByteArray >& partIdentifiers)
+    {
+      if ( item.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::itemChanged( item, partIdentifiers );
+    }
+
+    // TODO move the move translation code from AgebtBasePrivate here, it's wrong for agents
+    void itemMoved(const Akonadi::Item &item, const Akonadi::Collection &source, const Akonadi::Collection &destination)
+    {
+      if ( item.remoteId().isEmpty() || destination.remoteId().isEmpty() || destination == source ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::itemMoved( item, source, destination );
+    }
+
+    void itemRemoved(const Akonadi::Item& item)
+    {
+      if ( item.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::itemRemoved( item );
+    }
+
+    void collectionAdded(const Akonadi::Collection& collection, const Akonadi::Collection& parent)
+    {
+      if ( parent.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::collectionAdded( collection, parent );
+    }
+
+    void collectionChanged(const Akonadi::Collection& collection)
+    {
+      if ( collection.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::collectionChanged( collection );
+    }
+
+    void collectionChanged(const Akonadi::Collection& collection, const QSet< QByteArray >& partIdentifiers)
+    {
+      if ( collection.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::collectionChanged( collection, partIdentifiers );
+    }
+
+    // TODO move the move translation code from AgebtBasePrivate here, it's wrong for agents
+    void collectionMoved(const Akonadi::Collection& collection, const Akonadi::Collection& source, const Akonadi::Collection& destination)
+    {
+      if ( collection.remoteId().isEmpty() || destination.remoteId().isEmpty() || source == destination ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::collectionMoved( collection, source, destination );
+    }
+
+    void collectionRemoved(const Akonadi::Collection& collection)
+    {
+      if ( collection.remoteId().isEmpty() ) {
+        changeProcessed();
+        return;
+      }
+      AgentBasePrivate::collectionRemoved( collection );
+    }
+
+  public:
     // synchronize states
     Collection currentCollection;
 
     ResourceScheduler *scheduler;
     ItemSync *mItemSyncer;
+    ItemFetchScope *mItemSyncFetchScope;
+    ItemSync::TransactionMode mItemTransactionMode;
     CollectionSync *mCollectionSyncer;
     bool mHierarchicalRid;
+    QTimer mProgressEmissionCompressor;
+    int mUnemittedProgress;
+    QMap<Akonadi::Collection::Id, QVariantMap> mUnemittedAdvancedStatus;
+    bool mAutomaticProgressReporting;
 };
 
 ResourceBase::ResourceBase( const QString & id )
@@ -118,43 +298,58 @@ ResourceBase::ResourceBase( const QString & id )
 {
   Q_D( ResourceBase );
 
-  new ResourceAdaptor( this );
+  new Akonadi__ResourceAdaptor( this );
 
   d->scheduler = new ResourceScheduler( this );
 
-  d->mMonitor->setChangeRecordingEnabled( true );
-  connect( d->mMonitor, SIGNAL( changesAdded() ),
-           d->scheduler, SLOT( scheduleChangeReplay() ) );
+  d->mChangeRecorder->setChangeRecordingEnabled( true );
+  connect( d->mChangeRecorder, SIGNAL(changesAdded()),
+           d->scheduler, SLOT(scheduleChangeReplay()) );
 
-  d->mMonitor->setResourceMonitored( d->mId.toLatin1() );
+  d->mChangeRecorder->setResourceMonitored( d->mId.toLatin1() );
+  d->mChangeRecorder->fetchCollection( true );
 
-  connect( d->scheduler, SIGNAL( executeFullSync() ),
-           SLOT( retrieveCollections() ) );
-  connect( d->scheduler, SIGNAL( executeCollectionTreeSync() ),
-           SLOT( retrieveCollections() ) );
-  connect( d->scheduler, SIGNAL( executeCollectionSync( const Akonadi::Collection& ) ),
-           SLOT( slotSynchronizeCollection( const Akonadi::Collection& ) ) );
-  connect( d->scheduler, SIGNAL( executeItemFetch( const Akonadi::Item&, const QSet<QByteArray>& ) ),
-           SLOT( slotPrepareItemRetrieval(Akonadi::Item)) );
-  connect( d->scheduler, SIGNAL( executeResourceCollectionDeletion() ),
-           SLOT( slotDeleteResourceCollection() ) );
-  connect( d->scheduler, SIGNAL( status( int, const QString& ) ),
-           SIGNAL( status( int, const QString& ) ) );
-  connect( d->scheduler, SIGNAL( executeChangeReplay() ),
-           d->mMonitor, SLOT( replayNext() ) );
-  connect( d->scheduler, SIGNAL( fullSyncComplete() ), SIGNAL( synchronized() ) );
-  connect( d->mMonitor, SIGNAL( nothingToReplay() ), d->scheduler, SLOT( taskDone() ) );
-  connect( d->mMonitor, SIGNAL(collectionRemoved(Akonadi::Collection)),
+  connect( d->scheduler, SIGNAL(executeFullSync()),
+           SLOT(retrieveCollections()) );
+  connect( d->scheduler, SIGNAL(executeCollectionTreeSync()),
+           SLOT(retrieveCollections()) );
+  connect( d->scheduler, SIGNAL(executeCollectionSync(Akonadi::Collection)),
+           SLOT(slotSynchronizeCollection(Akonadi::Collection)) );
+  connect( d->scheduler, SIGNAL(executeCollectionAttributesSync(Akonadi::Collection)),
+           SLOT(slotSynchronizeCollectionAttributes(Akonadi::Collection)) );
+  connect( d->scheduler, SIGNAL(executeItemFetch(Akonadi::Item,QSet<QByteArray>)),
+           SLOT(slotPrepareItemRetrieval(Akonadi::Item)) );
+  connect( d->scheduler, SIGNAL(executeResourceCollectionDeletion()),
+           SLOT(slotDeleteResourceCollection()) );
+  connect ( d->scheduler, SIGNAL(executeCacheInvalidation(Akonadi::Collection)),
+            SLOT(slotInvalidateCache(Akonadi::Collection)) );
+  connect( d->scheduler, SIGNAL(status(int,QString)),
+           SIGNAL(status(int,QString)) );
+  connect( d->scheduler, SIGNAL(executeChangeReplay()),
+           d->mChangeRecorder, SLOT(replayNext()) );
+  connect( d->scheduler, SIGNAL(fullSyncComplete()), SIGNAL(synchronized()) );
+  connect( d->scheduler, SIGNAL(collectionTreeSyncComplete()), SIGNAL(collectionTreeSynchronized()) );
+  connect( d->mChangeRecorder, SIGNAL(nothingToReplay()), d->scheduler, SLOT(taskDone()) );
+  connect( d->mChangeRecorder, SIGNAL(collectionRemoved(Akonadi::Collection)),
            d->scheduler, SLOT(collectionRemoved(Akonadi::Collection)) );
-  connect( this, SIGNAL( synchronized() ), d->scheduler, SLOT( taskDone() ) );
-  connect( this, SIGNAL( agentNameChanged( const QString& ) ),
-           this, SIGNAL( nameChanged( const QString& ) ) );
+  connect( this, SIGNAL(abortRequested()), this, SLOT(slotAbortRequested()) );
+  connect( this, SIGNAL(synchronized()), d->scheduler, SLOT(taskDone()) );
+  connect( this, SIGNAL(collectionTreeSynchronized()), d->scheduler, SLOT(taskDone()) );
+  connect( this, SIGNAL(agentNameChanged(QString)),
+           this, SIGNAL(nameChanged(QString)) );
+
+  connect( &d->mProgressEmissionCompressor, SIGNAL(timeout()),
+           this, SLOT(slotDelayedEmitProgress()) );
 
   d->scheduler->setOnline( d->mOnline );
-  if ( !d->mMonitor->isEmpty() )
+  if ( !d->mChangeRecorder->isEmpty() )
     d->scheduler->scheduleChangeReplay();
 
+  DBusConnectionPool::threadConnection().registerObject( QLatin1String( "/Debug" ), d, QDBusConnection::ExportScriptableSlots );
+
   new ResourceSelectJob( identifier() );
+
+  connect( d->mChangeRecorder->session(), SIGNAL(reconnected()), SLOT(slotSessionReconnected()) );
 }
 
 ResourceBase::~ResourceBase()
@@ -194,20 +389,17 @@ QString ResourceBase::parseArguments( int argc, char **argv )
     exit( 1 );
   }
 
-  QByteArray catalog;
-  char *p = strrchr( argv[0], '/' );
-  if ( p )
-    catalog = QByteArray( p + 1 );
-  else
-    catalog = QByteArray( argv[0] );
+  const QFileInfo fi( QString::fromLocal8Bit( argv[0] ) );
+  // strip off full path and possible .exe suffix
+  const QByteArray catalog = fi.baseName().toLatin1();
 
   KCmdLineArgs::init( argc, argv, identifier.toLatin1(), catalog,
-                      ki18nc("@title, application name", "Akonadi Resource"), "0.1",
-                      ki18nc("@title, application description", "Akonadi Resource") );
+                      ki18nc( "@title application name", "Akonadi Resource" ), KDEPIMLIBS_VERSION,
+                      ki18nc( "@title application description", "Akonadi Resource" ) );
 
   KCmdLineOptions options;
   options.add( "identifier <argument>",
-               ki18nc("@label, commandline option", "Resource identifier") );
+               ki18nc( "@label commandline option", "Resource identifier" ) );
   KCmdLineArgs::addCmdLineOptions( options );
 
   return identifier;
@@ -216,19 +408,26 @@ QString ResourceBase::parseArguments( int argc, char **argv )
 int ResourceBase::init( ResourceBase *r )
 {
   QApplication::setQuitOnLastWindowClosed( false );
+  KGlobal::locale()->insertCatalog( QLatin1String( "libakonadi" ) );
   int rv = kapp->exec();
   delete r;
   return rv;
 }
 
+void ResourceBasePrivate::slotAbortRequested()
+{
+  Q_Q( ResourceBase );
+
+  scheduler->cancelQueues();
+  QMetaObject::invokeMethod( q, "abortActivity" );
+}
+
 void ResourceBase::itemRetrieved( const Item &item )
 {
   Q_D( ResourceBase );
   Q_ASSERT( d->scheduler->currentTask().type == ResourceScheduler::FetchItem );
   if ( !item.isValid() ) {
-    QDBusMessage reply( d->scheduler->currentTask().dbusMsg );
-    reply << false;
-    QDBusConnection::sessionBus().send( reply );
+    d->scheduler->currentTask().sendDBusReplies( false );
     d->scheduler->taskDone();
     return;
   }
@@ -244,21 +443,42 @@ void ResourceBase::itemRetrieved( const Item &item )
   ItemModifyJob *job = new ItemModifyJob( i );
   // FIXME: remove once the item with which we call retrieveItem() has a revision number
   job->disableRevisionCheck();
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( slotDeliveryDone( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), SLOT(slotDeliveryDone(KJob*)) );
 }
 
 void ResourceBasePrivate::slotDeliveryDone(KJob * job)
 {
   Q_Q( ResourceBase );
   Q_ASSERT( scheduler->currentTask().type == ResourceScheduler::FetchItem );
-  QDBusMessage reply( scheduler->currentTask().dbusMsg );
   if ( job->error() ) {
     emit q->error( QLatin1String( "Error while creating item: " ) + job->errorString() );
-    reply << false;
-  } else {
-    reply << true;
   }
-  QDBusConnection::sessionBus().send( reply );
+  scheduler->currentTask().sendDBusReplies( !job->error() );
+  scheduler->taskDone();
+}
+
+void ResourceBase::collectionAttributesRetrieved( const Collection &collection )
+{
+  Q_D( ResourceBase );
+  Q_ASSERT( d->scheduler->currentTask().type == ResourceScheduler::SyncCollectionAttributes );
+  if ( !collection.isValid() ) {
+    emit attributesSynchronized( d->scheduler->currentTask().collection.id() );
+    d->scheduler->taskDone();
+    return;
+  }
+
+  CollectionModifyJob *job = new CollectionModifyJob( collection );
+  connect( job, SIGNAL(result(KJob*)), SLOT(slotCollectionAttributesSyncDone(KJob*)) );
+}
+
+void ResourceBasePrivate::slotCollectionAttributesSyncDone(KJob * job)
+{
+  Q_Q( ResourceBase );
+  Q_ASSERT( scheduler->currentTask().type == ResourceScheduler::SyncCollectionAttributes );
+  if ( job->error() ) {
+    emit q->error( QLatin1String( "Error while updating collection: " ) + job->errorString() );
+  }
+  emit q->attributesSynchronized( scheduler->currentTask().collection.id() );
   scheduler->taskDone();
 }
 
@@ -268,7 +488,7 @@ void ResourceBasePrivate::slotDeleteResourceCollection()
 
   CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::FirstLevel );
   job->fetchScope().setResource( q->identifier() );
-  connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotDeleteResourceCollectionDone( KJob* ) ) );
+  connect( job, SIGNAL(result(KJob*)), q, SLOT(slotDeleteResourceCollectionDone(KJob*)) );
 }
 
 void ResourceBasePrivate::slotDeleteResourceCollectionDone( KJob *job )
@@ -282,7 +502,7 @@ void ResourceBasePrivate::slotDeleteResourceCollectionDone( KJob *job )
 
     if ( !fetchJob->collections().isEmpty() ) {
       CollectionDeleteJob *job = new CollectionDeleteJob( fetchJob->collections().first() );
-      connect( job, SIGNAL( result( KJob* ) ), q, SLOT( slotCollectionDeletionDone( KJob* ) ) );
+      connect( job, SIGNAL(result(KJob*)), q, SLOT(slotCollectionDeletionDone(KJob*)) );
     } else {
       // there is no resource collection, so just ignore the request
       scheduler->taskDone();
@@ -300,13 +520,20 @@ void ResourceBasePrivate::slotCollectionDeletionDone( KJob *job )
   scheduler->taskDone();
 }
 
+void ResourceBasePrivate::slotInvalidateCache( const Akonadi::Collection &collection )
+{
+  Q_Q( ResourceBase );
+  InvalidateCacheJob *job = new InvalidateCacheJob( collection, q );
+  connect( job, SIGNAL(result(KJob*)), scheduler, SLOT(taskDone()) );
+}
+
 void ResourceBase::changeCommitted( const Item& item )
 {
   Q_D( ResourceBase );
   ItemModifyJob *job = new ItemModifyJob( item );
   job->d_func()->setClean();
   job->disableRevisionCheck(); // TODO: remove, but where/how do we handle the error?
-  job->ignorePayload(); // we only want to reset the dirty flag and update the remote id
+  job->setIgnorePayload( true ); // we only want to reset the dirty flag and update the remote id
   d->changeProcessed();
 }
 
@@ -321,7 +548,7 @@ void ResourceBasePrivate::changeCommittedResult( KJob *job )
   Q_Q( ResourceBase );
   if ( job->error() )
     emit q->error( i18nc( "@info", "Updating local collection failed: %1.", job->errorText() ) );
-  mMonitor->d_ptr->invalidateCache( static_cast<CollectionModifyJob*>( job )->collection() );
+  mChangeRecorder->d_ptr->invalidateCache( static_cast<CollectionModifyJob*>( job )->collection() );
   changeProcessed();
 }
 
@@ -359,8 +586,8 @@ void ResourceBase::collectionsRetrieved( const Collection::List & collections )
   if ( !d->mCollectionSyncer ) {
     d->mCollectionSyncer = new CollectionSync( identifier() );
     d->mCollectionSyncer->setHierarchicalRemoteIds( d->mHierarchicalRid );
-    connect( d->mCollectionSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mCollectionSyncer, SIGNAL( result( KJob* ) ), SLOT( slotCollectionSyncDone( KJob* ) ) );
+    connect( d->mCollectionSyncer, SIGNAL(percent(KJob*,ulong)), SLOT(slotPercent(KJob*,ulong)) );
+    connect( d->mCollectionSyncer, SIGNAL(result(KJob*)), SLOT(slotCollectionSyncDone(KJob*)) );
   }
   d->mCollectionSyncer->setRemoteCollections( collections );
 }
@@ -376,8 +603,8 @@ void ResourceBase::collectionsRetrievedIncremental( const Collection::List & cha
   if ( !d->mCollectionSyncer ) {
     d->mCollectionSyncer = new CollectionSync( identifier() );
     d->mCollectionSyncer->setHierarchicalRemoteIds( d->mHierarchicalRid );
-    connect( d->mCollectionSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mCollectionSyncer, SIGNAL( result( KJob* ) ), SLOT( slotCollectionSyncDone( KJob* ) ) );
+    connect( d->mCollectionSyncer, SIGNAL(percent(KJob*,ulong)), SLOT(slotPercent(KJob*,ulong)) );
+    connect( d->mCollectionSyncer, SIGNAL(result(KJob*)), SLOT(slotCollectionSyncDone(KJob*)) );
   }
   d->mCollectionSyncer->setRemoteCollections( changedCollections, removedCollections );
 }
@@ -392,8 +619,8 @@ void ResourceBase::setCollectionStreamingEnabled( bool enable )
   if ( !d->mCollectionSyncer ) {
     d->mCollectionSyncer = new CollectionSync( identifier() );
     d->mCollectionSyncer->setHierarchicalRemoteIds( d->mHierarchicalRid );
-    connect( d->mCollectionSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mCollectionSyncer, SIGNAL( result( KJob* ) ), SLOT( slotCollectionSyncDone( KJob* ) ) );
+    connect( d->mCollectionSyncer, SIGNAL(percent(KJob*,ulong)), SLOT(slotPercent(KJob*,ulong)) );
+    connect( d->mCollectionSyncer, SIGNAL(result(KJob*)), SLOT(slotCollectionSyncDone(KJob*)) );
   }
   d->mCollectionSyncer->setStreamingEnabled( enable );
 }
@@ -421,14 +648,17 @@ void ResourceBasePrivate::slotCollectionSyncDone( KJob * job )
   Q_Q( ResourceBase );
   mCollectionSyncer = 0;
   if ( job->error() ) {
-    emit q->error( job->errorString() );
+    if ( job->error() != Job::UserCanceled )
+      emit q->error( job->errorString() );
   } else {
     if ( scheduler->currentTask().type == ResourceScheduler::SyncAll ) {
       CollectionFetchJob *list = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive );
+      list->setFetchScope( q->changeRecorder()->collectionFetchScope() );
       list->fetchScope().setResource( mId );
-      list->fetchScope().setAncestorRetrieval( q->changeRecorder()->collectionFetchScope().ancestorRetrieval() );
-      q->connect( list, SIGNAL( result( KJob* ) ), q, SLOT( slotLocalListDone( KJob* ) ) );
+      q->connect( list, SIGNAL(result(KJob*)), q, SLOT(slotLocalListDone(KJob*)) );
       return;
+    } else if ( scheduler->currentTask().type == ResourceScheduler::SyncCollectionTree ) {
+      scheduler->scheduleCollectionTreeSyncCompletion();
     }
   }
   scheduler->taskDone();
@@ -457,19 +687,33 @@ void ResourceBasePrivate::slotSynchronizeCollection( const Collection &col )
   QStringList contentTypes = currentCollection.contentMimeTypes();
   contentTypes.removeAll( Collection::mimeType() );
   if ( !contentTypes.isEmpty() || (col.rights() & (Collection::CanLinkItem)) ) { // HACK to check for virtual collections
-    emit q->status( AgentBase::Running, i18nc( "@info:status", "Syncing collection '%1'", currentCollection.name() ) );
+    if ( mAutomaticProgressReporting ) {
+      emit q->status( AgentBase::Running, i18nc( "@info:status", "Syncing folder '%1'", currentCollection.name() ) );
+    }
     q->retrieveItems( currentCollection );
     return;
   }
   scheduler->taskDone();
 }
 
+void ResourceBasePrivate::slotSynchronizeCollectionAttributes( const Collection &col )
+{
+  Q_Q( ResourceBase );
+  QMetaObject::invokeMethod( q, "retrieveCollectionAttributes", Q_ARG( Akonadi::Collection, col ) );
+}
+
 void ResourceBasePrivate::slotPrepareItemRetrieval( const Akonadi::Item &item )
 {
   Q_Q( ResourceBase );
   ItemFetchJob *fetch = new ItemFetchJob( item, this );
   fetch->fetchScope().setAncestorRetrieval( q->changeRecorder()->itemFetchScope().ancestorRetrieval() );
   fetch->fetchScope().setCacheOnly( true );
+
+  // copy list of attributes to fetch
+  const QSet<QByteArray> attributes = q->changeRecorder()->itemFetchScope().attributes();
+  foreach ( const QByteArray &attribute, attributes )
+    fetch->fetchScope().fetchAttribute( attribute );
+
   q->connect( fetch, SIGNAL(result(KJob*)), SLOT(slotPrepareItemRetrievalResult(KJob*)) );
 }
 
@@ -485,7 +729,7 @@ void ResourceBasePrivate::slotPrepareItemRetrievalResult( KJob* job )
   }
   ItemFetchJob *fetch = qobject_cast<ItemFetchJob*>( job );
   if ( fetch->items().count() != 1 ) {
-    q->cancelTask( QLatin1String("The requested item does no longer exist") );
+    q->cancelTask( i18n( "The requested item no longer exists" ) );
     return;
   }
   const Item item = fetch->items().first();
@@ -513,6 +757,12 @@ void ResourceBase::clearCache()
   d->scheduler->scheduleResourceCollectionDeletion();
 }
 
+void ResourceBase::invalidateCache(const Collection& collection)
+{
+  Q_D( ResourceBase );
+  d->scheduler->scheduleCacheInvalidation( collection );
+}
+
 Collection ResourceBase::currentCollection() const
 {
   Q_D( const ResourceBase );
@@ -546,6 +796,20 @@ void ResourceBase::cancelTask()
     case ResourceScheduler::ChangeReplay:
       d->changeProcessed();
       break;
+    case ResourceScheduler::SyncCollectionTree:
+    case ResourceScheduler::SyncAll:
+      if ( d->mCollectionSyncer )
+        d->mCollectionSyncer->rollback();
+      else
+        d->scheduler->taskDone();
+      break;
+    case ResourceScheduler::SyncCollection:
+      if ( d->mItemSyncer ) {
+        d->mItemSyncer->rollback();
+      } else {
+        d->scheduler->taskDone();
+      }
+      break;
     default:
       d->scheduler->taskDone();
   }
@@ -571,19 +835,50 @@ void ResourceBase::doSetOnline( bool state )
 
 void ResourceBase::synchronizeCollection( qint64 collectionId )
 {
-  CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), CollectionFetchJob::Base );
+  synchronizeCollection( collectionId, false );
+}
+
+void ResourceBase::synchronizeCollection( qint64 collectionId, bool recursive )
+{
+  CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), recursive ? CollectionFetchJob::Recursive : CollectionFetchJob::Base );
+  job->setFetchScope( changeRecorder()->collectionFetchScope() );
   job->fetchScope().setResource( identifier() );
-  job->fetchScope().setAncestorRetrieval( changeRecorder()->collectionFetchScope().ancestorRetrieval() );
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( slotCollectionListDone( KJob* ) ) );
+  job->setProperty( "recursive", recursive );
+  connect( job, SIGNAL(result(KJob*)), SLOT(slotCollectionListDone(KJob*)) );
 }
 
 void ResourceBasePrivate::slotCollectionListDone( KJob *job )
+{
+  if ( !job->error() ) {
+    Collection::List list = static_cast<CollectionFetchJob*>( job )->collections();
+    if ( !list.isEmpty() ) {
+      if ( job->property( "recursive" ).toBool() ) {
+        Q_FOREACH ( const Collection &collection, list ) {
+          scheduler->scheduleSync( collection );
+        }
+      } else {
+        scheduler->scheduleSync( list.first() );
+      }
+    }
+  }
+  // TODO: error handling
+}
+
+void ResourceBase::synchronizeCollectionAttributes( qint64 collectionId )
+{
+  CollectionFetchJob* job = new CollectionFetchJob( Collection( collectionId ), CollectionFetchJob::Base );
+  job->setFetchScope( changeRecorder()->collectionFetchScope() );
+  job->fetchScope().setResource( identifier() );
+  connect( job, SIGNAL(result(KJob*)), SLOT(slotCollectionListForAttributesDone(KJob*)) );
+}
+
+void ResourceBasePrivate::slotCollectionListForAttributesDone( KJob *job )
 {
   if ( !job->error() ) {
     Collection::List list = static_cast<CollectionFetchJob*>( job )->collections();
     if ( !list.isEmpty() ) {
       Collection col = list.first();
-      scheduler->scheduleSync( col );
+      scheduler->scheduleAttributesSync( col );
     }
   }
   // TODO: error handling
@@ -594,66 +889,84 @@ void ResourceBase::setTotalItems( int amount )
   kDebug() << amount;
   Q_D( ResourceBase );
   setItemStreamingEnabled( true );
-  d->mItemSyncer->setTotalItems( amount );
+  if ( d->mItemSyncer ) {
+    d->mItemSyncer->setTotalItems( amount );
+  }
 }
 
 void ResourceBase::setItemStreamingEnabled( bool enable )
 {
   Q_D( ResourceBase );
-  Q_ASSERT_X( d->scheduler->currentTask().type == ResourceScheduler::SyncCollection,
-              "ResourceBase::setItemStreamingEnabled()",
-              "Calling setItemStreamingEnabled() although no item retrieval is in progress" );
-  if ( !d->mItemSyncer ) {
-    d->mItemSyncer = new ItemSync( currentCollection() );
-    connect( d->mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mItemSyncer, SIGNAL( result( KJob* ) ), SLOT( slotItemSyncDone( KJob* ) ) );
+  d->createItemSyncInstanceIfMissing();
+  if ( d->mItemSyncer ) {
+    d->mItemSyncer->setStreamingEnabled( enable );
   }
-  d->mItemSyncer->setStreamingEnabled( enable );
 }
 
 void ResourceBase::itemsRetrieved( const Item::List &items )
 {
   Q_D( ResourceBase );
-  Q_ASSERT_X( d->scheduler->currentTask().type == ResourceScheduler::SyncCollection,
-              "ResourceBase::itemsRetrieved()",
-              "Calling itemsRetrieved() although no item retrieval is in progress" );
-  if ( !d->mItemSyncer ) {
-    d->mItemSyncer = new ItemSync( currentCollection() );
-    connect( d->mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mItemSyncer, SIGNAL( result( KJob* ) ), SLOT( slotItemSyncDone( KJob* ) ) );
+  d->createItemSyncInstanceIfMissing();
+  if ( d->mItemSyncer ) {
+    d->mItemSyncer->setFullSyncItems( items );
   }
-  d->mItemSyncer->setFullSyncItems( items );
 }
 
-void ResourceBase::itemsRetrievedIncremental( const Item::List &changedItems, const Item::List &removedItems )
+void ResourceBase::itemsRetrievedIncremental( const Item::List &changedItems,
+                                              const Item::List &removedItems )
 {
   Q_D( ResourceBase );
-  Q_ASSERT_X( d->scheduler->currentTask().type == ResourceScheduler::SyncCollection,
-              "ResourceBase::itemsRetrievedIncremental()",
-              "Calling itemsRetrievedIncremental() although no item retrieval is in progress" );
-  if ( !d->mItemSyncer ) {
-    d->mItemSyncer = new ItemSync( currentCollection() );
-    connect( d->mItemSyncer, SIGNAL( percent( KJob*, unsigned long ) ), SLOT( slotPercent( KJob*, unsigned long ) ) );
-    connect( d->mItemSyncer, SIGNAL( result( KJob* ) ), SLOT( slotItemSyncDone( KJob* ) ) );
+  d->createItemSyncInstanceIfMissing();
+  if ( d->mItemSyncer ) {
+    d->mItemSyncer->setIncrementalSyncItems( changedItems, removedItems );
   }
-  d->mItemSyncer->setIncrementalSyncItems( changedItems, removedItems );
 }
 
 void ResourceBasePrivate::slotItemSyncDone( KJob *job )
 {
   mItemSyncer = 0;
   Q_Q( ResourceBase );
-  if ( job->error() ) {
+  if ( job->error() && job->error() != Job::UserCanceled ) {
     emit q->error( job->errorString() );
   }
   scheduler->taskDone();
 }
 
-void ResourceBasePrivate::slotPercent( KJob *job, unsigned long percent )
+
+void ResourceBasePrivate::slotDelayedEmitProgress()
 {
   Q_Q( ResourceBase );
-  Q_UNUSED( job );
-  emit q->percent( percent );
+  if ( mAutomaticProgressReporting ) {
+    emit q->percent( mUnemittedProgress );
+
+    Q_FOREACH( const QVariantMap &statusMap, mUnemittedAdvancedStatus ) {
+        emit q->advancedStatus( statusMap );
+    }
+  }
+  mUnemittedProgress = 0;
+  mUnemittedAdvancedStatus.clear();
+}
+
+void ResourceBasePrivate::slotPercent( KJob *job, unsigned long percent )
+{
+  mUnemittedProgress = percent;
+
+  const Collection collection = job->property( "collection" ).value<Collection>();
+  if ( collection.isValid() ) {
+    QVariantMap statusMap;
+    statusMap.insert( QLatin1String( "key" ), QString::fromLatin1( "collectionSyncProgress" ) );
+    statusMap.insert( QLatin1String( "collectionId" ), collection.id() );
+    statusMap.insert( QLatin1String( "percent" ), static_cast<unsigned int>( percent ) );
+
+    mUnemittedAdvancedStatus[collection.id()] = statusMap;
+  }
+  // deliver completion right away, intermediate progress at 1s intervals
+  if ( percent == 100 ) {
+    mProgressEmissionCompressor.stop();
+    slotDelayedEmitProgress();
+  } else if ( !mProgressEmissionCompressor.isActive() ) {
+    mProgressEmissionCompressor.start();
+  }
 }
 
 void ResourceBase::setHierarchicalRemoteIdentifiersEnabled( bool enable )
@@ -674,4 +987,47 @@ void ResourceBase::taskDone()
   d->scheduler->taskDone();
 }
 
+void ResourceBase::retrieveCollectionAttributes( const Collection &collection )
+{
+  collectionAttributesRetrieved( collection );
+}
+
+void Akonadi::ResourceBase::abortActivity()
+{
+
+}
+
+void ResourceBase::setItemTransactionMode(ItemSync::TransactionMode mode)
+{
+  Q_D( ResourceBase );
+  d->mItemTransactionMode = mode;
+}
+
+void ResourceBase::setItemSynchronizationFetchScope(const ItemFetchScope& fetchScope)
+{
+  Q_D( ResourceBase );
+  if ( !d->mItemSyncFetchScope )
+    d->mItemSyncFetchScope = new ItemFetchScope;
+  *(d->mItemSyncFetchScope) = fetchScope;
+}
+
+void ResourceBase::setAutomaticProgressReporting( bool enabled )
+{
+  Q_D( ResourceBase );
+  d->mAutomaticProgressReporting = enabled;
+}
+
+QString ResourceBase::dumpNotificationListToString() const
+{
+  Q_D( const ResourceBase );
+  return d->dumpNotificationListToString();
+}
+
+QString ResourceBase::dumpSchedulerToString() const
+{
+  Q_D( const ResourceBase );
+  return d->dumpToString();
+}
+
 #include "resourcebase.moc"
+#include "moc_resourcebase.cpp"
