@@ -28,6 +28,7 @@
 #include <akonadi/agentbase.h>
 #include <akonadi/collection.h>
 #include <akonadi/item.h>
+#include <akonadi/itemsync.h>
 
 class KJob;
 class Akonadi__ResourceAdaptor;
@@ -101,10 +102,11 @@ class ResourceBasePrivate;
  * - itemAdded()
  * - itemChanged()
  * - itemRemoved()
+ *
  * Note that these three functions don't get the full payload of the items by default,
  * you need to change the item fetch scope of the change recorder to fetch the full
  * payload. This can be expensive with big payloads, though.<br>
- * Once you have handled changes in these methods call changeCommitted().
+ * Once you have handled changes in these methods, call changeCommitted().
  * These methods are called whenever a local item related to this resource is
  * added, modified or deleted. They are only called if the resource is online, otherwise
  * all changes are recorded and replayed as soon the resource is online again.
@@ -196,6 +198,20 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     QString name() const;
 
+    /**
+     * Enable or disable automatic progress reporting. By default, it is enabled.
+     * When enabled, the resource will automatically emit the signals percent() and status()
+     * while syncing items or collections.
+     *
+     * The automatic progress reporting is done on a per item / per collection basis, so if a
+     * finer granularity is desired, automatic reporting should be disabled and the subclass should
+     * emit the percent() and status() signals itself.
+     *
+     * @param enabled Whether or not automatic emission of the signals is enabled.
+     * @since 4.7
+     */
+    void setAutomaticProgressReporting( bool enabled );
+
   Q_SIGNALS:
     /**
      * This signal is emitted whenever the name of the resource has changed.
@@ -209,6 +225,21 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void synchronized();
 
+    /**
+     * Emitted when a collection attributes synchronization has been completed.
+     *
+     * @param collectionId The identifier of the collection whose attributes got synchronized.
+     * @since 4.6
+     */
+    void attributesSynchronized( qlonglong collectionId );
+
+    /**
+     * Emitted when a collection tree synchronization has been completed.
+     *
+     * @since 4.8
+     */
+    void collectionTreeSynchronized();
+
   protected Q_SLOTS:
     /**
      * Retrieve the collection tree from the remote server and supply it via
@@ -217,6 +248,20 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     virtual void retrieveCollections() = 0;
 
+    /**
+     * Retrieve the attributes of a single collection from the backend. The
+     * collection to retrieve attributes for is provided as @p collection.
+     * Add the attributes parts and call collectionAttributesRetrieved()
+     * when done.
+     *
+     * @param collection The collection whose attributes should be retrieved.
+     * @see collectionAttributesRetrieved()
+     * @since 4.6
+     */
+    // KDE5: Make it pure virtual, for now can be called only by invokeMethod()
+    //       in order to simulate polymorphism
+    void retrieveCollectionAttributes( const Akonadi::Collection &collection );
+
     /**
      * Retrieve all (new/changed) items in collection @p collection.
      * It is recommended to use incremental retrieval if the backend supports that
@@ -243,6 +288,15 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     virtual bool retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts ) = 0;
 
+    /**
+     * Abort any activity in progress in the backend. By default this method does nothing.
+     *
+     * @since 4.6
+     */
+    // KDE5: Make it pure virtual, for now can be called only by invokeMethod()
+    //       in order to simulate polymorphism
+    void abortActivity();
+
   protected:
     /**
      * Creates a base resource.
@@ -263,6 +317,14 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void itemRetrieved( const Item &item );
 
+    /**
+     * Call this method from retrieveCollectionAttributes() once the result is available.
+     *
+     * @param item The collection whose attributes got retrieved.
+     * @since 4.6
+     */
+    void collectionAttributesRetrieved( const Collection &collection );
+
     /**
      * Resets the dirty flag of the given item and updates the remote id.
      *
@@ -346,6 +408,26 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void setItemStreamingEnabled( bool enable );
 
+    /**
+     * Set transaction mode for item sync'ing.
+     * @see Akonadi::ItemSync::TransactionMode
+     * @since 4.6
+     */
+    void setItemTransactionMode( ItemSync::TransactionMode mode );
+
+    /**
+     * Set the fetch scope applied for item synchronization.
+     * By default, the one set on the changeRecorder() is used. However, it can make sense
+     * to specify a specialized fetch scope for synchronization to improve performance.
+     * The rule of thumb is to remove anything from this fetch scope that does not provide
+     * additional information regarding whether and item has changed or not. This is primarily
+     * relevant for backends not supporting incremental retrieval.
+     * @param fetchScope The fetch scope to use by the internal Akonadi::ItemSync instance.
+     * @see Akonadi::ItemSync
+     * @since 4.6
+     */
+    void setItemSynchronizationFetchScope( const ItemFetchScope &fetchScope );
+
     /**
      * Call this method to supply incrementally retrieved items from the remote server.
      *
@@ -370,20 +452,34 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      * Call this method to remove all items and collections of the resource from the
      * server cache.
      *
-     * The method should be used whenever the configuration of the resource has changed
-     * and therefor the cached items might not be valid any longer.
+     * The method should not be used anymore
      *
+     * @see invalidateCache()
      * @since 4.3
      */
     void clearCache();
 
+    /**
+     * Call this method to invalidate all cached content in @p collection.
+     *
+     * The method should be used when the backend indicated that the cached content
+     * is no longer valid.
+     *
+     * @since 4.8
+     */
+    void invalidateCache( const Collection &collection );
+
     /**
      * Returns the collection that is currently synchronized.
+     * @note Calling this method is only allowed during a collection synchronization task, that
+     * is directly or indirectly from retrieveItems().
      */
     Collection currentCollection() const;
 
     /**
      * Returns the item that is currently retrieved.
+     * @note Calling this method is only allowed during fetching a single item, that
+     * is directly or indirectly from retrieveItem().
      */
     Item currentItem() const;
 
@@ -398,6 +494,22 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void synchronizeCollection( qint64 id );
 
+    /**
+     * This method is called whenever the collection with the given @p id
+     * shall be synchronized.
+     * @param recursive if true, a recursive synchronization is done
+     */
+    void synchronizeCollection( qint64 id, bool recursive );
+
+    /**
+     * This method is called whenever the collection with the given @p id
+     * shall have its attributes synchronized.
+     *
+     * @param id The id of the collection to synchronize
+     * @since 4.6
+     */
+    void synchronizeCollectionAttributes( qint64 id );
+
     /**
      * Refetches the Collections.
      */
@@ -446,6 +558,7 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
     void setHierarchicalRemoteIdentifiersEnabled( bool enable );
 
     friend class ResourceScheduler;
+    friend class ResourceState;
 
     /**
      * Describes the scheduling priority of a task that has been queued
@@ -455,9 +568,9 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      * @since 4.4
      */
     enum SchedulePriority {
-      Prepend,            ///> The task will be executed as soon as the current task has finished.
-      AfterChangeReplay,  ///> The task is scheduled after the last ChangeReplay task in the queue
-      Append              ///> The task will be executed after all tasks currently in the queue are finished
+      Prepend,            ///< The task will be executed as soon as the current task has finished.
+      AfterChangeReplay,  ///< The task is scheduled after the last ChangeReplay task in the queue
+      Append              ///< The task will be executed after all tasks currently in the queue are finished
     };
 
     /**
@@ -485,6 +598,18 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
      */
     void taskDone();
 
+    /**
+     * Dump the contents of the current ChangeReplay
+     * @since 4.8.1
+     */
+    QString dumpNotificationListToString() const;
+
+    /**
+     * Dump the state of the scheduler
+     * @since 4.8.1
+     */
+    QString dumpSchedulerToString() const;
+
   private:
     static QString parseArguments( int, char** );
     static int init( ResourceBase *r );
@@ -497,19 +622,26 @@ class AKONADI_EXPORT ResourceBase : public AgentBase
   private:
     Q_DECLARE_PRIVATE( ResourceBase )
 
+    Q_PRIVATE_SLOT( d_func(), void slotAbortRequested() )
     Q_PRIVATE_SLOT( d_func(), void slotDeliveryDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotCollectionSyncDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotDeleteResourceCollection() )
     Q_PRIVATE_SLOT( d_func(), void slotDeleteResourceCollectionDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotCollectionDeletionDone( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void slotInvalidateCache( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void slotLocalListDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotSynchronizeCollection( const Akonadi::Collection& ) )
     Q_PRIVATE_SLOT( d_func(), void slotCollectionListDone( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void slotSynchronizeCollectionAttributes( const Akonadi::Collection& ) )
+    Q_PRIVATE_SLOT( d_func(), void slotCollectionListForAttributesDone( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void slotCollectionAttributesSyncDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotItemSyncDone( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void slotPercent( KJob*, unsigned long ) )
+    Q_PRIVATE_SLOT( d_func(), void slotDelayedEmitProgress() )
     Q_PRIVATE_SLOT( d_func(), void slotPrepareItemRetrieval( const Akonadi::Item& item ) )
     Q_PRIVATE_SLOT( d_func(), void slotPrepareItemRetrievalResult( KJob* ) )
     Q_PRIVATE_SLOT( d_func(), void changeCommittedResult( KJob* ) )
+    Q_PRIVATE_SLOT( d_func(), void slotSessionReconnected() )
 };
 
 }
