@@ -17,18 +17,24 @@
     02110-1301, USA.
 */
 
+#include "collectionjobtest.h"
+
 #include <sys/types.h>
 
-#include "collectionjobtest.h"
-#include <qtest_kde.h>
+#include <qtest_akonadi.h>
+#include "test_utils.h"
+#include "testattribute.h"
 
+#include "agentmanager.h"
+#include "agentinstance.h"
+#include "attributefactory.h"
 #include "cachepolicy.h"
 #include "collection.h"
 #include "collectioncreatejob.h"
 #include "collectiondeletejob.h"
 #include "collectionfetchjob.h"
 #include "collectionmodifyjob.h"
-#include "collectionselectjob.h"
+#include "collectionselectjob_p.h"
 #include "collectionstatistics.h"
 #include "collectionstatisticsjob.h"
 #include "collectionpathresolver_p.h"
@@ -36,20 +42,22 @@
 #include "control.h"
 #include "item.h"
 #include "kmime/messageparts.h"
-
-#include <QDBusConnection>
-#include <QDBusConnectionInterface>
-
-
+#include "resourceselectjob_p.h"
+#include "collectionfetchscope.h"
 
 using namespace Akonadi;
 
-QTEST_KDEMAIN( CollectionJobTest, NoGUI )
+QTEST_AKONADIMAIN( CollectionJobTest, NoGUI )
 
 void CollectionJobTest::initTestCase()
 {
   qRegisterMetaType<Akonadi::Collection::List>();
+  AttributeFactory::registerAttribute<TestAttribute>();
   Control::start();
+
+  // switch all resources offline to reduce interference from them
+  foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() ) //krazy:exclude=foreach
+    agent.setIsOnline( false );
 }
 
 static Collection findCol( const Collection::List &list, const QString &name ) {
@@ -99,8 +107,8 @@ void CollectionJobTest::testTopLevelList( )
   res1ColId = col.id(); // for the next test
   QVERIFY( res1ColId > 0 );
   QVERIFY( CollectionUtils::isResource( col ) );
-  QCOMPARE( col.parent(), Collection::root().id() );
-  QCOMPARE( col.resource(), QLatin1String("akonadi_dummy_resource_1") );
+  QCOMPARE( col.parentCollection(), Collection::root() );
+  QCOMPARE( col.resource(), QLatin1String("akonadi_knut_resource_0") );
 
   QVERIFY( findCol( list, "res2" ).isValid() );
   res2ColId = findCol( list, "res2" ).id();
@@ -143,13 +151,14 @@ void CollectionJobTest::testFolderList( )
 
   col = findCol( list, "foo" );
   QVERIFY( col.isValid() );
-  QCOMPARE( col.parent(), res1ColId );
+  QCOMPARE( col.parentCollection().id(), res1ColId );
   QVERIFY( CollectionUtils::isFolder( col ) );
-  contentTypes << "message/rfc822" << "text/calendar" << "text/vcard" << "application/octet-stream";
+  contentTypes << "message/rfc822" << "text/calendar" << "text/directory"
+               << "application/octet-stream" << "inode/directory";
   compareLists( col.contentMimeTypes(), contentTypes );
 
   QVERIFY( findCol( list, "bar" ).isValid() );
-  QCOMPARE( findCol( list, "bar" ).parent(), col.id() );
+  QCOMPARE( findCol( list, "bar" ).parentCollection(), col );
   QVERIFY( findCol( list, "bla" ).isValid() );
 }
 
@@ -178,25 +187,19 @@ void CollectionJobTest::testSearchFolderList( )
   QVERIFY( job->exec() );
   Collection::List list = job->collections();
 
-  QCOMPARE( list.count(), 3 );
-  Collection col = findCol( list, "Test ?er" );
-  QVERIFY( col.isValid() );
-  QVERIFY( CollectionUtils::isVirtual( col ) );
-  QCOMPARE( col.resource(), QLatin1String("akonadi_search_resource" ) );
-  QVERIFY( findCol( list, "all" ).isValid() );
-  QVERIFY( findCol( list, "kde-core-devel" ).isValid() );
+  QCOMPARE( list.count(), 0 );
 }
 
 void CollectionJobTest::testResourceFolderList()
 {
   // non-existing resource
   CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::FirstLevel );
-  job->setResource( "i_dont_exist" );
+  job->fetchScope().setResource( "i_dont_exist" );
   QVERIFY( !job->exec() );
 
   // recursive listing of all collections of an existing resource
   job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive );
-  job->setResource( "akonadi_dummy_resource_1" );
+  job->fetchScope().setResource( "akonadi_knut_resource_0" );
   QVERIFY( job->exec() );
 
   Collection::List list = job->collections();
@@ -209,7 +212,7 @@ void CollectionJobTest::testResourceFolderList()
 
   // limited listing of a resource
   job = new CollectionFetchJob( Collection( fooId ), CollectionFetchJob::Recursive );
-  job->setResource( "akonadi_dummy_resource_1" );
+  job->fetchScope().setResource( "akonadi_knut_resource_0" );
   QVERIFY( job->exec() );
 
   list = job->collections();
@@ -218,6 +221,33 @@ void CollectionJobTest::testResourceFolderList()
   QVERIFY( findCol( list, "bla" ).isValid() );
 }
 
+void CollectionJobTest::testMimeTypeFilter()
+{
+  CollectionFetchJob *job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive );
+  job->fetchScope().setContentMimeTypes( QStringList() << "message/rfc822" );
+  AKVERIFYEXEC( job );
+
+  Collection::List list = job->collections();
+  QCOMPARE( list.count(), 2 );
+  QVERIFY( findCol( list, "res1" ).isValid() );
+  QVERIFY( findCol( list, "foo" ).isValid() );
+  int fooId = findCol( list, "foo" ).id();
+
+  // limited listing of a resource
+  job = new CollectionFetchJob( Collection( fooId ), CollectionFetchJob::Recursive );
+  job->fetchScope().setContentMimeTypes( QStringList() << "message/rfc822" );
+  AKVERIFYEXEC( job );
+
+  list = job->collections();
+  QCOMPARE( list.count(), 0 );
+
+  // non-existing mimetype
+  job = new CollectionFetchJob( Collection::root(), CollectionFetchJob::Recursive, this );
+  job->fetchScope().setContentMimeTypes( QStringList() << "something/non-existing" );
+  AKVERIFYEXEC( job );
+  QCOMPARE( job->collections().size(), 0 );
+}
+
 void CollectionJobTest::testCreateDeleteFolder_data()
 {
   QTest::addColumn<Collection>("collection");
@@ -226,10 +256,10 @@ void CollectionJobTest::testCreateDeleteFolder_data()
   Collection col;
   QTest::newRow("empty") << col << false;
   col.setName( "new folder" );
-  col.setParent( res3ColId );
+  col.parentCollection().setId( res3ColId );
   QTest::newRow("simple") << col << true;
 
-  col.setParent( res3ColId );
+  col.parentCollection().setId( res3ColId );
   col.setName( "foo" );
   QTest::newRow( "existing in different resource" ) << col << true;
 
@@ -249,25 +279,34 @@ void CollectionJobTest::testCreateDeleteFolder_data()
 
   col = Collection();
   col.setName( "New Folder" );
-  col.setParent( searchColId );
+  col.parentCollection().setId( searchColId );
   QTest::newRow( "search folder" ) << col << false;
 
-  col.setParent( res2ColId );
+  col.parentCollection().setId( res2ColId );
   col.setName( "foo2" );
   QTest::newRow( "already existing" ) << col << false;
 
+  col.parentCollection().setId( res2ColId ); // Sibling of collection 'foo2'
+  col.setName( "foo2 " );
+  QTest::newRow( "name of an sibling with an additional ending space" ) << col << true;
+
   col.setName( "Bla" );
-  col.setParent( 2 );
+  col.parentCollection().setId( 2 );
   QTest::newRow( "already existing with different case" ) << col << true;
 
   CollectionPathResolver *resolver = new CollectionPathResolver( "res2/foo2", this );
   QVERIFY( resolver->exec() );
-  col.setParent( resolver->collection() );
+  col.parentCollection().setId( resolver->collection() );
   col.setName( "new folder" );
   QTest::newRow( "parent noinferior" ) << col << false;
 
-  col.setParent( INT_MAX );
+  col.parentCollection().setId( INT_MAX );
   QTest::newRow( "missing parent" ) << col << false;
+
+  col = Collection();
+  col.setName( "rid parent" );
+  col.parentCollection().setRemoteId( "8" );
+  QTest::newRow( "rid parent" ) << col << false; // missing resource context
 }
 
 void CollectionJobTest::testCreateDeleteFolder()
@@ -283,12 +322,12 @@ void CollectionJobTest::testCreateDeleteFolder()
   Collection createdCol = createJob->collection();
   QVERIFY( createdCol.isValid() );
   QCOMPARE( createdCol.name(), collection.name() );
-  QCOMPARE( createdCol.parent(), collection.parent() );
+  QCOMPARE( createdCol.parentCollection(), collection.parentCollection() );
   QCOMPARE( createdCol.remoteId(), collection.remoteId() );
   QCOMPARE( createdCol.cachePolicy(), collection.cachePolicy() );
 
-  CollectionFetchJob *listJob = new CollectionFetchJob( Collection( collection.parent() ), CollectionFetchJob::FirstLevel, this );
-  QVERIFY( listJob->exec() );
+  CollectionFetchJob *listJob = new CollectionFetchJob( collection.parentCollection(), CollectionFetchJob::FirstLevel, this );
+  AKVERIFYEXEC( listJob );
   Collection listedCol = findCol( listJob->collections(), collection.name() );
   QCOMPARE( listedCol, createdCol );
   QCOMPARE( listedCol.remoteId(), collection.remoteId() );
@@ -296,9 +335,9 @@ void CollectionJobTest::testCreateDeleteFolder()
 
   // fetch parent to compare inherited collection properties
   Collection parentCol = Collection::root();
-  if ( collection.parent() > 0 ) {
-    CollectionFetchJob *listJob = new CollectionFetchJob( Collection( collection.parent() ), CollectionFetchJob::Base, this );
-    QVERIFY( listJob->exec() );
+  if ( collection.parentCollection().isValid() ) {
+    CollectionFetchJob *listJob = new CollectionFetchJob( collection.parentCollection(), CollectionFetchJob::Base, this );
+    AKVERIFYEXEC( listJob );
     QCOMPARE( listJob->collections().count(), 1 );
     parentCol = listJob->collections().first();
   }
@@ -314,10 +353,10 @@ void CollectionJobTest::testCreateDeleteFolder()
     QCOMPARE( listedCol.resource(), collection.resource() );
 
   CollectionDeleteJob *delJob = new CollectionDeleteJob( createdCol, this );
-  QVERIFY( delJob->exec() );
+  AKVERIFYEXEC( delJob );
 
-  listJob = new CollectionFetchJob( Collection( collection.parent() ), CollectionFetchJob::FirstLevel, this );
-  QVERIFY( listJob->exec() );
+  listJob = new CollectionFetchJob( collection.parentCollection(), CollectionFetchJob::FirstLevel, this );
+  AKVERIFYEXEC( listJob );
   QVERIFY( !findCol( listJob->collections(), collection.name() ).isValid() );
 }
 
@@ -354,102 +393,116 @@ void CollectionJobTest::testStatistics()
   QCOMPARE( s.unreadCount(), 14ll );
 }
 
+void CollectionJobTest::testModify_data()
+{
+  QTest::addColumn<qint64>( "uid" );
+  QTest::addColumn<QString>( "rid" );
+
+  QTest::newRow( "uid" ) << collectionIdFromPath( "res1/foo" ) << QString();
+  QTest::newRow( "rid" ) << -1ll << QString( "10" );
+}
+
+#define RESET_COLLECTION_ID \
+  col.setId( uid ); \
+  if ( !rid.isEmpty() ) col.setRemoteId( rid )
+
 void CollectionJobTest::testModify()
 {
+  QFETCH( qint64, uid );
+  QFETCH( QString, rid );
+
+  if ( !rid.isEmpty() ) {
+    ResourceSelectJob *rjob = new ResourceSelectJob( "akonadi_knut_resource_0" );
+    AKVERIFYEXEC( rjob );
+  }
+
   QStringList reference;
-  reference << "text/calendar" << "text/vcard" << "message/rfc822" << "application/octet-stream";
+  reference << "text/calendar" << "text/directory" << "message/rfc822" << "application/octet-stream" << "inode/directory";
 
   Collection col;
-  CollectionFetchJob *ljob = new CollectionFetchJob( Collection( res1ColId ), CollectionFetchJob::FirstLevel );
-  QVERIFY( ljob->exec() );
-  col = findCol( ljob->collections(), "foo" );
-  QVERIFY( col.isValid() );
+  RESET_COLLECTION_ID;
 
   // test noop modify
   CollectionModifyJob *mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
+  AKVERIFYEXEC( mod );
 
-  ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
-  QVERIFY( ljob->exec() );
+  CollectionFetchJob* ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
+  AKVERIFYEXEC( ljob );
   QCOMPARE( ljob->collections().count(), 1 );
   col = ljob->collections().first();
   compareLists( col.contentMimeTypes(), reference );
 
   // test clearing content types
+  RESET_COLLECTION_ID;
   col.setContentMimeTypes( QStringList() );
   mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
+  AKVERIFYEXEC( mod );
 
   ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
-  QVERIFY( ljob->exec() );
+  AKVERIFYEXEC( ljob );
   QCOMPARE( ljob->collections().count(), 1 );
   col = ljob->collections().first();
   QVERIFY( col.contentMimeTypes().isEmpty() );
 
   // test setting contnet types
+  RESET_COLLECTION_ID;
   col.setContentMimeTypes( reference );
   mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
+  AKVERIFYEXEC( mod );
 
   ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
-  QVERIFY( ljob->exec() );
+  AKVERIFYEXEC( ljob );
   QCOMPARE( ljob->collections().count(), 1 );
   col = ljob->collections().first();
   compareLists( col.contentMimeTypes(), reference );
 
-  // renaming
-  col.setName( "foo (renamed)" );
+  // add attribute
+  RESET_COLLECTION_ID;
+  col.attribute<TestAttribute>( Collection::AddIfMissing )->data = "new";
   mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
+  AKVERIFYEXEC( mod );
 
   ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
-  QVERIFY( ljob->exec() );
+  AKVERIFYEXEC( ljob );
+  QVERIFY( ljob->collections().first().hasAttribute<TestAttribute>() );
+  QCOMPARE( ljob->collections().first().attribute<TestAttribute>()->data, QByteArray( "new" ) );
+
+  // modify existing attribute
+  RESET_COLLECTION_ID;
+  col.attribute<TestAttribute>()->data = "modified";
+  mod = new CollectionModifyJob( col, this );
+  AKVERIFYEXEC( mod );
+
+  ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
+  AKVERIFYEXEC( ljob );
+  QVERIFY( ljob->collections().first().hasAttribute<TestAttribute>() );
+  QCOMPARE( ljob->collections().first().attribute<TestAttribute>()->data, QByteArray( "modified" ) );
+
+  // renaming
+  RESET_COLLECTION_ID;
+  col.setName( "foo (renamed)" );
+  mod = new CollectionModifyJob( col, this );
+  AKVERIFYEXEC( mod );
+
+  ljob = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
+  AKVERIFYEXEC( ljob );
   QCOMPARE( ljob->collections().count(), 1 );
   col = ljob->collections().first();
   QCOMPARE( col.name(), QString( "foo (renamed)" ) );
 
+  RESET_COLLECTION_ID;
   col.setName( "foo" );
   mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
+  AKVERIFYEXEC( mod );
 }
 
-void CollectionJobTest::testMove()
-{
-  Collection col( res1ColId );
-  col.setParent( res2ColId );
-  CollectionModifyJob *mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
-
-  CollectionFetchJob *ljob = new CollectionFetchJob( Collection( res2ColId ), CollectionFetchJob::Recursive );
-  QVERIFY( ljob->exec() );
-  Collection::List list = ljob->collections();
-
-  QCOMPARE( list.count(), 7 );
-  QVERIFY( findCol( list, "res1" ).isValid() );
-  QVERIFY( findCol( list, "foo" ).isValid() );
-  QVERIFY( findCol( list, "bar" ).isValid() );
-  QVERIFY( findCol( list, "bla" ).isValid() );
-
-  ljob = new CollectionFetchJob( Collection( res1ColId ), CollectionFetchJob::Base );
-  QVERIFY( ljob->exec() );
-  list = ljob->collections();
-
-  QCOMPARE( list.count(), 1 );
-  col = list.first();
-  QCOMPARE( col.name(), QLatin1String("res1") );
-  QCOMPARE( col.parent(), res2ColId );
-
-  // cleanup
-  col.setParent( Collection::root() );
-  mod = new CollectionModifyJob( col, this );
-  QVERIFY( mod->exec() );
-}
+#undef RESET_COLLECTION_ID
 
 void CollectionJobTest::testIllegalModify()
 {
   // non-existing collection
   Collection col( INT_MAX );
-  col.setParent( res1ColId );
+  col.parentCollection().setId( res1ColId );
   CollectionModifyJob *mod = new CollectionModifyJob( col, this );
   QVERIFY( !mod->exec() );
 
@@ -458,44 +511,42 @@ void CollectionJobTest::testIllegalModify()
   col.setName( "res2" );
   mod = new CollectionModifyJob( col, this );
   QVERIFY( !mod->exec() );
+}
 
-  // move to non-existing target
-  col = Collection( res1ColId );
-  col.setParent( INT_MAX );
-  mod = new CollectionModifyJob( col, this );
-  QVERIFY( !mod->exec() );
+void CollectionJobTest::testUtf8CollectionName_data()
+{
+  QTest::addColumn<QString>( "folderName" );
 
-  // moving root
-  col = Collection::root();
-  col.setParent( res1ColId );
-  mod = new CollectionModifyJob( col, this );
-  QVERIFY( !mod->exec() );
+  QTest::newRow( "Umlaut" ) << QString::fromUtf8( "ä" );
+  QTest::newRow( "Garbage" ) << QString::fromUtf8( "đ→³}đþøæſð" );
+  QTest::newRow( "Utf8" ) << QString::fromUtf8( "日本語" );
 }
 
 void CollectionJobTest::testUtf8CollectionName()
 {
-  QString folderName = QString::fromUtf8( "ä" );
+  QFETCH( QString, folderName );
 
   // create collection
   Collection col;
-  col.setParent( res3ColId );
+  col.parentCollection().setId( res3ColId );
   col.setName( folderName );
   CollectionCreateJob *create = new CollectionCreateJob( col, this );
-  QVERIFY( create->exec() );
+  AKVERIFYEXEC( create );
   col = create->collection();
   QVERIFY( col.isValid() );
+  QCOMPARE( col.name(), folderName );
 
   // list parent
   CollectionFetchJob *list = new CollectionFetchJob( Collection( res3ColId ), CollectionFetchJob::Recursive, this );
   QVERIFY( list->exec() );
   QCOMPARE( list->collections().count(), 1 );
-  QCOMPARE( col, list->collections().first() );
-  QCOMPARE( col.name(), folderName );
+  QCOMPARE( list->collections().first(), col );
+  QCOMPARE( list->collections().first().name(), col.name() );
 
   // modify collection
   col.setContentMimeTypes( QStringList( "message/rfc822'" ) );
   CollectionModifyJob *modify = new CollectionModifyJob( col, this );
-  QVERIFY( modify->exec() );
+  AKVERIFYEXEC( modify );
 
   // collection statistics
   CollectionStatisticsJob *statistics = new CollectionStatisticsJob( col, this );
@@ -506,7 +557,7 @@ void CollectionJobTest::testUtf8CollectionName()
 
   // delete collection
   CollectionDeleteJob *del = new CollectionDeleteJob( col, this );
-  QVERIFY( del->exec() );
+  AKVERIFYEXEC( del );
 }
 
 void CollectionJobTest::testMultiList()
@@ -529,12 +580,9 @@ void CollectionJobTest::testSelect()
 
   CollectionSelectJob *job = new CollectionSelectJob( col, this );
   QVERIFY( job->exec() );
-  QCOMPARE( job->unseen(), -1 );
 
   job = new CollectionSelectJob( col, this );
-  job->setRetrieveStatus( true );
   QVERIFY( job->exec() );
-  QVERIFY( job->unseen() > -1 );
 
   job = new CollectionSelectJob( Collection::root(), this );
   QVERIFY( job->exec() );
@@ -543,4 +591,87 @@ void CollectionJobTest::testSelect()
   QVERIFY( !job->exec() );
 }
 
+void CollectionJobTest::testRidFetch()
+{
+  Collection col;
+  col.setRemoteId( "10" );
+
+  CollectionFetchJob *job = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
+  job->fetchScope().setResource( "akonadi_knut_resource_0" );
+  QVERIFY( job->exec() );
+  QCOMPARE( job->collections().count(), 1 );
+  col = job->collections().first();
+  QVERIFY( col.isValid() );
+  QCOMPARE( col.remoteId(), QString::fromLatin1( "10" ) );
+}
+
+void CollectionJobTest::testRidCreateDelete_data()
+{
+  QTest::addColumn<QString>( "remoteId" );
+  QTest::newRow( "ASCII" ) << QString::fromUtf8( "MY REMOTE ID" );
+  QTest::newRow( "LATIN1" ) << QString::fromUtf8( "MY REMÖTE ID" );
+  QTest::newRow( "UTF8" ) << QString::fromUtf8( "MY REMOTE 検索表" );
+}
+
+void CollectionJobTest::testRidCreateDelete()
+{
+  QFETCH( QString, remoteId );
+  Collection collection;
+  collection.setName( "rid create" );
+  collection.parentCollection().setRemoteId( "8" );
+  collection.setRemoteId( remoteId );
+
+  ResourceSelectJob *resSel = new ResourceSelectJob( "akonadi_knut_resource_2" );
+  AKVERIFYEXEC( resSel );
+
+  CollectionCreateJob *createJob = new CollectionCreateJob( collection, this );
+  AKVERIFYEXEC( createJob );
+
+  Collection createdCol = createJob->collection();
+  QVERIFY( createdCol.isValid() );
+  QCOMPARE( createdCol.name(), collection.name() );
+
+  CollectionFetchJob *listJob = new CollectionFetchJob( Collection( res3ColId ), CollectionFetchJob::FirstLevel, this );
+  AKVERIFYEXEC( listJob );
+  Collection listedCol = findCol( listJob->collections(), collection.name() );
+  QCOMPARE( listedCol, createdCol );
+  QCOMPARE( listedCol.name(), collection.name() );
+
+  QVERIFY( !collection.isValid() );
+  CollectionDeleteJob *delJob = new CollectionDeleteJob( collection, this );
+  AKVERIFYEXEC( delJob );
+
+  listJob = new CollectionFetchJob( Collection( res3ColId ), CollectionFetchJob::FirstLevel, this );
+  AKVERIFYEXEC( listJob );
+  QVERIFY( !findCol( listJob->collections(), collection.name() ).isValid() );
+}
+
+void CollectionJobTest::testAncestorRetrieval()
+{
+  Collection col;
+  col.setRemoteId( "10" );
+
+  CollectionFetchJob *job = new CollectionFetchJob( col, CollectionFetchJob::Base, this );
+  job->fetchScope().setResource( "akonadi_knut_resource_0" );
+  job->fetchScope().setAncestorRetrieval( CollectionFetchScope::All );
+  AKVERIFYEXEC( job );
+  QCOMPARE( job->collections().count(), 1 );
+  col = job->collections().first();
+  QVERIFY( col.isValid() );
+  QVERIFY( col.parentCollection().isValid() );
+  QCOMPARE( col.parentCollection().remoteId(), QString( "6" ) );
+  QCOMPARE( col.parentCollection().parentCollection(), Collection::root() );
+
+  ResourceSelectJob* select = new ResourceSelectJob( "akonadi_knut_resource_0", this );
+  AKVERIFYEXEC( select );
+  Collection col2( col );
+  col2.setId( -1 ); // make it invalid but keep the ancestor chain
+  job = new CollectionFetchJob( col2, CollectionFetchJob::Base, this );
+  AKVERIFYEXEC( job );
+  QCOMPARE( job->collections().count(), 1 );
+  col2 = job->collections().first();
+  QVERIFY( col2.isValid() );
+  QCOMPARE( col, col2 );
+}
+
 #include "collectionjobtest.moc"
