@@ -20,6 +20,7 @@
 #include <akonadi/agentinstance.h>
 #include <akonadi/agentmanager.h>
 #include <akonadi/collection.h>
+#include <akonadi/collectionstatistics.h>
 #include <akonadi/control.h>
 #include <akonadi/itemcopyjob.h>
 #include <akonadi/itemfetchjob.h>
@@ -40,7 +41,7 @@ class ItemCopyTest : public QObject
     {
       Control::start();
       // switch target resources offline to reduce interference from them
-      foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() ) {
+      foreach ( Akonadi::AgentInstance agent, Akonadi::AgentManager::self()->instances() ) { //krazy:exclude=foreach
         if ( agent.identifier() == "akonadi_knut_resource_2" )
           agent.setIsOnline( false );
       }
@@ -73,6 +74,8 @@ class ItemCopyTest : public QObject
       QCOMPARE( item.size(), source.size() );
       QCOMPARE( item.attributes().count(), 1 );
       QVERIFY( item.remoteId().isEmpty() );
+      QEXPECT_FAIL("", "statistics are not properly updated after copy", Abort);
+      QCOMPARE( target.statistics().count(), 1ll );
     }
 
     void testIlleagalCopy()
