@@ -32,6 +32,7 @@
 #include <akonadi/itemfetchjob.h>
 #include <akonadi/itemfetchscope.h>
 #include <akonadi/itemmodifyjob.h>
+#include <akonadi/itemmodifyjob_p.h>
 #include <akonadi/resourceselectjob_p.h>
 #include <qtest_akonadi.h>
 #include "test_utils.h"
@@ -151,6 +152,7 @@ void ItemStoreTest::testDataChange()
 
   ItemFetchJob *fjob = new ItemFetchJob( Item( 1 ) );
   fjob->fetchScope().fetchFullPayload();
+  fjob->fetchScope().setCacheOnly( true );
   AKVERIFYEXEC( fjob );
   QCOMPARE( fjob->items().count(), 1 );
   item = fjob->items()[0];
@@ -188,6 +190,8 @@ void ItemStoreTest::testRemoteId()
 
   item.setRemoteId( rid );
   ItemModifyJob *store = new ItemModifyJob( item, this );
+  store->disableRevisionCheck();
+  store->setIgnorePayload( true ); // we only want to update the remote id
   AKVERIFYEXEC( store );
 
   ItemFetchJob *fetch = new ItemFetchJob( item, this );
@@ -298,6 +302,7 @@ void ItemStoreTest::testRevisionCheck()
   // try to store second item with modifications (should be detected as a conflict)
   item2.attribute<TestAttribute>( Item::AddIfMissing )->data = "random stuff 2";
   ItemModifyJob *sjob2 = new ItemModifyJob( item2 );
+  sjob2->disableAutomaticConflictHandling();
   QVERIFY( !sjob2->exec() );
 
   // fetch same again
@@ -335,8 +340,8 @@ void ItemStoreTest::testModificationTime()
   item2 = fjob->items().first();
   QCOMPARE( initialDateTime, item2.modificationTime() );
 
-  // Lets wait half a second.
-  QTest::qWait( 500 );
+  // Lets wait at least a second, which is the resolution of mtime
+  QTest::qWait( 1000 );
 
   // Modify the item
   item.attribute<TestAttribute>( Item::AddIfMissing )->data = "extra";
