@@ -31,14 +31,13 @@ class ItemStoreTest : public QObject
     void testFlagChange();
     void testDataChange_data();
     void testDataChange();
-    void testItemMove();
-    void testIllegalItemMove();
     void testRemoteId_data();
     void testRemoteId();
     void testMultiPart();
     void testPartRemove();
     void testRevisionCheck();
     void testModificationTime();
+    void testRemoteIdRace();
 };
 
 #endif
