@@ -67,8 +67,8 @@ void MovableType::listRecentPosts( int number )
     args << QVariant( number );
     d->mXmlRpcClient->call(
       "metaWeblog.getRecentPosts", args,
-      this, SLOT(slotListRecentPosts(const QList<QVariant>&,const QVariant&)),
-      this, SLOT(slotError(int,const QString&,const QVariant&)),
+      this, SLOT(slotListRecentPosts(QList<QVariant>,QVariant)),
+      this, SLOT(slotError(int,QString,QVariant)),
       QVariant( number ) );
 }
 
@@ -82,8 +82,8 @@ void MovableType::listTrackBackPings( KBlog::BlogPost *post )
   d->mCallMap[ i ] = post;
   d->mXmlRpcClient->call(
     "mt.getTrackbackPings", args,
-    this, SLOT(slotListTrackbackPings(const QList<QVariant>&,const QVariant&)),
-    this, SLOT(slotError(int,const QString&,const QVariant&)),
+    this, SLOT(slotListTrackbackPings(QList<QVariant>,QVariant)),
+    this, SLOT(slotError(int,QString,QVariant)),
     QVariant( i ) );
 }
 
@@ -100,8 +100,8 @@ void MovableType::fetchPost( BlogPost *post )
       return;
     }
 
-    connect( this, SIGNAL( listedCategories( const QList<QMap<QString,QString> >& ) )
-        , this, SLOT( slotTriggerFetchPost() ) );
+    connect( this, SIGNAL(listedCategories(QList<QMap<QString,QString> >))
+        , this, SLOT(slotTriggerFetchPost()) );
     listCategories();
   } else {
     MetaWeblog::fetchPost( post );
@@ -121,7 +121,7 @@ void MovableType::createPost( BlogPost *post )
   if(d->mCategoriesList.isEmpty()&&!post->categories().isEmpty()){
     kDebug() << "No categories in the cache yet. Have to fetch them first.";
     d->mCreatePostCache << post;
-    connect(this,SIGNAL(listedCategories(const QList<QMap<QString,QString> >&)),
+    connect(this,SIGNAL(listedCategories(QList<QMap<QString,QString> >)),
             this,SLOT(slotTriggerCreatePost()));
     listCategories();
   }
@@ -156,7 +156,7 @@ void MovableType::modifyPost( BlogPost *post )
   if(d->mCategoriesList.isEmpty() && !post->categories().isEmpty()){
     kDebug() << "No categories in the cache yet. Have to fetch them first.";
     d->mModifyPostCache << post;
-    connect(this,SIGNAL(listedCategories(const QList<QMap<QString,QString> >&)),
+    connect(this,SIGNAL(listedCategories(QList<QMap<QString,QString> >)),
             this,SLOT(slotTriggerModifyPost()));
     listCategories();
   }
@@ -170,7 +170,7 @@ void MovableTypePrivate::slotTriggerCreatePost()
   kDebug();
   Q_Q( MovableType );
 
-  q->disconnect(q,SIGNAL(listedCategories(const QList<QMap<QString,QString> >&)),
+  q->disconnect(q,SIGNAL(listedCategories(QList<QMap<QString,QString> >)),
           q,SLOT(slotTriggerCreatePost()));
   // now we can recall createPost with the posts from the cache
   QList<BlogPost*>::Iterator it = mCreatePostCache.begin();
@@ -186,7 +186,7 @@ void MovableTypePrivate::slotTriggerModifyPost()
   kDebug();
   Q_Q( MovableType );
 
-  q->disconnect(q,SIGNAL(listedCategories(const QList<QMap<QString,QString> >&)),
+  q->disconnect(q,SIGNAL(listedCategories(QList<QMap<QString,QString> >)),
           q,SLOT(slotTriggerModifyPost()));
   // now we can recall createPost with the posts from the cache
   QList<BlogPost*>::Iterator it = mModifyPostCache.begin();
@@ -202,8 +202,8 @@ void MovableTypePrivate::slotTriggerFetchPost()
   kDebug();
   Q_Q( MovableType );
 
-  q->disconnect( q,SIGNAL( listedCategories( const QList<QMap<QString,QString> >& ) ),
-      q,SLOT( slotTriggerFetchPost() ) );
+  q->disconnect( q,SIGNAL(listedCategories(QList<QMap<QString,QString> >)),
+      q,SLOT(slotTriggerFetchPost()) );
   QList<BlogPost*>::Iterator it = mFetchPostCache.begin();
   QList<BlogPost*>::Iterator end = mFetchPostCache.end();
   for ( ; it!=end; it++ ) {
@@ -288,8 +288,8 @@ void MovableTypePrivate::slotFetchPost( const QList<QVariant> &result, const QVa
     mCallMap[ i ] = post;
     mXmlRpcClient->call(
       "mt.getPostCategories", args,
-      q, SLOT(slotGetPostCategories(const QList<QVariant>&,const QVariant&)),
-      q, SLOT(slotError(int, const QString&,const QVariant&)),
+      q, SLOT(slotGetPostCategories(QList<QVariant>,QVariant)),
+      q, SLOT(slotError(int,QString,QVariant)),
       QVariant( i ) );
   } else {
     kDebug() << "Emitting fetchedPost()";
@@ -360,8 +360,8 @@ void MovableTypePrivate::setPostCategories( BlogPost *post, bool publishAfterCat
 
   mXmlRpcClient->call(
     "mt.setPostCategories", args,
-    q, SLOT(slotSetPostCategories(const QList<QVariant>&,const QVariant&)),
-    q, SLOT(slotError(int, const QString&,const QVariant&)),
+    q, SLOT(slotSetPostCategories(QList<QVariant>,QVariant)),
+    q, SLOT(slotError(int,QString,QVariant)),
     QVariant( i ) );
 }
 
