@@ -266,7 +266,7 @@ void HtmlExport::createEventList( QTextStream *ts )
   *ts << "    <th>" << i18nc( "@title:column event description",
                               "Event" ) << "</th>" << endl;
   if ( d->mSettings->eventLocation() ) {
-    *ts << "    <th>" << i18nc( "@title:column event locatin",
+    *ts << "    <th>" << i18nc( "@title:column event location",
                                 "Location" ) << "</th>" << endl;
     ++columns;
   }
@@ -683,11 +683,11 @@ void HtmlExport::createFooter( QTextStream *ts )
   if ( !d->mSettings->eMail().isEmpty() ) {
     if ( !d->mSettings->name().isEmpty() ) {
       trailer += i18nc( "@info/plain page creator email link with name",
-                        "by <link url='mailto:%1'>%2</link>",
+                        "by <link url='mailto:%1'>%2</link> ",
                         d->mSettings->eMail(), d->mSettings->name() );
     } else {
       trailer += i18nc( "@info/plain page creator email link",
-                        "by <link url='mailto:%1'>%2</link>",
+                        "by <link url='mailto:%1'>%2</link> ",
                         d->mSettings->eMail(), d->mSettings->eMail() );
     }
   } else {
