@@ -41,9 +41,9 @@
 #include "journal.h"
 
 extern "C" {
-  #include <ical.h>
-  #include <icalparser.h>
-  #include <icalrestriction.h>
+  #include <libical/ical.h>
+  #include <libical/icalparser.h>
+  #include <libical/icalrestriction.h>
 }
 
 #include <QtCore/QString>
@@ -78,8 +78,9 @@ static QString quoteForParam( const QString &text )
 {
   QString tmp = text;
   tmp.remove( '"' );
-  if ( tmp.contains( ';' ) || tmp.contains( ':' ) || tmp.contains( ',' ) )
+  if ( tmp.contains( ';' ) || tmp.contains( ':' ) || tmp.contains( ',' ) ) {
     return tmp; // libical quotes in this case already, see icalparameter_as_ical_string()
+  }
   return QString::fromLatin1( "\"" ) + tmp + QString::fromLatin1( "\"" );
 }
 
@@ -487,6 +488,14 @@ void ICalFormatImpl::writeIncidence( icalcomponent *parent,
     icalcomponent_add_property( parent, icalproperty_new_class( secClass ) );
   }
 
+  // geo
+  if ( incidence->hasGeo() ) {
+    icalgeotype geo;
+    geo.lat = incidence->geoLatitude();
+    geo.lon = incidence->geoLongitude();
+    icalcomponent_add_property( parent, icalproperty_new_geo( geo ) );
+  }
+
   // priority
   if ( incidence->priority() > 0 ) { // 0 is undefined priority
     icalcomponent_add_property(
@@ -494,11 +503,10 @@ void ICalFormatImpl::writeIncidence( icalcomponent *parent,
   }
 
   // categories
-  QStringList categories = incidence->categories();
-  QStringList::const_iterator it;
-  for ( it = categories.constBegin(); it != categories.constEnd(); ++it ) {
+  QString categories = incidence->categories().join( "," );
+  if ( !categories.isEmpty() ) {
     icalcomponent_add_property(
-      parent, icalproperty_new_categories( (*it).toUtf8() ) );
+      parent, icalproperty_new_categories( categories.toUtf8() ) );
   }
 
   // related event
@@ -518,7 +526,7 @@ void ICalFormatImpl::writeIncidence( icalcomponent *parent,
   RecurrenceRule::List::ConstIterator exit;
   for ( exit = exrules.constBegin(); exit != exrules.constEnd(); ++exit ) {
     icalcomponent_add_property(
-      parent, icalproperty_new_rrule( writeRecurrenceRule( (*exit) ) ) );
+      parent, icalproperty_new_exrule( writeRecurrenceRule( (*exit) ) ) );
   }
 
   DateList dateList = incidence->recurrence()->exDates();
@@ -620,7 +628,7 @@ void ICalFormatImpl::Private::writeCustomProperties( icalcomponent *parent,
     if ( !c.key().startsWith( "X-KDE" ) &&          //krazy:exclude=strings
          !c.key().startsWith( "X-LibKCal" ) &&      //krazy:exclude=strings
          !c.key().startsWith( "X-MICROSOFT" ) &&    //krazy:exclude=strings
-         !c.key().startsWith( "X-MOZILLA" ) &&      //krazy:exclude=strings
+         !c.key().startsWith( "X-MOZ" ) &&          //krazy:exclude=strings
          !c.key().startsWith( "X-PILOT" ) ) {       //krazy:exclude=strings
       // use text values for the typical X-FOO property.
       // except for vendor specific X-FOO properties.
@@ -759,6 +767,14 @@ icalproperty *ICalFormatImpl::writeAttendee( Attendee *attendee )
     icalproperty_add_parameter( p, icalparameter_delegator );
   }
 
+  const QMap<QByteArray, QString> customMap = attendee->customProperties().customProperties();
+  for ( QMap<QByteArray, QString>::ConstIterator cit = customMap.constBegin();
+        cit != customMap.constEnd(); ++cit ) {
+    icalparameter *icalparameter_x = icalparameter_new_x( cit.value().toUtf8() );
+    icalparameter_set_xname( icalparameter_x, cit.key() );
+    icalproperty_add_parameter( p, icalparameter_x );
+  }
+
   return p;
 }
 
@@ -768,7 +784,11 @@ icalproperty *ICalFormatImpl::writeAttachment( Attachment *att )
   if ( att->isUri() ) {
     attach = icalattach_new_from_url( att->uri().toUtf8().data() );
   } else {
+#ifdef USE_ICAL_0_46
+    attach = icalattach_new_from_data ( ( const char * )att->data(), 0, 0 );
+#else
     attach = icalattach_new_from_data ( ( unsigned char * )att->data(), 0, 0 );
+#endif
   }
   icalproperty *p = icalproperty_new_attach( attach );
 
@@ -847,49 +867,50 @@ icalrecurrencetype ICalFormatImpl::writeRecurrenceRule( RecurrenceRule *recur )
   bys = recur->bySeconds();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_second[index++] = *it;
+    r.by_second[index++] = static_cast<short>( *it );
   }
 
   bys = recur->byMinutes();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_minute[index++] = *it;
+    r.by_minute[index++] = static_cast<short>( *it );
   }
 
   bys = recur->byHours();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_hour[index++] = *it;
+    r.by_hour[index++] = static_cast<short>( *it );
   }
 
   bys = recur->byMonthDays();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_month_day[index++] = icalrecurrencetype_day_position( (*it) * 8 );
+    short dShort = static_cast<short>( ( *it ) * 8 );
+    r.by_month_day[index++] = static_cast<short>( icalrecurrencetype_day_position( dShort ) );
   }
 
   bys = recur->byYearDays();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_year_day[index++] = *it;
+    r.by_year_day[index++] = static_cast<short>( *it );
   }
 
   bys = recur->byWeekNumbers();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_week_no[index++] = *it;
+    r.by_week_no[index++] = static_cast<short>( *it );
   }
 
   bys = recur->byMonths();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_month[index++] = *it;
+    r.by_month[index++] = static_cast<short>( *it );
   }
 
   bys = recur->bySetPos();
   index = 0;
   for ( it = bys.constBegin(); it != bys.constEnd(); ++it ) {
-    r.by_set_pos[index++] = *it;
+    r.by_set_pos[index++] = static_cast<short>( *it );
   }
 
   QList<RecurrenceRule::WDayPos> byd = recur->byDays();
@@ -904,7 +925,7 @@ icalrecurrencetype ICalFormatImpl::writeRecurrenceRule( RecurrenceRule *recur )
     } else {
       day += (*dit).pos() * 8;
     }
-    r.by_day[index++] = day;
+    r.by_day[index++] = static_cast<short>( day );
   }
 
   r.week_start =
@@ -912,7 +933,7 @@ icalrecurrencetype ICalFormatImpl::writeRecurrenceRule( RecurrenceRule *recur )
 
   if ( recur->frequency() > 1 ) {
     // Dont' write out INTERVAL=1, because that's the default anyway
-    r.interval = recur->frequency();
+    r.interval = static_cast<short>( recur->frequency() );
   }
 
   if ( recur->duration() > 0 ) {
@@ -1046,8 +1067,6 @@ Todo *ICalFormatImpl::readTodo( icalcomponent *vtodo, ICalTimeZones *tzlist )
 
   icalproperty *p = icalcomponent_get_first_property( vtodo, ICAL_ANY_PROPERTY );
 
-  QStringList categories;
-
   while ( p ) {
     icalproperty_kind kind = icalproperty_isa(p);
     switch ( kind ) {
@@ -1108,9 +1127,6 @@ Event *ICalFormatImpl::readEvent( icalcomponent *vevent, ICalTimeZones *tzlist )
 
   icalproperty *p = icalcomponent_get_first_property( vevent, ICAL_ANY_PROPERTY );
 
-  QStringList categories;
-  icalproperty_transp transparency;
-
   bool dtEndProcessed = false;
 
   while ( p ) {
@@ -1142,13 +1158,15 @@ Event *ICalFormatImpl::readEvent( icalcomponent *vevent, ICalTimeZones *tzlist )
       break;
 
     case ICAL_TRANSP_PROPERTY:  // Transparency
-      transparency = icalproperty_get_transp( p );
+    {
+      icalproperty_transp transparency = icalproperty_get_transp( p );
       if ( transparency == ICAL_TRANSP_TRANSPARENT ) {
         event->setTransparency( Event::Transparent );
       } else {
         event->setTransparency( Event::Opaque );
       }
       break;
+    }
 
     default:
       // TODO: do something about unknown properties?
@@ -1187,10 +1205,10 @@ FreeBusy *ICalFormatImpl::readFreeBusy( icalcomponent *vfreebusy )
   icalproperty *p = icalcomponent_get_first_property( vfreebusy, ICAL_ANY_PROPERTY );
 
   FreeBusyPeriod::List periods;
-
   while ( p ) {
     icalproperty_kind kind = icalproperty_isa( p );
     switch ( kind ) {
+
     case ICAL_DTSTART_PROPERTY:  // start date and time (UTC)
       freebusy->setDtStart( readICalUtcDateTimeProperty( p ) );
       break;
@@ -1211,10 +1229,19 @@ FreeBusy *ICalFormatImpl::readFreeBusy( icalcomponent *vfreebusy )
         Duration duration ( readICalDuration( icalperiod.duration ) );
         period = FreeBusyPeriod( period_start, duration );
       }
-      QByteArray param = icalproperty_get_parameter_as_string( p, "X-SUMMARY" );
-      period.setSummary( QString::fromUtf8( KCodecs::base64Decode( param ) ) );
-      param = icalproperty_get_parameter_as_string( p, "X-LOCATION" );
-      period.setLocation( QString::fromUtf8( KCodecs::base64Decode( param ) ) );
+
+      icalparameter *param = icalproperty_get_first_parameter( p, ICAL_X_PARAMETER );
+      while ( param ) {
+        if ( strncmp( icalparameter_get_xname( param ), "X-SUMMARY", 9 ) == 0 ) {
+          period.setSummary( QString::fromUtf8(
+                               KCodecs::base64Decode( icalparameter_get_xvalue( param ) ) ) );
+        }
+        if ( strncmp( icalparameter_get_xname( param ), "X-LOCATION", 10 ) == 0 ) {
+          period.setLocation( QString::fromUtf8(
+                                KCodecs::base64Decode( icalparameter_get_xvalue( param ) ) ) );
+        }
+        param = icalproperty_get_next_parameter( p, ICAL_X_PARAMETER );
+      }
       periods.append( period );
       break;
     }
@@ -1250,7 +1277,7 @@ Attendee *ICalFormatImpl::readAttendee( icalproperty *attendee )
   icalparameter *p = 0;
 
   QString email = QString::fromUtf8( icalproperty_get_attendee( attendee ) );
-  if ( email.startsWith( "mailto:", Qt::CaseInsensitive ) ) {
+  if ( email.startsWith( QLatin1String( "mailto:" ), Qt::CaseInsensitive ) ) {
     email = email.mid( 7 );
   }
 
@@ -1323,16 +1350,20 @@ Attendee *ICalFormatImpl::readAttendee( icalproperty *attendee )
   }
 
   p = icalproperty_get_first_parameter( attendee, ICAL_X_PARAMETER );
+  QMap<QByteArray, QString> custom;
   while ( p ) {
     QString xname = QString( icalparameter_get_xname( p ) ).toUpper();
     QString xvalue = QString::fromUtf8( icalparameter_get_xvalue( p ) );
     if ( xname == "X-UID" ) {
       uid = xvalue;
+    } else {
+      custom[xname.toUtf8()] = xvalue;
     }
     p = icalproperty_get_next_parameter( attendee, ICAL_X_PARAMETER );
   }
 
   Attendee *a = new Attendee( name, email, rsvp, status, role, uid );
+  a->customProperties().setCustomProperties( custom );
 
   p = icalproperty_get_first_parameter( attendee, ICAL_DELEGATEDTO_PARAMETER );
   if ( p ) {
@@ -1350,7 +1381,7 @@ Attendee *ICalFormatImpl::readAttendee( icalproperty *attendee )
 Person ICalFormatImpl::readOrganizer( icalproperty *organizer )
 {
   QString email = QString::fromUtf8( icalproperty_get_organizer( organizer ) );
-  if ( email.startsWith( "mailto:", Qt::CaseInsensitive ) ) {
+  if ( email.startsWith( QLatin1String( "mailto:" ), Qt::CaseInsensitive ) ) {
     email = email.mid( 7 );
   }
   QString cn;
@@ -1431,8 +1462,8 @@ Attachment *ICalFormatImpl::readAttachment( icalproperty *attach )
 
     p = icalproperty_get_first_parameter( attach, ICAL_X_PARAMETER );
     while ( p ) {
-      if ( strncmp ( icalparameter_get_xname( p ), "X-LABEL", 7 ) == 0 ) {
-       attachment->setLabel( icalparameter_get_xvalue( p ) );
+      if ( strncmp( icalparameter_get_xname( p ), "X-LABEL", 7 ) == 0 ) {
+        attachment->setLabel( QString::fromUtf8( icalparameter_get_xvalue( p ) ) );
       }
       p = icalproperty_get_next_parameter( attach, ICAL_X_PARAMETER );
     }
@@ -1515,6 +1546,12 @@ void ICalFormatImpl::readIncidence( icalcomponent *parent,
 
     case ICAL_LOCATION_PROPERTY:  // location
     {
+      if ( !icalproperty_get_value( p ) ) {
+        //Fix for #191472. This is a pre-crash guard in case libical was
+        //compiled in superstrict mode (--enable-icalerrors-are-fatal)
+        //TODO: pre-crash guard other property getters too.
+        break;
+      }
       QString textStr = QString::fromUtf8( icalproperty_get_location( p ) );
       if ( !textStr.isEmpty() ) {
         QString valStr = QString::fromUtf8(
@@ -1572,6 +1609,15 @@ void ICalFormatImpl::readIncidence( icalcomponent *parent,
       break;
     }
 
+    case ICAL_GEO_PROPERTY:  // geo
+    {
+      icalgeotype geo = icalproperty_get_geo( p );
+      incidence->setGeoLatitude( geo.lat );
+      incidence->setGeoLongitude( geo.lon );
+      incidence->setHasGeo( true );
+      break;
+    }
+
     case ICAL_PRIORITY_PROPERTY:  // priority
       intvalue = icalproperty_get_priority( p );
       if ( d->mCompat ) {
@@ -1581,9 +1627,20 @@ void ICalFormatImpl::readIncidence( icalcomponent *parent,
       break;
 
     case ICAL_CATEGORIES_PROPERTY:  // categories
+    {
+      // We have always supported multiple CATEGORIES properties per component
+      // even though the RFC seems to indicate only 1 is permitted.
+      // We can't change that -- in order to retain backwards compatibility.
       text = icalproperty_get_categories( p );
-      categories.append( QString::fromUtf8( text ) );
+      const QString val = QString::fromUtf8( text );
+      foreach ( const QString &cat, val.split( ',', QString::SkipEmptyParts ) ) {
+        // ensure no duplicates
+        if ( !categories.contains( cat ) ) {
+          categories.append( cat );
+        }
+      }
       break;
+    }
 
     case ICAL_RRULE_PROPERTY:
       readRecurrenceRule( p, incidence );
@@ -1806,7 +1863,7 @@ void ICalFormatImpl::readRecurrence( const struct icalrecurrencetype &r,
   }
 
   // Week start setting
-  int wkst = ( r.week_start + 5 ) % 7 + 1;
+  short wkst = static_cast<short>( ( r.week_start + 5 ) % 7 + 1 );
   recur->setWeekStart( wkst );
 
   // And now all BY*
@@ -1842,10 +1899,10 @@ void ICalFormatImpl::readRecurrence( const struct icalrecurrencetype &r,
   // BYDAY is a special case, since it's not an int list
   QList<RecurrenceRule::WDayPos> wdlst;
   short day;
-  index=0;
+  index = 0;
   while ( ( day = r.by_day[index++] ) != ICAL_RECURRENCE_ARRAY_MAX ) {
     RecurrenceRule::WDayPos pos;
-    pos.setDay( ( icalrecurrencetype_day_day_of_week( day ) + 5 ) % 7 + 1 );
+    pos.setDay( static_cast<short>( ( icalrecurrencetype_day_day_of_week( day ) + 5 ) % 7 + 1 ) );
     pos.setPos( icalrecurrencetype_day_position( day ) );
     wdlst.append( pos );
   }
@@ -1960,7 +2017,7 @@ void ICalFormatImpl::readAlarm( icalcomponent *alarm,
     case ICAL_ATTENDEE_PROPERTY:
     { // Only in EMAIL alarm
       QString email = QString::fromUtf8( icalproperty_get_attendee( p ) );
-      if ( email.startsWith( "mailto:", Qt::CaseInsensitive ) ) {
+      if ( email.startsWith( QLatin1String( "mailto:" ), Qt::CaseInsensitive ) ) {
         email = email.mid( 7 );
       }
       QString name;
@@ -2247,6 +2304,10 @@ KDateTime ICalFormatImpl::readICalDateTimeProperty( icalproperty *p,
 
 icaldurationtype ICalFormatImpl::writeICalDuration( const Duration &duration )
 {
+  // should be able to use icaldurationtype_from_int(), except we know
+  // that some older tools do not properly support weeks. So we never
+  // set a week duration, only days
+
   icaldurationtype d;
 
   int value = duration.value();
@@ -2254,6 +2315,7 @@ icaldurationtype ICalFormatImpl::writeICalDuration( const Duration &duration )
   if ( value < 0 ) {
     value = -value;
   }
+
   if ( duration.isDaily() ) {
     d.weeks = 0;
     d.days  = value;
@@ -2306,12 +2368,26 @@ icalcomponent *ICalFormatImpl::createCalendarComponent( Calendar *cal )
   p = icalproperty_new_prodid( CalFormat::productId().toUtf8() );
   icalcomponent_add_property( calendar, p );
 
-  // TODO: Add time zone
-
   // iCalendar version (2.0)
   p = icalproperty_new_version( const_cast<char *>(_ICAL_VERSION) );
   icalcomponent_add_property( calendar, p );
 
+  // Add time zone
+  if ( cal && cal->timeZones() ) {
+    const ICalTimeZones::ZoneMap zmaps = cal->timeZones()->zones();
+    for ( ICalTimeZones::ZoneMap::ConstIterator it=zmaps.constBegin();
+          it != zmaps.constEnd(); ++it ) {
+      icaltimezone *icaltz = (*it).icalTimezone();
+      if ( !icaltz ) {
+        kError() << "bad time zone";
+      } else {
+        icalcomponent *tz = icalcomponent_new_clone( icaltimezone_get_component( icaltz ) );
+        icalcomponent_add_component( calendar, tz );
+        icaltimezone_free( icaltz, 1 );
+      }
+    }
+  }
+
   // Custom properties
   if( cal != 0 ) {
     d->writeCustomProperties( calendar, cal );
@@ -2354,16 +2430,24 @@ bool ICalFormatImpl::populate( Calendar *cal, icalcomponent *calendar )
     return false;
   } else {
     const char *version = icalproperty_get_version( p );
-
+    if ( !version ) {
+      kDebug() << "No VERSION property found";
+      d->mParent->setException( new ErrorFormat(
+                                  ErrorFormat::CalVersionUnknown,
+                                  i18n( "No VERSION property found" ) ) );
+      return false;
+    }
     if ( strcmp( version, "1.0" ) == 0 ) {
       kDebug() << "Expected iCalendar, got vCalendar";
-      d->mParent->setException(
-        new ErrorFormat( ErrorFormat::CalVersion1,
-                         i18n( "Expected iCalendar format" ) ) );
+      d->mParent->setException( new ErrorFormat(
+                                  ErrorFormat::CalVersion1,
+                                  i18n( "Expected iCalendar, got vCalendar format" ) ) );
       return false;
     } else if ( strcmp( version, "2.0" ) != 0 ) {
       kDebug() << "Expected iCalendar, got unknown format";
-      d->mParent->setException( new ErrorFormat( ErrorFormat::CalVersionUnknown ) );
+      d->mParent->setException( new ErrorFormat(
+                                  ErrorFormat::CalVersionUnknown,
+                                  i18n( "Expected iCalendar, got unknown format" ) ) );
       return false;
     }
   }
@@ -2384,6 +2468,8 @@ bool ICalFormatImpl::populate( Calendar *cal, icalcomponent *calendar )
   icalcomponent *c;
 
   // Iterate through all todos
+  cal->beginBatchAdding();
+
   c = icalcomponent_get_first_component( calendar, ICAL_VTODO_COMPONENT );
   while ( c ) {
     Todo *todo = readTodo( c, tzlist );
@@ -2427,6 +2513,8 @@ bool ICalFormatImpl::populate( Calendar *cal, icalcomponent *calendar )
     c = icalcomponent_get_next_component( calendar, ICAL_VJOURNAL_COMPONENT );
   }
 
+  cal->endBatchAdding();
+
   // Post-Process list of events with relations, put Event objects in relation
   Event::List::ConstIterator eIt;
   for ( eIt = d->mEventsRelate.constBegin(); eIt != d->mEventsRelate.constEnd(); ++eIt ) {
@@ -2461,7 +2549,7 @@ void ICalFormatImpl::dumpIcalRecurrence( icalrecurrencetype r )
 {
   int i;
 
-  kDebug() << " Freq:" << r.freq;
+  kDebug() << " Freq:" << int( r.freq );
   kDebug() << " Until:" << icaltime_as_ical_string( r.until );
   kDebug() << " Count:" << r.count;
   if ( r.by_day[0] != ICAL_RECURRENCE_ARRAY_MAX ) {
@@ -2512,9 +2600,91 @@ icalcomponent *ICalFormatImpl::createScheduleComponent( IncidenceBase *incidence
 {
   icalcomponent *message = createCalendarComponent();
 
+#if 0
+  //
+  // Old versions of Outlook 2003 cannot handle TZIDs. So rely on UTC times.
+  // Keep the code here in case we ever need it.
+  //
+
+  // Create VTIMEZONE components for this incidence
+  ICalTimeZones zones;
+  if ( incidence ) {
+    if ( incidence->type() == "Event" ) {
+      Event *ev = static_cast<Event *>( incidence );
+      if ( ev ) {
+        if ( ev->dtStart().isValid() ) {
+          zones.add( ICalTimeZone( ev->dtStart().timeZone() ) );
+        }
+        if ( ev->hasEndDate() && ev->dtEnd().isValid() ) {
+          zones.add( ICalTimeZone( ev->dtEnd().timeZone() ) );
+        }
+      }
+    } else if ( incidence->type() == "Todo" ) {
+      Todo *t = static_cast<Todo *>( incidence );
+      if ( t ) {
+        if ( t->hasStartDate() && t->dtStart().isValid() ) {
+          zones.add( ICalTimeZone( t->dtStart( true ).timeZone() ) );
+        }
+        if ( t->hasDueDate() && t->dtDue().isValid() ) {
+          zones.add( ICalTimeZone( t->dtDue().timeZone() ) );
+        }
+      }
+    } else if ( incidence->type() == "Journal" ) {
+      Journal *j = static_cast<Journal *>( incidence );
+      if ( j ) {
+        if ( j->dtStart().isValid() ) {
+          zones.add( ICalTimeZone( j->dtStart().timeZone() ) );
+        }
+      }
+    }
+    const ICalTimeZones::ZoneMap zmaps = zones.zones();
+    for ( ICalTimeZones::ZoneMap::ConstIterator it=zmaps.constBegin();
+          it != zmaps.constEnd(); ++it ) {
+      icaltimezone *icaltz = (*it).icalTimezone();
+      if ( !icaltz ) {
+        kError() << "bad time zone";
+      } else {
+        icalcomponent *tz = icalcomponent_new_clone( icaltimezone_get_component( icaltz ) );
+        icalcomponent_add_component( message, tz );
+        icaltimezone_free( icaltz, 1 );
+      }
+    }
+  }
+#else
+  if ( incidence ) {
+    if ( incidence->type() == "Event" ) {
+      Event *ev = static_cast<Event *>( incidence );
+      if ( ev ) {
+        if ( ev->dtStart().isValid() ) {
+          ev->setDtStart( ev->dtStart().toUtc() );
+        }
+        if ( ev->hasEndDate() && ev->dtEnd().isValid() ) {
+          ev->setDtEnd( ev->dtEnd().toUtc() );
+        }
+      }
+    } else if ( incidence->type() == "Todo" ) {
+      Todo *t = static_cast<Todo *>( incidence );
+      if ( t ) {
+        if ( t->hasStartDate() && t->dtStart().isValid() ) {
+          t->setDtStart( t->dtStart().toUtc() );
+        }
+        if ( t->hasDueDate() && t->dtDue().isValid() ) {
+          t->setDtDue( t->dtDue().toUtc() );
+        }
+      }
+    } else if ( incidence->type() == "Journal" ) {
+      Journal *j = static_cast<Journal *>( incidence );
+      if ( j ) {
+        if ( j->dtStart().isValid() ) {
+          j->setDtStart( j->dtStart().toUtc() );
+        }
+      }
+    }
+  }
+#endif
   icalproperty_method icalmethod = ICAL_METHOD_NONE;
 
-  switch (method) {
+  switch ( method ) {
   case iTIPPublish:
     icalmethod = ICAL_METHOD_PUBLISH;
     break;
