@@ -27,7 +27,7 @@
 #include <QtCore/QDebug>
 #include <QtCore/QTimer>
 
-#include <KDE/KMessageBox>
+#include <KDebug>
 #include <KDE/KLocale>
 
 #include "job.h"
@@ -47,7 +47,7 @@ Session::Session( const QString &hostName, quint16 port, QObject *parent)
   : QObject(parent), d(new SessionPrivate(this))
 {
   if ( !qgetenv( "KIMAP_LOGFILE" ).isEmpty() ) {
-    d->logger = SessionLogger::self();
+    d->logger = new SessionLogger;
   }
 
   d->isSocketConnected = false;
@@ -55,10 +55,11 @@ Session::Session( const QString &hostName, quint16 port, QObject *parent)
   d->jobRunning = false;
 
   d->thread = new SessionThread(hostName, port, this);
-  connect(d->thread, SIGNAL(encryptionNegotiationResult(bool, KTcpSocket::SslVersion)),
-          d, SLOT(onEncryptionNegotiationResult(bool, KTcpSocket::SslVersion)));
-  connect(d->thread, SIGNAL(sslError(const KSslErrorUiData&)), this, SLOT(handleSslError(const KSslErrorUiData&)));
+  connect(d->thread, SIGNAL(encryptionNegotiationResult(bool,KTcpSocket::SslVersion)),
+          d, SLOT(onEncryptionNegotiationResult(bool,KTcpSocket::SslVersion)));
+  connect(d->thread, SIGNAL(sslError(KSslErrorUiData)), this, SLOT(handleSslError(KSslErrorUiData)));
 
+  d->startSocketTimer();
   d->thread->start();
 }
 
@@ -92,6 +93,11 @@ Session::State Session::state() const
   return d->state;
 }
 
+QString Session::userName() const
+{
+    return d->userName;
+}
+
 QByteArray Session::serverGreeting() const
 {
   return d->greeting;
@@ -102,6 +108,11 @@ int Session::jobQueueSize() const
   return d->queue.size() + ( d->jobRunning ? 1 : 0 );
 }
 
+void KIMAP::Session::close()
+{
+  d->thread->closeSocket();
+}
+
 void SessionPrivate::handleSslError(const KSslErrorUiData& errorData)
 {
   if (uiProxy && uiProxy->ignoreSslError(errorData)) {
@@ -123,6 +134,11 @@ SessionPrivate::SessionPrivate( Session *session )
 {
 }
 
+SessionPrivate::~SessionPrivate()
+{
+  delete logger;
+}
+
 void SessionPrivate::addJob(Job *job)
 {
   queue.append(job);
@@ -198,8 +214,11 @@ void SessionPrivate::responseReceived( const Message &response )
 
   switch ( state ) {
   case Session::Disconnected:
+    if (socketTimer.isActive()) {
+      stopSocketTimer();
+    }
     if ( code=="OK" ) {
-      state = Session::NotAuthenticated;
+      setState(Session::NotAuthenticated);
 
       Message simplified = response;
       simplified.content.removeFirst(); // Strip the tag
@@ -208,7 +227,7 @@ void SessionPrivate::responseReceived( const Message &response )
 
       startNext();
     } else if ( code=="PREAUTH" ) {
-      state = Session::Authenticated;
+      setState(Session::Authenticated);
 
       Message simplified = response;
       simplified.content.removeFirst(); // Strip the tag
@@ -218,24 +237,23 @@ void SessionPrivate::responseReceived( const Message &response )
       startNext();
     } else {
       thread->closeSocket();
-      QTimer::singleShot( 1000, thread, SLOT( reconnect() ) );
     }
     return;
   case Session::NotAuthenticated:
     if ( code=="OK" && tag==authTag ) {
-      state = Session::Authenticated;
+      setState(Session::Authenticated);
     }
     break;
   case Session::Authenticated:
     if ( code=="OK" && tag==selectTag ) {
-      state = Session::Selected;
+      setState(Session::Selected);
       currentMailBox = upcomingMailBox;
     }
     break;
   case Session::Selected:
     if ( ( code=="OK" && tag==closeTag )
       || ( code!="OK" && tag==selectTag) ) {
-      state = Session::Authenticated;
+      setState(Session::Authenticated);
       currentMailBox = QByteArray();
     } else if ( code=="OK" && tag==selectTag ) {
       currentMailBox = upcomingMailBox;
@@ -258,6 +276,15 @@ void SessionPrivate::responseReceived( const Message &response )
   }
 }
 
+void SessionPrivate::setState(Session::State s)
+{
+  if (s != state) {
+    Session::State oldState = state;
+    state = s;
+    emit q->stateChanged(state, oldState);
+  }
+}
+
 QByteArray SessionPrivate::sendCommand( const QByteArray &command, const QByteArray &args )
 {
   QByteArray tag = 'A' + QByteArray::number(++tagCount).rightJustified(6, '0');
@@ -297,6 +324,7 @@ void SessionPrivate::sendData( const QByteArray &data )
 
 void SessionPrivate::socketConnected()
 {
+  stopSocketTimer();
   isSocketConnected = true;
 
   bool willUseSsl = false;
@@ -307,38 +335,72 @@ void SessionPrivate::socketConnected()
                 || ( login->encryptionMode() == KIMAP::LoginJob::SslV3 )
                 || ( login->encryptionMode() == KIMAP::LoginJob::SslV3_1 )
                 || ( login->encryptionMode() == KIMAP::LoginJob::AnySslVersion );
+
+      userName = login->userName();
     }
   }
 
   if ( state == Session::Disconnected && willUseSsl ) {
     startNext();
+  } else {
+    startSocketTimer();
   }
 }
 
 void SessionPrivate::socketDisconnected()
 {
+  if (socketTimer.isActive()) {
+    stopSocketTimer();
+  }
+
   if ( logger && ( state==Session::Authenticated || state==Session::Selected ) ) {
     logger->disconnectionOccured();
   }
 
-
-  if ( state==Session::Authenticated || state==Session::Selected ) {
+  if ( state != Session::Disconnected ) {
+    setState(Session::Disconnected);
     emit q->connectionLost();
+  } else {
+    emit q->connectionFailed();
   }
 
   isSocketConnected = false;
-  state = Session::Disconnected;
-  thread->closeSocket();
 
-  if ( currentJob ) {
-    currentJob->connectionLost();
-  }
+  clearJobQueue();
+}
+
+void SessionPrivate::socketActivity()
+{
+  restartSocketTimer();
 }
 
 void SessionPrivate::socketError()
 {
-  //qWarning() << "Socket error occurred:" << socket->errorString();
-  socketDisconnected();
+  if (socketTimer.isActive()) {
+    stopSocketTimer();
+  }
+
+  if ( isSocketConnected ) {
+    thread->closeSocket();
+  } else {
+    emit q->connectionFailed();
+    emit q->connectionLost();    // KDE5: Remove this. We shouldn't emit connectionLost() if we weren't connected in the first place
+    clearJobQueue();
+  }
+}
+
+void SessionPrivate::clearJobQueue()
+{
+  if ( currentJob ) {
+    currentJob->connectionLost();
+  } else if ( !queue.isEmpty() ) {
+    currentJob = queue.takeFirst();
+    currentJob->connectionLost();
+  }
+
+  qDeleteAll(queue);
+  queue.clear();
+  emit q->jobQueueSizeChanged( 0 );
 }
 
 void SessionPrivate::startSsl(const KTcpSocket::SslVersion &version)
@@ -405,7 +467,6 @@ void SessionPrivate::stopSocketTimer()
   if ( socketTimerInterval<0 ) {
     return;
   }
-  Q_ASSERT( socketTimer.isActive() );
 
   socketTimer.stop();
 
@@ -415,13 +476,21 @@ void SessionPrivate::stopSocketTimer()
 
 void SessionPrivate::restartSocketTimer()
 {
-  stopSocketTimer();
+  if ( socketTimer.isActive() ) {
+    stopSocketTimer();
+  }
   startSocketTimer();
 }
 
 void SessionPrivate::onSocketTimeout()
 {
-  socketDisconnected();
+  kDebug() << "Socket timeout!";
+  thread->closeSocket();
+}
+
+void Session::setTimeout( int timeout )
+{
+  d->setSocketTimeout( timeout * 1000 );
 }
 
 #include "session.moc"
