@@ -145,7 +145,7 @@ ssize_t POP3Protocol::myRead(void *data, ssize_t len)
     memcpy(data, readBuffer, copyLen);
     readBufferLen -= copyLen;
     if (readBufferLen)
-      memcpy(readBuffer, &readBuffer[copyLen], readBufferLen);
+      memmove(readBuffer, &readBuffer[copyLen], readBufferLen);
     return copyLen;
   }
   waitForResponse(600);
@@ -164,7 +164,7 @@ ssize_t POP3Protocol::myReadLine(char *data, ssize_t len)
       data[copyLen] = '\0';
       readBufferLen -= copyLen;
       if (readBufferLen)
-        memcpy(readBuffer, &readBuffer[copyLen], readBufferLen);
+        memmove(readBuffer, &readBuffer[copyLen], readBufferLen);
       return copyLen;
     }
     waitForResponse(600);
@@ -692,9 +692,8 @@ bool POP3Protocol::pop3_open()
         error(ERR_SLAVE_DEFINED,
               i18n("Your POP3 server claims to "
                    "support TLS but negotiation "
-                   "was unsuccessful. You can "
-                   "disable TLS in KDE using the "
-                   "crypto settings module."));
+                   "was unsuccessful.\nYou can "
+                   "disable TLS in the POP account settings dialog."));
         closeConnection();
         return false;
       }
@@ -890,7 +889,8 @@ void POP3Protocol::get(const KUrl & url)
         }
       } else {
         closeConnection();
-        error(ERR_COULD_NOT_READ, m_sError);
+        error( ERR_SLAVE_DEFINED, i18n( "Error during communication with the POP3 server while "
+                                        "trying to list mail: %1", m_sError ) );
         return;
       }
     }
@@ -900,9 +900,9 @@ void POP3Protocol::get(const KUrl & url)
     bool firstCommand = true;
     while (it != waitingCommands.end() || activeCommands > 0) {
       while (activeCommands < maxCommands && it != waitingCommands.end()) {
-        sendCommand(((cmd ==
-                      "headers") ? "TOP " + *it + " 0" : "RETR " +
-                     *it).toLatin1());
+        sendCommand(QString((cmd ==
+                      "headers") ? QString("TOP " + *it + " 0") : QString("RETR " +
+                     *it)).toLatin1());
         activeCommands++;
         it++;
       }
@@ -990,7 +990,8 @@ void POP3Protocol::get(const KUrl & url)
       } else {
         kDebug(7105) << "Could not login. Bad RETR Sorry";
         closeConnection();
-        error(ERR_COULD_NOT_READ, m_sError);
+        error( ERR_SLAVE_DEFINED, i18n( "Error during communication with the POP3 server while "
+                                        "trying to download mail: %1", m_sError ) );
         return;
       }
     }
