@@ -26,6 +26,7 @@
 #include "kmime_headerfactory_p.h"
 #include "kmime_headers.h"
 #include "kmime_util.h"
+#include "kmime_util_p.h"
 #include "kmime_dateformatter.h"
 #include "kmime_warning.h"
 
@@ -63,29 +64,35 @@ static QString addr_spec_as_string( const AddrSpec & as, bool pretty )
     return QString();
   }
 
+  static QChar dotChar = QLatin1Char( '.' );
+  static QChar backslashChar = QLatin1Char( '\\' );
+  static QChar quoteChar = QLatin1Char( '"' );
+
   bool needsQuotes = false;
   QString result;
   result.reserve( as.localPart.length() + as.domain.length() + 1 );
   for ( int i = 0 ; i < as.localPart.length() ; ++i ) {
-    const char ch = as.localPart[i].toLatin1();
-    if ( ch == '.' || isAText( ch ) ) {
+    const QChar ch = as.localPart.at( i );
+    if ( ch == dotChar || isAText( ch.toLatin1() ) ) {
       result += ch;
     } else {
       needsQuotes = true;
-      if ( ch == '\\' || ch == '"' ) {
-        result += '\\';
+      if ( ch == backslashChar || ch == quoteChar ) {
+        result += backslashChar;
       }
       result += ch;
     }
   }
   const QString dom = pretty ? QUrl_fromAce_wrapper( as.domain ) : as.domain ;
   if ( needsQuotes ) {
-    result = '"' + result + "\"";
+    result = quoteChar + result + quoteChar;
   }
   if( dom.isEmpty() ) {
     return result;
   } else {
-    return result + '@' + dom;
+    result += QLatin1Char( '@' );
+    result += dom;
+    return result;
   }
 }
 
@@ -106,7 +113,13 @@ bool AddrSpec::isEmpty() const
 
 QByteArray Mailbox::address() const
 {
-  return mAddrSpec.asString().toLatin1();
+  QByteArray result;
+  const QString asString = addr_spec_as_string( mAddrSpec, false );
+  if ( !asString.isEmpty() ) {
+    result = asString.toLatin1();
+  }
+  return result;
+  //return mAddrSpec.asString().toLatin1();
 }
 
 AddrSpec Mailbox::addrSpec() const
@@ -160,20 +173,29 @@ bool Mailbox::hasName() const
 }
 
 QString Mailbox::prettyAddress() const
+{
+  return prettyAddress( QuoteNever );
+}
+
+QString Mailbox::prettyAddress( Quoting quoting ) const
 {
   if ( !hasName() ) {
-    return address();
+    return QLatin1String( address() );
   }
   QString s = name();
+  if ( quoting != QuoteNever ) {
+    addQuotes( s, quoting == QuoteAlways /*bool force*/ );
+  }
+
   if ( hasAddress() ) {
-    s += QLatin1String(" <") + address() + QLatin1Char('>');
+    s += QLatin1String(" <") + QLatin1String( address() ) + QLatin1Char('>');
   }
   return s;
 }
 
 void Mailbox::fromUnicodeString( const QString &s )
 {
-  from7BitString( encodeRFC2047String( s, "utf-8", false ) );
+  from7BitString( encodeRFC2047Sentence( s, "utf-8" ) );
 }
 
 void Mailbox::from7BitString( const QByteArray &s )
@@ -345,12 +367,12 @@ bool parseEncodedWord( const char* &scursor, const char * const send,
   bool matchOK = false;
   QTextCodec *textCodec = 0;
   if ( forceCS || maybeCharset.isEmpty() ) {
-    textCodec = KGlobal::charsets()->codecForName( defaultCS, matchOK );
+    textCodec = KGlobal::charsets()->codecForName( QLatin1String( defaultCS ), matchOK );
     usedCS = cachedCharset( defaultCS );
   } else {
-    textCodec = KGlobal::charsets()->codecForName( maybeCharset, matchOK );
+    textCodec = KGlobal::charsets()->codecForName( QLatin1String( maybeCharset ), matchOK );
     if ( !matchOK ) {  //no suitable codec found => use default charset
-      textCodec = KGlobal::charsets()->codecForName( defaultCS, matchOK );
+      textCodec = KGlobal::charsets()->codecForName( QLatin1String( defaultCS ), matchOK );
       usedCS = cachedCharset( defaultCS );
     } else {
       usedCS = cachedCharset( maybeCharset );
@@ -441,6 +463,8 @@ bool parseAtom( const char * &scursor, const char * const send,
   return success;
 }
 
+// FIXME: Remove this and the other parseToken() method. add a new one where "result" is a
+//        QByteArray.
 bool parseToken( const char * &scursor, const char * const send,
                  QString &result, bool allow8Bit )
 {
@@ -493,6 +517,8 @@ bool parseToken( const char * &scursor, const char * const send,
 //
 // - doesn't handle quoted CRLF
 
+// FIXME: Why is result a QString? This should be a QByteArray, since at this level, we don't
+//        know about encodings yet!
 bool parseGenericQuotedString( const char* &scursor, const char * const send,
                                QString &result, bool isCRLF,
                                const char openChar, const char closeChar )
@@ -520,7 +546,7 @@ bool parseGenericQuotedString( const char* &scursor, const char * const send,
       // misses "\" CRLF LWSP-char handling, see rfc822, 3.4.5
       READ_ch_OR_FAIL;
       KMIME_WARN_IF_8BIT( ch );
-      result += QChar( ch );
+      result += QLatin1Char( ch );
       break;
     case '\r':
       // ###
@@ -534,7 +560,7 @@ bool parseGenericQuotedString( const char* &scursor, const char * const send,
       if ( ch != '\n' ) {
         // CR on it's own...
         KMIME_WARN_LONE( CR );
-        result += QChar('\r');
+        result += QLatin1Char('\r');
         scursor--; // points to after the '\r' again
       } else {
         // CRLF encountered.
@@ -544,13 +570,13 @@ bool parseGenericQuotedString( const char* &scursor, const char * const send,
           // correct folding;
           // position cursor behind the CRLF WSP (unfolding)
           // and add the WSP to the result
-          result += QChar( ch );
+          result += QLatin1Char( ch );
         } else {
           // this is the "shouldn't happen"-case. There is a CRLF
           // inside a quoted-string without it being part of FWS.
           // We take it verbatim.
           KMIME_WARN_NON_FOLDING( CRLF );
-          result += "\r\n";
+          result += QLatin1String( "\r\n" );
           // the cursor is decremented again, so's we need not
           // duplicate the whole switch here. "ch" could've been
           // everything (incl. openChar or closeChar).
@@ -570,19 +596,42 @@ bool parseGenericQuotedString( const char* &scursor, const char * const send,
       if ( !isCRLF && ( ch == ' ' || ch == '\t' ) ) {
         // folding
         // correct folding
-        result += QChar( ch );
+        result += QLatin1Char( ch );
       } else {
         // non-folding
         KMIME_WARN_LONE( LF );
-        result += QChar('\n');
+        result += QLatin1Char( '\n' );
         // pos is decremented, so's we need not duplicate the whole
         // switch here. ch could've been everything (incl. <">, "\").
         scursor--;
       }
       break;
+    case '=':
+    {
+      // ### Work around broken clients that send encoded words in quoted-strings
+      //     For example, older KMail versions.
+      if( scursor == send )
+        break;
+
+      const char *oldscursor = scursor;
+      QString tmp;
+      QByteArray lang, charset;
+      if( *scursor++ == '?' ) {
+        --scursor;
+        if( parseEncodedWord( scursor, send, tmp, lang, charset ) ) {
+          result += tmp;
+          break;
+        } else {
+          scursor = oldscursor;
+        }
+      } else {
+        scursor = oldscursor;
+      }
+      // fall through
+    }
     default:
       KMIME_WARN_IF_8BIT( ch );
-      result += QChar( ch );
+      result += QLatin1Char( ch );
     }
   }
 
@@ -617,7 +666,7 @@ bool parseComment( const char* &scursor, const char * const send,
           result += cmntPart;
           if ( commentNestingDepth > 1 ) {
             // don't add the outermost ')'...
-            result += QChar(')');
+            result += QLatin1Char( ')' );
           }
           maybeCmnt.clear();
         }
@@ -629,7 +678,7 @@ bool parseComment( const char* &scursor, const char * const send,
           // don't add to "result" yet, because we might find that we
           // are already outside the (broken) comment...
           maybeCmnt += cmntPart;
-          maybeCmnt += QChar('(');
+          maybeCmnt += QLatin1Char( '(' );
         }
         ++commentNestingDepth;
         break;
@@ -676,9 +725,9 @@ bool parsePhrase( const char* &scursor, const char * const send,
         return false;
       } else {
         if ( scursor != send && ( *scursor == ' ' || *scursor == '\t' ) ) {
-          result += ". ";
+          result += QLatin1String( ". " );
         } else {
-          result += '.';
+          result += QLatin1Char( '.' );
         }
         successfullyParsed = scursor;
       }
@@ -697,7 +746,7 @@ bool parsePhrase( const char* &scursor, const char * const send,
         case EncodedWord:
         case QuotedString:
           found = Phrase;
-          result += QChar(' '); // rfc822, 3.4.4
+          result += QLatin1Char(' '); // rfc822, 3.4.4
           break;
         default:
           assert( 0 );
@@ -711,7 +760,7 @@ bool parsePhrase( const char* &scursor, const char * const send,
         if ( found == None ) {
           return false;
         } else {
-          result += QChar(' '); // rfc822, 3.4.4
+          result += QLatin1Char(' '); // rfc822, 3.4.4
           result += tmp;
           return true;
         }
@@ -749,7 +798,7 @@ bool parsePhrase( const char* &scursor, const char * const send,
         case Atom:
         case QuotedString:
           if ( !lastWasEncodedWord ) {
-            result += QChar(' '); // rfc822, 3.4.4
+            result += QLatin1Char(' '); // rfc822, 3.4.4
           }
           found = Phrase;
           break;
@@ -778,7 +827,7 @@ bool parsePhrase( const char* &scursor, const char * const send,
         case EncodedWord:
         case QuotedString:
           found = Phrase;
-          result += QChar(' '); // rfc822, 3.4.4
+          result += QLatin1Char(' '); // rfc822, 3.4.4
           break;
         default:
           assert( 0 );
@@ -800,6 +849,7 @@ bool parsePhrase( const char* &scursor, const char * const send,
   return found != None;
 }
 
+// FIXME: This should probably by QByteArray &result instead?
 bool parseDotAtom( const char* &scursor, const char * const send,
                    QString &result, bool isCRLF )
 {
@@ -838,7 +888,7 @@ bool parseDotAtom( const char* &scursor, const char * const send,
       return true;
     }
 
-    result += QChar('.');
+    result += QLatin1Char('.');
     result += maybeAtom;
     successfullyParsed = scursor;
   }
@@ -912,7 +962,7 @@ bool parseDomain( const char* &scursor, const char * const send,
       // we hit openChar in parseGenericQuotedString.
       // include it in maybeDomainLiteral and keep on parsing:
       if ( *(scursor-1) == '[' ) {
-        maybeDomainLiteral += QChar('[');
+        maybeDomainLiteral += QLatin1Char('[');
         continue;
       }
       // OK, real end of domain-literal:
@@ -926,7 +976,7 @@ bool parseDomain( const char* &scursor, const char * const send,
       result = maybeDotAtom;
       // Domain may end with '.', if so preserve it'
       if ( scursor != send && *scursor == '.' ) {
-        result += QChar('.');
+        result += QLatin1Char('.');
         scursor++;
       }
       return true;
@@ -1014,7 +1064,7 @@ bool parseAddrSpec( const char* &scursor, const char * const send,
     char ch = *scursor++;
     switch ( ch ) {
     case '.': // dot
-      maybeLocalPart += QChar('.');
+      maybeLocalPart += QLatin1Char('.');
       break;
 
     case '@':
@@ -1110,6 +1160,16 @@ bool parseAngleAddr( const char* &scursor, const char * const send,
 
 }
 
+static QString stripQuotes( const QString &input )
+{
+  const QLatin1Char quotes( '"' );
+  if ( input.startsWith( quotes ) && input.endsWith( quotes ) ) {
+    QString stripped( input.mid( 1, input.size() - 2 ) );
+    return stripped;
+  }
+  else return input;
+}
+
 bool parseMailbox( const char* &scursor, const char * const send,
                    Mailbox &result, bool isCRLF )
 {
@@ -1133,7 +1193,7 @@ bool parseMailbox( const char* &scursor, const char * const send,
         return false;
       }
     }
-    result.setNameFrom7Bit( maybeDisplayName.toLatin1() );
+    result.setName( stripQuotes( maybeDisplayName ) );
     return true;
   }
   scursor = oldscursor;
@@ -1167,7 +1227,7 @@ bool parseMailbox( const char* &scursor, const char * const send,
     }
   }
 
-  result.setName( maybeDisplayName );
+  result.setName( stripQuotes( maybeDisplayName ) );
   result.setAddress( maybeAddrSpec );
   return true;
 }
@@ -1321,6 +1381,9 @@ static QString asterisk = QString::fromLatin1( "*0*", 1 );
 static QString asteriskZero = QString::fromLatin1( "*0*", 2 );
 //static QString asteriskZeroAsterisk = QString::fromLatin1( "*0*", 3 );
 
+// FIXME: Get rid of the very ugly "QStringOrQPair" thing. At this level, we are supposed to work
+//        on byte arrays, not strings! The result parameter should be a simple
+//        QPair<QByteArray,QByteArray>, which is the attribute name and the value.
 bool parseParameter( const char* &scursor, const char * const send,
                      QPair<QString,QStringOrQPair> &result, bool isCRLF )
 {
@@ -1342,6 +1405,7 @@ bool parseParameter( const char* &scursor, const char * const send,
   //
   // parse the parameter name:
   //
+  // FIXME: maybeAttribute should be a QByteArray
   QString maybeAttribute;
   if ( !parseToken( scursor, send, maybeAttribute, false /* no 8bit */ ) ) {
     return false;
@@ -1402,6 +1466,8 @@ bool parseParameter( const char* &scursor, const char * const send,
   return true;
 }
 
+// FIXME: Get rid of QStringOrQPair: Use a simply QMap<QByteArray, QByteArray> for "result"
+//        instead!
 bool parseRawParameterList( const char* &scursor, const char * const send,
                             QMap<QString,QStringOrQPair> &result,
                             bool isCRLF )
@@ -1470,7 +1536,7 @@ bool parseRawParameterList( const char* &scursor, const char * const send,
 static void decodeRFC2231Value( Codec* &rfc2231Codec,
                                 QTextCodec* &textcodec,
                                 bool isContinuation, QString &value,
-                                QPair<const char*,int> &source )
+                                QPair<const char*,int> &source, QByteArray& charset )
 {
   //
   // parse the raw value into (charset,language,text):
@@ -1499,7 +1565,7 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
       return;
     }
 
-    QByteArray charset( decBegin, decCursor - decBegin );
+    charset = QByteArray( decBegin, decCursor - decBegin );
 
     const char * oldDecCursor = ++decCursor;
     // find the second single quote (we ignore the language tag):
@@ -1526,7 +1592,7 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
     //
 
     bool matchOK = false;
-    textcodec = KGlobal::charsets()->codecForName( charset, matchOK );
+    textcodec = KGlobal::charsets()->codecForName( QLatin1String( charset ), matchOK );
     if ( !matchOK ) {
       textcodec = 0;
       KMIME_WARN_UNKNOWN( Charset, charset );
@@ -1572,10 +1638,12 @@ static void decodeRFC2231Value( Codec* &rfc2231Codec,
 //  - permutes rfc2231 continuations when the total number of parts
 //    exceeds 10 (other-sections then becomes *xy, ie. two digits)
 
-bool parseParameterList( const char* &scursor, const char * const send,
-                         QMap<QString,QString> &result, bool isCRLF )
+bool parseParameterListWithCharset( const char* &scursor,
+                                                const char * const send,
+                                                QMap<QString,QString> &result,
+                                                QByteArray& charset, bool isCRLF )
 {
-  // parse the list into raw attribute-value pairs:
+// parse the list into raw attribute-value pairs:
   QMap<QString,QStringOrQPair> rawParameterList;
   if (!parseRawParameterList( scursor, send, rawParameterList, isCRLF ) ) {
     return false;
@@ -1594,9 +1662,15 @@ bool parseParameterList( const char* &scursor, const char * const send,
   QTextCodec * textcodec = 0;
   QString attribute;
   QString value;
-  enum Modes {
+  enum Mode {
     NoMode = 0x0, Continued = 0x1, Encoded = 0x2
-  } mode;
+  };
+
+  enum EncodingMode {
+    NoEncoding,
+    RFC2047,
+    RFC2231
+  };
 
   QMap<QString,QStringOrQPair>::Iterator it, end = rawParameterList.end();
 
@@ -1613,24 +1687,37 @@ bool parseParameterList( const char* &scursor, const char * const send,
       // and extract the information from the new raw attribute:
       value.clear();
       attribute = it.key();
-      mode = NoMode;
-      // is the value encoded?
+      int mode = NoMode;
+      EncodingMode encodingMode = NoEncoding;
+
+      // is the value rfc2331-encoded?
       if ( attribute.endsWith( asterisk ) ) {
         attribute.truncate( attribute.length() - 1 );
-        mode = (Modes) ((int) mode | Encoded);
+        mode |= Encoded;
+        encodingMode = RFC2231;
+      }
+      // is the value rfc2047-encoded?
+      if( !(*it).qstring.isNull() && (*it).qstring.contains( QLatin1String( "=?" ) ) ) {
+        mode |= Encoded;
+        encodingMode = RFC2047;
       }
       // is the value continued?
       if ( attribute.endsWith( asteriskZero ) ) {
         attribute.truncate( attribute.length() - 2 );
-        mode = (Modes) ((int) mode | Continued);
+        mode |= Continued;
       }
       //
       // decode if necessary:
       //
       if ( mode & Encoded ) {
-        decodeRFC2231Value( rfc2231Codec, textcodec,
-                            false, /* isn't continuation */
-                            value, (*it).qpair );
+        if ( encodingMode == RFC2231 ) {
+          decodeRFC2231Value( rfc2231Codec, textcodec,
+                              false, /* isn't continuation */
+                              value, (*it).qpair, charset );
+        }
+        else if ( encodingMode == RFC2047 ) {
+          value += decodeRFC2047String( (*it).qstring.toLatin1(), charset );
+        }
       } else {
         // not encoded.
         if ( (*it).qpair.first ) {
@@ -1660,7 +1747,7 @@ bool parseParameterList( const char* &scursor, const char * const send,
         // encoded
         decodeRFC2231Value( rfc2231Codec, textcodec,
                             true, /* is continuation */
-                            value, (*it).qpair );
+                            value, (*it).qpair, charset );
       } else {
         // not encoded
         if ( (*it).qpair.first ) {
@@ -1680,6 +1767,14 @@ bool parseParameterList( const char* &scursor, const char * const send,
   return true;
 }
 
+
+bool parseParameterList( const char* &scursor, const char * const send,
+                         QMap<QString,QString> &result, bool isCRLF )
+{
+  QByteArray charset;
+  return parseParameterListWithCharset( scursor, send, result, charset, isCRLF );
+}
+
 static const char * const stdDayNames[] = {
   "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
@@ -1785,6 +1880,15 @@ static bool parseAlphaNumericTimeZone( const char* &scursor,
                                        long int &secsEastOfGMT,
                                        bool &timeZoneKnown )
 {
+  // allow the timezone to be wrapped in quotes; bug 260761
+  if ( *scursor == '"' ) {
+    scursor++;
+
+    if ( scursor == send ) {
+      return false;
+    }
+  }
+
   QPair<const char*,int> maybeTimeZone( 0, 0 );
   if ( !parseToken( scursor, send, maybeTimeZone, false /*no 8bit*/ ) ) {
     return false;
@@ -1795,6 +1899,11 @@ static bool parseAlphaNumericTimeZone( const char* &scursor,
       scursor += maybeTimeZone.second;
       secsEastOfGMT = timeZones[i].secsEastOfGMT;
       timeZoneKnown = true;
+
+      if ( *scursor == '"' ) {
+        scursor++;
+      }
+
       return true;
     }
   }
@@ -1899,7 +2008,8 @@ bool parseTime( const char* &scursor, const char * send,
   }
 
   eatCFWS( scursor, send, isCRLF );
-  if ( scursor == send ) {
+  // there might be no timezone but a year following
+  if ( ( scursor == send ) || isdigit( *scursor ) ) {
     timeZoneKnown = false;
     secsEastOfGMT = 0;
     return true; // allow missing timezone
@@ -1966,6 +2076,15 @@ bool parseDateTime( const char* &scursor, const char * const send,
     }
   }
 
+  int maybeMonth = -1;
+  bool asctimeFormat = false;
+
+  // ANSI-C asctime() format is: Wed Jun 30 21:49:08 1993
+  if ( !isdigit( *scursor ) && parseMonthName( scursor, send, maybeMonth ) ) {
+    asctimeFormat = true;
+    eatCFWS( scursor, send, isCRLF );
+  }
+
   //
   // 1*2DIGIT representing "day" (of month):
   //
@@ -1979,11 +2098,15 @@ bool parseDateTime( const char* &scursor, const char * const send,
     return false;
   }
 
+  // ignore ","; bug 54098
+  if ( *scursor == ',' ) {
+    scursor++;
+  }
+
   //
   // month-name:
   //
-  int maybeMonth = 0;
-  if ( !parseMonthName( scursor, send, maybeMonth ) ) {
+  if ( !asctimeFormat && !parseMonthName( scursor, send, maybeMonth ) ) {
     return false;
   }
   if ( scursor == send ) {
@@ -1997,31 +2120,26 @@ bool parseDateTime( const char* &scursor, const char * const send,
     return false;
   }
 
+  // check for "year HH:MM:SS" or only "HH:MM:SS" (or "H:MM:SS")
+  bool timeAfterYear = true;
+  if ( ( send - scursor > 3 ) && ( ( scursor[1] == ':' ) || ( scursor[2] == ':' ) ) ) {
+    timeAfterYear = false;  // first read time, then year
+  }
+
   //
   // 2*DIGIT representing "year":
   //
-  int maybeYear;
-  if ( !parseDigits( scursor, send, maybeYear ) ) {
+  int maybeYear = 0;
+
+  if ( timeAfterYear && !parseDigits( scursor, send, maybeYear ) ) {
     return false;
   }
-  // RFC 2822 4.3 processing:
-  if ( maybeYear < 50 ) {
-    maybeYear += 2000;
-  } else if ( maybeYear < 1000 ) {
-    maybeYear += 1900;
-  }
-  // else keep as is
-  if ( maybeYear < 1900 ) {
-    return false; // rfc2822, 3.3
-  }
 
   eatCFWS( scursor, send, isCRLF );
   if ( scursor == send ) {
     return false;
   }
 
-  maybeDateTime.setDate( QDate( maybeYear, maybeMonth, maybeDay ) );
-
   //
   // time
   //
@@ -2035,7 +2153,32 @@ bool parseDateTime( const char* &scursor, const char * const send,
     return false;
   }
 
+  // in asctime() the year follows the time
+  if ( !timeAfterYear ) {
+    eatCFWS( scursor, send, isCRLF );
+    if ( scursor == send ) {
+      return false;
+    }
+
+    if ( !parseDigits( scursor, send, maybeYear ) ) {
+      return false;
+    }
+  }
+
+  // RFC 2822 4.3 processing:
+  if ( maybeYear < 50 ) {
+    maybeYear += 2000;
+  } else if ( maybeYear < 1000 ) {
+    maybeYear += 1900;
+  }
+  // else keep as is
+  if ( maybeYear < 1900 ) {
+    return false; // rfc2822, 3.3
+  }
+
+  maybeDateTime.setDate( QDate( maybeYear, maybeMonth, maybeDay ) );
   maybeDateTime.setTime( QTime( maybeHour, maybeMinute, maybeSecond ) );
+
   if ( !maybeDateTime.isValid() )
     return false;
 
@@ -2048,11 +2191,10 @@ bool parseDateTime( const char* &scursor, const char * const send,
 Headers::Base *extractFirstHeader( QByteArray &head )
 {
   int endOfFieldBody = 0;
-  int len = head.length() - 1;
   bool folded = false;
   Headers::Base *header = 0;
 
-  int startOfFieldBody = head.indexOf( ":" );
+  int startOfFieldBody = head.indexOf( ':' );
   const int endOfFieldHeader = startOfFieldBody;
 
   if ( startOfFieldBody > -1 ) {    //there is another header
@@ -2060,44 +2202,17 @@ Headers::Base *extractFirstHeader( QByteArray &head )
     if ( head[startOfFieldBody] == ' ' ) { // skip the space after the ':', if there
       startOfFieldBody++;
     }
-    endOfFieldBody = startOfFieldBody;
-
-    // If the first line contains nothing, but the next line starts with a space
-    // or a tab, that means a stupid mail client has made the first header field line
-    // entirely empty, and has folded the rest to the next line(s).
-    if ( head[endOfFieldBody] == '\n' && endOfFieldBody + 1 < len &&
-         ( head[endOfFieldBody + 1] == ' ' ||
-           head[endOfFieldBody + 1] == '\t' ) ) {
-
-      // Skip \n and first whitespace
-      startOfFieldBody += 2;
-      endOfFieldBody += 2;
-    }
-    
-    if ( head[endOfFieldBody] != '\n' ) {  // check if the header is not empty
-      while ( 1 ) {
-        endOfFieldBody = head.indexOf( '\n', endOfFieldBody + 1 );
-        if ( endOfFieldBody == -1 || endOfFieldBody == len ||
-              ( head[endOfFieldBody+1] != ' ' &&
-                head[endOfFieldBody+1] != '\t' ) ) {
-          //break if we reach the end of the string, honor folded lines
-          break;
-        } else {
-          folded = true;
-        }
-      }
-    }
-
-    if ( endOfFieldBody < 0 ) {
-      endOfFieldBody = len + 1; //take the rest of the string
-    }
+    endOfFieldBody = findHeaderLineEnd( head, startOfFieldBody, &folded );
 
     QByteArray rawType = head.left( endOfFieldHeader );
     QByteArray rawFieldBody = head.mid( startOfFieldBody, endOfFieldBody - startOfFieldBody );
     if ( folded ) {
       rawFieldBody = unfoldHeader( rawFieldBody );
     }
-    header = HeaderFactory::self()->createHeader( rawType );
+    // We might get an invalid mail without a field name, don't crash on that.
+    if ( !rawType.isEmpty() ) {
+      header = HeaderFactory::self()->createHeader( rawType );
+    }
     if( !header ) {
       //kWarning() << "Returning Generic header of type" << rawType;
       header = new Headers::Generic( rawType );
@@ -2112,6 +2227,26 @@ Headers::Base *extractFirstHeader( QByteArray &head )
   return header;
 }
 
+void extractHeaderAndBody( const QByteArray &content, QByteArray &header, QByteArray &body )
+{
+  header.clear();
+  body.clear();
+
+  // empty header
+  if ( content.startsWith( '\n' ) ) {
+    body = content.right( content.length() - 1 );
+    return;
+  }
+
+  int pos = content.indexOf( "\n\n", 0 );
+  if ( pos > -1 ) {
+    header = content.left( ++pos );  //header *must* end with "\n" !!
+    body = content.mid( pos + 1, content.length() - pos - 1 );
+  } else {
+    header = content;
+  }
+}
+
 Headers::Base::List parseHeaders( const QByteArray &head )
 {
   Headers::Base::List ret;
