@@ -41,6 +41,7 @@
 #include "kmime_headers_p.h"
 
 #include "kmime_util.h"
+#include "kmime_util_p.h"
 #include "kmime_content.h"
 #include "kmime_codecs.h"
 #include "kmime_header_parsing.h"
@@ -205,17 +206,17 @@ const char *Base::type() const
 
 bool Base::is( const char *t ) const
 {
-  return strcasecmp( t, type() ) == 0;
+  return qstricmp( t, type() ) == 0;
 }
 
 bool Base::isMimeHeader() const
 {
-  return strncasecmp( type(), "Content-", 8 ) == 0;
+  return qstrnicmp( type(), "Content-", 8 ) == 0;
 }
 
 bool Base::isXHeader() const
 {
-  return strncmp( type(), "X-", 2 ) == 0;
+  return qstrncmp( type(), "X-", 2 ) == 0;
 }
 
 QByteArray Base::typeIntro() const
@@ -409,7 +410,7 @@ void MailboxList::fromUnicodeString( const QString &s, const QByteArray &b )
 {
   Q_D(MailboxList);
   d->encCS = cachedCharset( b );
-  from7BitString( encodeRFC2047String( s, b, false ) );
+  from7BitString( encodeRFC2047Sentence( s, b ) );
 }
 
 QString MailboxList::asUnicodeString() const
@@ -561,7 +562,7 @@ void AddressList::fromUnicodeString( const QString &s, const QByteArray &b )
 {
   Q_D(AddressList);
   d->encCS = cachedCharset( b );
-  from7BitString( encodeRFC2047String( s, b, false ) );
+  from7BitString( encodeRFC2047Sentence( s, b ) );
 }
 
 QString AddressList::asUnicodeString() const
@@ -982,11 +983,18 @@ QByteArray Ident::as7BitString( bool withHeaderType ) const
     rv = typeIntro();
   }
   foreach ( const Types::AddrSpec &addr, d->msgIdList ) {
-    rv += '<';
-    rv += addr.asString().toLatin1(); // FIXME: change parsing to use QByteArrays
-    rv += "> ";
+    if ( !addr.isEmpty() ) {
+      const QString asString = addr.asString();
+      rv += '<';
+      if ( !asString.isEmpty() ) {
+        rv += asString.toLatin1(); // FIXME: change parsing to use QByteArrays
+      }
+      rv += "> ";
+    }
+  }
+  if ( !rv.isEmpty() ) {
+    rv.resize( rv.length() - 1 );
   }
-  rv.resize( rv.length() - 1 );
   return rv;
 }
 
@@ -994,6 +1002,7 @@ void Ident::clear()
 {
   Q_D(Ident);
   d->msgIdList.clear();
+  d->cachedIdentifier.clear();
 }
 
 bool Ident::isEmpty() const
@@ -1012,6 +1021,7 @@ bool Ident::parse( const char* &scursor, const char * const send, bool isCRLF )
   // msg-id   := angle-addr
 
   d->msgIdList.clear();
+  d->cachedIdentifier.clear();
 
   while ( scursor != send ) {
     eatCFWS( scursor, send, isCRLF );
@@ -1048,7 +1058,12 @@ QList<QByteArray> Ident::identifiers() const
 {
   QList<QByteArray> rv;
   foreach ( const Types::AddrSpec &addr, d_func()->msgIdList ) {
-    rv.append( addr.asString().toLatin1() ); // FIXME change parsing to create QByteArrays
+    if ( !addr.isEmpty() ) {
+      const QString asString = addr.asString();
+      if ( !asString.isEmpty() ) {
+        rv.append( asString.toLatin1() ); // FIXME: change parsing to use QByteArrays
+      }
+    }
   }
   return rv;
 }
@@ -1086,13 +1101,25 @@ QByteArray SingleIdent::identifier() const
   if ( d_func()->msgIdList.isEmpty() ) {
     return QByteArray();
   }
-  return identifiers().first();
+
+  if ( d_func()->cachedIdentifier.isEmpty() ) {
+    const Types::AddrSpec &addr = d_func()->msgIdList.first();
+    if ( !addr.isEmpty() ) {
+      const QString asString = addr.asString();
+      if ( !asString.isEmpty() ) {
+        d_func()->cachedIdentifier = asString.toLatin1();// FIXME: change parsing to use QByteArrays
+      }
+    }
+  }
+
+  return d_func()->cachedIdentifier;
 }
 
 void SingleIdent::setIdentifier( const QByteArray &id )
 {
   Q_D(SingleIdent);
   d->msgIdList.clear();
+  d->cachedIdentifier.clear();
   appendIdentifier( id );
 }
 
@@ -1271,12 +1298,12 @@ void Generic::setType( const char *type )
 //-----<MessageID>-----------------------------
 
 //@cond PRIVATE
-kmime_mk_trivial_ctor_with_name( MessageID, Generics::SingleIdent, Message-Id )
+kmime_mk_trivial_ctor_with_name( MessageID, Generics::SingleIdent, Message-ID )
 //@endcond
 
 void MessageID::generate( const QByteArray &fqdn )
 {
-  setIdentifier( uniqueString() + '@' + fqdn + '>' );
+  setIdentifier( '<' + uniqueString() + '@' + fqdn + '>' );
 }
 
 //-----</MessageID>----------------------------
@@ -1692,7 +1719,6 @@ void ContentType::clear()
   Q_D(ContentType);
   d->category = CCsingle;
   d->mimeType.clear();
-  d->mimeSubType.clear();
   Parametrized::clear();
 }
 
@@ -1718,35 +1744,33 @@ QByteArray ContentType::as7BitString( bool withHeaderType ) const
 QByteArray ContentType::mimeType() const
 {
   Q_D(const ContentType);
-  QByteArray mt;
-  mt.reserve( d->mimeType.size() + d->mimeSubType.size() + 1 );
-  mt.append( d->mimeType );
-  mt.append( '/' );
-  mt.append( d->mimeSubType );
-  return mt;
+  return d->mimeType;
 }
 
 QByteArray ContentType::mediaType() const
 {
-  return d_func()->mimeType;
+  Q_D(const ContentType);
+  const int pos = d->mimeType.indexOf( '/' );
+  if ( pos < 0 )
+    return d->mimeType;
+  else
+    return d->mimeType.left( pos );
 }
 
 QByteArray ContentType::subType() const
 {
-  return d_func()->mimeSubType;
+  Q_D(const ContentType);
+  const int pos = d->mimeType.indexOf( '/' );
+  if ( pos < 0 )
+    return QByteArray();
+  else
+    return d->mimeType.mid( pos + 1);
 }
 
 void ContentType::setMimeType( const QByteArray &mimeType )
 {
   Q_D(ContentType);
-  int pos = mimeType.indexOf( '/' );
-  if ( pos < 0 ) {
-    d->mimeType = mimeType;
-    d->mimeSubType.clear();
-  } else {
-    d->mimeType = mimeType.left( pos );
-    d->mimeSubType = mimeType.mid( pos + 1 );
-  }
+  d->mimeType = mimeType;
   Parametrized::clear();
 
   if ( isMultipart() ) {
@@ -1758,49 +1782,54 @@ void ContentType::setMimeType( const QByteArray &mimeType )
 
 bool ContentType::isMediatype( const char *mediatype ) const
 {
-  return strncasecmp( mediaType().constData(), mediatype, strlen( mediatype ) ) == 0;
+  Q_D(const ContentType);
+  const int len = strlen( mediatype );
+  return qstrnicmp( d->mimeType.constData(), mediatype, len ) == 0 && (d->mimeType.at(len) == '/' || d->mimeType.size() == len);
 }
 
 bool ContentType::isSubtype( const char *subtype ) const
 {
-  return strncasecmp( subType().constData(), subtype, strlen( subtype ) ) == 0;
+  Q_D(const ContentType);
+  const int pos = d->mimeType.indexOf( '/' );
+  if ( pos < 0 )
+    return false;
+  const int len = strlen( subtype );
+  return qstrnicmp( d->mimeType.constData() + pos + 1, subtype, len ) == 0 && d->mimeType.size() == pos + len + 1;
 }
 
 bool ContentType::isText() const
 {
-  return ( strncasecmp( mediaType().constData(), "text", 4 ) == 0
-          || isEmpty() );
+  return ( isMediatype( "text" ) || isEmpty() );
 }
 
 bool ContentType::isPlainText() const
 {
-  return ( strcasecmp( mimeType().constData(), "text/plain" ) == 0
-          || isEmpty() );
+  return ( qstricmp( d_func()->mimeType.constData(), "text/plain" ) == 0 || isEmpty() );
 }
 
 bool ContentType::isHTMLText() const
 {
-  return strcasecmp( mimeType().constData(), "text/html" ) == 0;
+  return qstricmp( d_func()->mimeType.constData(), "text/html" ) == 0;
 }
 
 bool ContentType::isImage() const
 {
-  return strncasecmp( mediaType().constData(), "image", 5 ) == 0;
+  return isMediatype( "image" );
 }
 
 bool ContentType::isMultipart() const
 {
-  return strncasecmp( mediaType().constData(), "multipart", 9 ) == 0;
+  return isMediatype( "multipart" );
 }
 
 bool ContentType::isPartial() const
 {
-  return strcasecmp( mimeType().constData(), "message/partial" ) == 0;
+  return qstricmp( d_func()->mimeType.constData(), "message/partial" ) == 0;
 }
 
 QByteArray ContentType::charset() const
 {
-  QByteArray ret = parameter( "charset" ).toLatin1();
+  QByteArray ret = parameter( QLatin1String( "charset" ) ).toLatin1();
   if ( ret.isEmpty() || forceDefaultCharset() ) {
     //return the default-charset if necessary
     ret = defaultCharset();
@@ -1810,44 +1839,44 @@ QByteArray ContentType::charset() const
 
 void ContentType::setCharset( const QByteArray &s )
 {
-  setParameter( "charset", QString::fromLatin1( s ) );
+  setParameter( QLatin1String( "charset" ), QString::fromLatin1( s ) );
 }
 
 QByteArray ContentType::boundary() const
 {
-  return parameter( "boundary" ).toLatin1();
+  return parameter( QLatin1String( "boundary" ) ).toLatin1();
 }
 
 void ContentType::setBoundary( const QByteArray &s )
 {
-  setParameter( "boundary", QString::fromLatin1( s ) );
+  setParameter( QLatin1String( "boundary" ), QString::fromLatin1( s ) );
 }
 
 QString ContentType::name() const
 {
-  return parameter( "name" );
+  return parameter( QLatin1String( "name" ) );
 }
 
 void ContentType::setName( const QString &s, const QByteArray &cs )
 {
   Q_D(ContentType);
   d->encCS = cs;
-  setParameter( "name", s );
+  setParameter( QLatin1String( "name" ), s );
 }
 
 QByteArray ContentType::id() const
 {
-  return parameter( "id" ).toLatin1();
+  return parameter( QLatin1String( "id" ) ).toLatin1();
 }
 
 void ContentType::setId( const QByteArray &s )
 {
-  setParameter( "id", s );
+  setParameter( QLatin1String( "id" ), QString::fromLatin1( s ) );
 }
 
 int ContentType::partialNumber() const
 {
-  QByteArray p = parameter( "number" ).toLatin1();
+  QByteArray p = parameter( QLatin1String( "number" ) ).toLatin1();
   if ( !p.isEmpty() ) {
     return p.toInt();
   } else {
@@ -1857,7 +1886,7 @@ int ContentType::partialNumber() const
 
 int ContentType::partialCount() const
 {
-  QByteArray p = parameter( "total" ).toLatin1();
+  QByteArray p = parameter( QLatin1String( "total" ) ).toLatin1();
   if ( !p.isEmpty() ) {
     return p.toInt();
   } else {
@@ -1878,8 +1907,8 @@ void ContentType::setCategory( contentCategory c )
 
 void ContentType::setPartialParams( int total, int number )
 {
-  setParameter( "number", QString::number( number ) );
-  setParameter( "total", QString::number( total ) );
+  setParameter( QLatin1String( "number" ), QString::number( number ) );
+  setParameter( QLatin1String( "total" ), QString::number( total ) );
 }
 
 bool ContentType::parse( const char* &scursor, const char * const send,
@@ -1899,7 +1928,6 @@ bool ContentType::parse( const char* &scursor, const char * const send,
   if ( !parseToken( scursor, send, maybeMimeType, false /* no 8Bit */ ) ) {
     return false;
   }
-  d->mimeType = QByteArray( maybeMimeType.first, maybeMimeType.second ).toLower();
 
   // subtype
   eatCFWS( scursor, send, isCRLF );
@@ -1916,7 +1944,10 @@ bool ContentType::parse( const char* &scursor, const char * const send,
   if ( !parseToken( scursor, send, maybeSubType, false /* no 8bit */ ) ) {
     return false;
   }
-  d->mimeSubType = QByteArray( maybeSubType.first, maybeSubType.second ).toLower();
+
+  d->mimeType.reserve( maybeMimeType.second + maybeSubType.second + 1 );
+  d->mimeType = QByteArray( maybeMimeType.first, maybeMimeType.second ).toLower()
+                + '/' + QByteArray( maybeSubType.first, maybeSubType.second ).toLower();
 
   // parameter list
   eatCFWS( scursor, send, isCRLF );
@@ -1961,6 +1992,7 @@ bool ContentID::parse( const char* &scursor, const char *const send, bool isCRLF
   {
     scursor = origscursor;
     d->msgIdList.clear();
+    d->cachedIdentifier.clear();
 
     while ( scursor != send )
     {
@@ -1992,7 +2024,7 @@ bool ContentID::parse( const char* &scursor, const char *const send, bool isCRLF
       }
 
       // Save chars untill '>''
-      QString result = "";
+      QString result;
       if( !parseAtom(scursor, send, result, false) ) {
         return false;
       }
@@ -2104,7 +2136,7 @@ bool ContentTransferEncoding::parse( const char *& scursor,
 
   // TODO: error handling in case of an unknown encoding?
   for ( int i = 0; encTable[i].s != 0; ++i ) {
-    if ( strcasecmp( token().constData(), encTable[i].s ) == 0 ) {
+    if ( qstricmp( token().constData(), encTable[i].s ) == 0 ) {
       d->cte = ( contentEncoding )encTable[i].e;
       break;
     }
@@ -2173,12 +2205,12 @@ void ContentDisposition::setDisposition( contentDisposition disp )
 
 QString KMime::Headers::ContentDisposition::filename() const
 {
-  return parameter( "filename" );
+  return parameter( QLatin1String( "filename" ) );
 }
 
 void ContentDisposition::setFilename( const QString &filename )
 {
-  setParameter( "filename", filename );
+  setParameter( QLatin1String( "filename" ), filename );
 }
 
 bool ContentDisposition::parse( const char *& scursor, const char * const send,
@@ -2233,6 +2265,12 @@ bool Subject::isReply() const
   return asUnicodeString().indexOf( QLatin1String( "Re:" ), 0, Qt::CaseInsensitive ) == 0;
 }
 
+Base* createHeader( const QByteArray& type )
+{
+  return HeaderFactory::self()->createHeader( type );
+}
+
+
 //@cond PRIVATE
 kmime_mk_trivial_ctor_with_name( ContentDescription,
                                  Generics::Unstructured, Content-Description )
