@@ -23,18 +23,22 @@
   static methods for email address validation.
 
   @author Matt Douhan \<matt@fruitsalad.org\>
- */
-
+*/
 #include "email.h"
 
-#include <kdebug.h>
-#include <klocale.h>
-#include <kurl.h>
-#include <kmime_util.h>
+#include <kmime/kmime_util.h>
+
+#include <KDebug>
+#include <KLocale>
+#include <KUrl>
 
 #include <QtCore/QRegExp>
 #include <QtCore/QByteArray>
 
+#include <kglobal.h>
+
+static const KCatalogLoader loader( "libkpimutils" );
+
 using namespace KPIMUtils;
 
 //-----------------------------------------------------------------------------
@@ -295,6 +299,8 @@ EmailParseResult KPIMUtils::splitAddress( const QString &address,
                                           QString &comment )
 {
   QByteArray d, a, c;
+  // FIXME: toUtf8() is probably not safe here, what if the second byte of a multi-byte character
+  //        has the same code as one of the ASCII characters that splitAddress uses as delimiters?
   EmailParseResult result = splitAddress( address.toUtf8(), d, a, c );
 
   if ( result == AddressOk ) {
@@ -531,7 +537,7 @@ QString KPIMUtils::emailParseResultToString( EmailParseResult errorCode )
                  "change your address." );
   case TooFewAts :
     return i18n( "The email address you entered is not valid because it "
-                 "does not contain a @."
+                 "does not contain a @. "
                  "You will not create valid messages if you do not "
                  "change your address." );
   case AddressEmpty :
@@ -590,6 +596,14 @@ bool KPIMUtils::isValidSimpleAddress( const QString &aStr )
   int atChar = aStr.lastIndexOf( '@' );
   QString domainPart = aStr.mid( atChar + 1 );
   QString localPart = aStr.left( atChar );
+
+  // Both of these parts must be non empty
+  // after all we cannot have emails like:
+  // @kde.org, or  foo@
+  if ( localPart.isEmpty() || domainPart.isEmpty() ) {
+    return false;
+  }
+
   bool tooManyAtsFlag = false;
   bool inQuotedString = false;
   int atCount = localPart.count( '@' );
@@ -620,7 +634,7 @@ bool KPIMUtils::isValidSimpleAddress( const QString &aStr )
   if ( domainPart[ 0 ] == '[' || domainPart[ domainPart.length()-1 ] == ']' ) {
     addrRx += "\\[[0-9]{,3}(\\.[0-9]{,3}){3}\\]";
   } else {
-    addrRx += "[\\w-]+(\\.[\\w-]+)*";
+    addrRx += "[\\w-#]+(\\.[\\w-#]+)*";
   }
   QRegExp rx( addrRx );
   return  rx.exactMatch( aStr ) && !tooManyAtsFlag;
@@ -761,7 +775,7 @@ bool KPIMUtils::extractEmailAddressAndName( const QString &aStr,
       } else if ( bInQuotesOutsideOfEmail ) {
         if ( cQuotes == c ) {
           bInQuotesOutsideOfEmail = false;
-        } else {
+        } else if ( c != '\\' ) {
           name.prepend( c );
         }
       } else {
@@ -834,7 +848,7 @@ bool KPIMUtils::extractEmailAddressAndName( const QString &aStr,
       } else if ( bInQuotesOutsideOfEmail ) {
         if ( cQuotes == c ) {
           bInQuotesOutsideOfEmail = false;
-        } else {
+        } else if ( c != '\\' ) {
           name.append( c );
         }
       } else {
@@ -1067,15 +1081,14 @@ QString KPIMUtils::quoteNameIfNecessary( const QString &str )
 KUrl KPIMUtils::encodeMailtoUrl( const QString &mailbox )
 {
   const QByteArray encodedPath = KMime::encodeRFC2047String( mailbox, "utf-8" );
-  qDebug() << encodedPath.data();
   KUrl mailtoUrl;
   mailtoUrl.setProtocol( "mailto" );
   mailtoUrl.setPath( encodedPath );
   return mailtoUrl;
 }
 
-QString KPIMUtils::decodeMailtoUrl( const KUrl& mailtoUrl )
+QString KPIMUtils::decodeMailtoUrl( const KUrl &mailtoUrl )
 {
   Q_ASSERT( mailtoUrl.protocol().toLower() == "mailto" );
-  return KMime::decodeRFC2047String( mailtoUrl.path().toLatin1() );
+  return KMime::decodeRFC2047String( mailtoUrl.path().toUtf8() );
 }
